@@ -39,5 +39,7 @@ setup(name='elisa-plugin-amazon',
     packages=packages,
     package_dir=package_dir,
     entry_points="""\
+    [elisa.core.components.resource_provider]
+    AmazonResourceProvider = elisa.plugins.amazon.resource_provider:AmazonResourceProvider
     """,
     cmdclass=cmdclass)
