@@ -129,15 +129,18 @@ def timeout_runner(cookie, connection_string):
 class MasterMixin(object):
 
     def tearDown(self):
+        if hasattr(self, "current"):
+            self.master._slave_runner = self.current
+            del self.current
+
         return self.master.stop()
 
-    def _resetRunner(self, runner):
-        self.master._slave_runner = runner
-
     def setRunner(self, runner):
-        current = self.master._slave_runner
+        msg = "setRunner may only called once per test"
+        assert hasattr(self, "current") is False, msg
+
+        self.current = self.master._slave_runner
         self.master._slave_runner = '%s.%s' % (__name__, runner)
-        self.addCleanup(self._resetRunner, current)
 
     def testStartSlavesFail(self):
         """
