@@ -30,4 +30,8 @@ setup(name='elisa-plugin-avahi',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    AvahiResourceProvider = elisa.plugins.avahi.avahi_resource_provider:AvahiResourceProvider
+    """)
