@@ -18,4 +18,12 @@ setup(name='elisa-plugin-coherence',
       packages=packages,
       package_dir=package_dir,
       package_data={'': ['*.png', '*.mo', '*.po']},
-      cmdclass=cmdclass)
+      cmdclass=cmdclass,
+      entry_points="""
+      [elisa.core.components.resource_provider]
+      CoherenceResource = coherence.coherence_resource:CoherenceResource
+      UpnpResource = elisa.plugins.coherence.upnp_resource:UpnpResource
+
+      [elisa.core.components.service_provider]
+      CoherenceService = elisa.plugins.coherence.coherence_service:CoherenceService
+      """)
