@@ -233,7 +233,8 @@ class UPnPMediaServer(pb.Referenceable, service_provider.ServiceProvider):
     def clean(self):
         if self._locations_observable:
             self._locations_observable.remove_observer(self._locations_observer)
-        service_provider.ServiceProvider.clean(self)
+
+        return super(UPnPMediaServer, self).clean()
 
     def add_media_location(self, location):
         # TODO: add support in the http server to serve daap and other
