@@ -21,7 +21,8 @@ parse the result from the typefinder and wrap it into the database
 """
 
 from elisa.plugins.database.models import Artist, MusicAlbum, MusicTrack, \
-    File, Tag, Image, PhotoAlbum, Video
+    File, Tag, Image, PhotoAlbum, Video, \
+    PICTURES_SECTION, MUSIC_SECTION, VIDEO_SECTION
 
 from elisa.plugins.gstreamer.amp_master import GstMetadataAmpClient
 
@@ -91,6 +92,7 @@ class DatabaseParser(Component):
         dfr.addCallback(self.load_processors)
         dfr.addCallback(create_client)
         dfr.addCallback(set_process_delay)
+        dfr.addCallback(self._reset_autocommit_delay)
         return dfr
 
     def load_processors(result, self):
@@ -130,7 +132,14 @@ class DatabaseParser(Component):
             dfr.addCallback(clean_metadata)
             return dfr
 
-        return clean_metadata(None)
+        dfr = clean_metadata(None)
+
+        def parent_clean(result):
+            return super(DatabaseParser, self).clean()
+
+        dfr.addCallback(parent_clean)
+
+        return dfr
 
     def mark_deleted(self, source_path):
         self.debug("Mark deleted %s", source_path)
@@ -142,24 +151,52 @@ class DatabaseParser(Component):
         dfr = self.store.find(File, File.source == source_path)
         dfr.addCallback(delete)
         return dfr
+
+    def delete_files(self, source_path, marked_only=True):
+        """
+        Delete file entries from the database that have the source set to the
+        given C{source_path}. If C{marked_only} is set, only the file entries
+        with the delete-marker are removed.
+
+        @param source_path: the source path of the files
+        @type source_path:  C{unicode}
+
+        @keyword marked_only: delete only the file entries that are marked as
+                              deleted
+        @type marked_only:    C{bool}
+
+        @rtype: L{elisa.core.utils.cancellable_defer.CancellableDeferred}
+        @return: triggered whenever the deletion is done
+        """
+
+        def delete(results):
+            return results.remove() 
+
+        if marked_only:
+           dfr = self.store.find(File, File.deleted == 1, \
+                File.source == source_path)
+        else:
+            dfr = self.store.find(File, File.source == source_path)
+
+        dfr.addCallback(delete)
+        return dfr
         
     def query_model(self, model, stat):
-        path = model.uri.path
-        self.debug("query for %s" % path)
+        self.debug("query for %s" % model.uri.path)
 
-        dfr = self.get_or_create(File, 'path', path)
+        dfr = self.get_or_create(File, 'path', model.uri.path)
         dfr.addCallback(self.process, model, stat)
 
         return dfr
 
-    def gst_process(self, file, model):
+    def gst_process(self, file, model, section):
 
         request_dict = dict(global_request_dict)
         request_dict['uri'] = model.uri
 
         dfr = self.metadata.get_metadata(request_dict)
         dfr.addBoth(self.update_modification_time, file)
-        dfr.addCallback(self.parse_tags, file)
+        dfr.addCallback(self.parse_tags, file, section)
         return dfr
 
     def process(self, file, model, source):
@@ -179,14 +216,15 @@ class DatabaseParser(Component):
         #dfr = self.pre_process(file, model)
         #dfr.addErrback(pre_process_failed, file, model)
         #dfr.addCallback(self.post_process, file, model)
-        dfr = self.gst_process(file, model)
+        dfr = self.gst_process(file, model, source.section)
         dfr.addCallback(self._commit)
-        dfr.addCallback(self._delay)
+        dfr.addBoth(self._delay)
         return dfr
 
-    def _delay(self, result):
+    def _delay(self, result_or_failure):
         dfr = defer.Deferred()
-        reactor.callLater(self.between_process_delay, dfr.callback, result)
+        reactor.callLater(self.between_process_delay, dfr.callback,
+                result_or_failure)
         return dfr
 
     def _commit(self, result):
@@ -395,8 +433,17 @@ class DatabaseParser(Component):
     # the case of videos
     def parse_into_video(self, video, metadata):
         self.debug("parsing video %s" % video.file_path)
-        pass
-
+        
+        # We'll use the filename as video name until we can get a proper name
+        video.name = os.path.basename(video.file_path)
+        
+        # We set the creation time as the date of the file
+        stamp = os.stat(video.file_path).st_mtime
+        video.creation_time = datetime.datetime.fromtimestamp(stamp)
+        
+        if 'duration' in metadata:
+            video.duration = int(metadata['duration'])
+        
     # the case of images
     def parse_into_image(self, image, metadata):
         self.debug("parsing image %s" % image.file_path)
@@ -434,16 +481,19 @@ class DatabaseParser(Component):
             stamp = os.stat(image.file_path).st_mtime
             image.shot_time = datetime.datetime.fromtimestamp(stamp)
 
-        album_name = os.path.basename(os.path.split(image.file_path)[0])
+        if image.section & PICTURES_SECTION:
+            album_name = os.path.basename(os.path.split(image.file_path)[0])
 
-        if album_name == '':
-            image.album_name = None
+            if album_name == '':
+                image.album_name = None
+            else:
+                image.album_name = album_name
+                # make sure the album exists
+                return self.get_or_create(PhotoAlbum, 'name', album_name)
         else:
-            image.album_name = album_name
-            # make sure the album exists
-            return self.get_or_create(PhotoAlbum, 'name', album_name)
+            image.album_name = None
 
-    def parse_tags(self, metadata, file):
+    def parse_tags(self, metadata, file, section=''):
         
         self.debug("got tags: %s" % metadata)
 
@@ -465,8 +515,22 @@ class DatabaseParser(Component):
             dfr.addCallback(self.parse_into_track, metadata)
 
         elif file_type == 'image':
+            def update_section(image, section):
+                section_masks = { 'pictures': PICTURES_SECTION,
+                                 'music':    MUSIC_SECTION,
+                                 'video':    VIDEO_SECTION,
+                                }
+                new_section = section_masks.get(section, 0)
+                if image.section == None:
+                    image.section = new_section
+                else:
+                    image.section |= new_section
+
+                return image
+
             tag = u'image'
             dfr = self.get_or_create(Image, 'file_path', file.path)
+            dfr.addCallback(update_section, section)
             dfr.addCallback(self.parse_into_image, metadata)
 
         if not dfr:
