@@ -20,34 +20,56 @@ import dbus
 import dbus.service
 import dbus.glib
 import gobject
+
 from twisted.internet import defer, task
 
 from elisa.core import common
+from elisa.core.media_uri import MediaUri
+from elisa.core.log import getFailureMessage
 from elisa.core.components.service_provider import ServiceProvider
-from elisa.core.application import ComponentsLoadedMessage
+from elisa.core.application import ComponentsLoadedMessage, PICTURES_CACHE
+
+from elisa.plugins.poblesec.music_library import album_cover_retriever
+
 from elisa.plugins.database.models import Artist, MusicAlbum, MusicTrack, \
-        PhotoAlbum, Image, File
+        PhotoAlbum, Image, File, PlayableModel
+
 from storm.expr import Desc, Asc
- 
+
+import os
+
+try:
+    from hashlib import md5
+except ImportError:
+    # hashlib is new in Python 2.5
+    from md5 import md5
+
+
 def result_set_order_offset_and_limit(result_set, field, offset, limit):
     result_set.order_by(field).config(offset=offset, limit=limit)
     return result_set
 
+
 def result_set_first(result_set):
     return result_set.first()
 
+
 def result_set_one(result_set):
     return result_set.one()
 
+
 def result_set_any(result_set):
     return result_set.any()
 
+
 def result_set_all(result_set):
     return result_set.all()
 
+
 def dbus_errback(failure, errback):
     errback(failure.value)
-    
+
+
 def build_last_track_played_tuple(music_track):
     def get_artist(album):
         def build_tuple(artist):
@@ -68,6 +90,13 @@ def build_last_track_played_tuple(music_track):
 
     return dfr
 
+
+def _thumbnail_file(uri):
+    thumbnail = md5(str(uri)).hexdigest() + '.jpg'
+    filename = os.path.join(PICTURES_CACHE, thumbnail)
+    return filename
+
+
 def build_last_picture_played_tuple(picture):
     def build_tuple(album):
         return album.name, picture.file_path
@@ -80,29 +109,41 @@ def build_last_picture_played_tuple(picture):
 
     return dfr
 
+
 class DatabaseError(Exception):
     pass
 
+
 class ArtistNotFoundError(DatabaseError):
     pass
 
+
 class MusicAlbumNotFoundError(DatabaseError):
     pass
 
+
 class MusicTrackNotFoundError(DatabaseError):
     pass
 
+
 class PhotoAlbumNotFoundError(DatabaseError):
     pass
 
+
 class PictureNotFoundError(DatabaseError):
     pass
 
+
+class AlbumArtworkNotAvailable(Exception):
+    pass
+
+
 class MusicImpl(object):
     def _build_artists_albums_list(self, albums):
         def find_album_tracks_iterator(albums, artists_albums_set):
             for album in albums:
-                dfr = album.tracks.find()
+                dfr = album.tracks.find(MusicTrack.file_path == File.path,
+                                        File.hidden == False)
                 dfr.addCallback(result_set_all)
                 dfr.addCallback(self._find_tracks_artists,
                         album, artists_albums_set)
@@ -157,17 +198,36 @@ class MusicImpl(object):
 
             return music_track.album
 
-        def get_album_cover_uri(album):
+        def get_uri(album):
             return album.cover_uri
 
+        def get_album_cover_uri(album):
+            if album.cover_uri:
+                return album.cover_uri
+
+            # album_cover_retriever.get_cover_uri() returns an Album [sic!]
+            dfr = album_cover_retriever.get_cover_uri(album, artist_name)
+            dfr.addCallback(get_uri)
+
+            return dfr
+
         def find_cover_art(artist):
             if artist is None:
-                raise ArtistNotFoundError()
+                # Try to find the album directly by name.  Not very reliable,
+                # and not very frequently needed, but it's better than no art?
+                # Especially if we want to preseed database with album art,
+                # we need this fallback.
+                dfr = common.application.store.find(MusicAlbum, MusicAlbum.name == album_name)
+                dfr.addCallback(result_set_one)
+                dfr.addCallback(get_album_cover_uri)
+                dfr.addCallback(self._get_cover_thumbnail)
+                return dfr
 
             dfr = artist.tracks.find(MusicTrack.album_name == album_name)
             dfr.addCallback(result_set_any)
             dfr.addCallback(get_album)
             dfr.addCallback(get_album_cover_uri)
+            dfr.addCallback(self._get_cover_thumbnail)
 
             return dfr
 
@@ -188,7 +248,9 @@ class MusicImpl(object):
             if artist is None:
                 raise ArtistNotFoundError()
 
-            dfr = artist.tracks.find(MusicTrack.album_name == album_name)
+            dfr = artist.tracks.find(MusicTrack.album_name == album_name,
+                                     MusicTrack.file_path == File.path,
+                                     File.hidden == False)
             dfr.addCallback(result_set_all)
             dfr.addCallback(build_track_list)
 
@@ -211,12 +273,39 @@ class MusicImpl(object):
 
         return dfr
 
+    def _get_cover_thumbnail(self, thumbnail_uri):
+        if thumbnail_uri is None:
+            # The cover art has never been requested from the UI for this album
+            # FIXME: request it (from Amazon)
+            raise AlbumArtworkNotAvailable()
+
+        thumbnail_file = _thumbnail_file(thumbnail_uri)
+        if os.path.exists(thumbnail_file):
+            return defer.succeed(thumbnail_file)
+        
+        return self._download_cover_thumbnail(thumbnail_uri,
+                thumbnail_file)
+
+    def _download_cover_thumbnail(self, thumbnail_uri, filename):
+        def got_thumbnail(data_model):
+            fd = file(filename, 'wb')
+            fd.write(data_model.data)
+            fd.close()
+            return filename
+
+        uri = MediaUri(thumbnail_uri)
+        model, deferred = common.application.resource_manager.get(uri)
+        deferred.addCallback(got_thumbnail)
+        return deferred
+
+
 class Music(dbus.service.Object):
     interface = 'com.fluendo.Elisa.Plugins.Database.Music'
+    implFactory = MusicImpl
 
     def __init__(self, *args, **kw):
         dbus.service.Object.__init__(self, *args, **kw)
-        self.impl = MusicImpl()
+        self.impl = self.implFactory()
 
     @dbus.service.method(dbus_interface=interface,
             in_signature='ii', out_signature='aas',
@@ -256,6 +345,7 @@ class Music(dbus.service.Object):
         # don't do anything, just so that the signal is emitted
         pass
 
+
 class PhotoImpl(object):
     def get_albums(self, offset, count):
         if offset < 0 or count <= 0:
@@ -279,7 +369,8 @@ class PhotoImpl(object):
             if album is None:
                 raise PhotoAlbumNotFoundError()
 
-            dfr = album.photos.find()
+            dfr = album.photos.find(Image.file_path == File.path,
+                                    File.hidden == False)
             dfr.addCallback(result_set_all)
             dfr.addCallback(build_pictures_list)
 
@@ -295,6 +386,7 @@ class PhotoImpl(object):
     def get_last_displayed_picture(self):
         dfr = common.application.store.find(Image,
                 Image.file_path == File.path,
+                File.hidden == False,
                 File.last_played != 0)
         dfr.addCallback(result_set_order_offset_and_limit,
                 Desc(File.last_played), offset=None, limit=None)
@@ -303,12 +395,14 @@ class PhotoImpl(object):
 
         return dfr
 
+
 class Photo(dbus.service.Object):
     interface = 'com.fluendo.Elisa.Plugins.Database.Photo'
-    
+    implFactory = PhotoImpl
+
     def __init__(self, *args, **kw):
         dbus.service.Object.__init__(self, *args, **kw)
-        self.impl = PhotoImpl()
+        self.impl = self.implFactory()
 
     @dbus.service.method(dbus_interface=interface,
             in_signature='ii', out_signature='as',
@@ -339,28 +433,29 @@ class Photo(dbus.service.Object):
         # don't do anything, just so that the signal is emitted
         pass
 
+
 class DatabaseDBusServiceProvider(ServiceProvider):
     def start(self): 
         self._last_played_track_path = None
         self._last_displayed_picture_path = None
-        
-        bus = dbus.SessionBus()
-        
-        # get known names
-        self.bus_name = \
-                dbus.service.BusName('com.fluendo.Elisa.Plugins.Database', bus)
-    
-        # create the objects we export
-        self.photo = Photo(bus, '/com/fluendo/Elisa/Plugins/Database/Photo',
-                self.bus_name)
-        self.music = Music(bus, '/com/fluendo/Elisa/Plugins/Database/Music',
-                self.bus_name)
 
+        self._start_dbus()
         self._register_player_status_changes()
 
         return defer.succeed(self)
 
     def stop(self):
+        self._stop_dbus()
+        return defer.succeed(self)
+
+    def _start_dbus(self):
+        bus = dbus.SessionBus()
+
+        # create the objects we export
+        self.photo = Photo(bus, '/com/fluendo/Elisa/Plugins/Database/Photo')
+        self.music = Music(bus, '/com/fluendo/Elisa/Plugins/Database/Music')
+
+    def _stop_dbus(self):
         bus = dbus.SessionBus()
 
         self.photo.remove_from_connection(bus,
@@ -368,11 +463,6 @@ class DatabaseDBusServiceProvider(ServiceProvider):
         self.music.remove_from_connection(bus,
                 '/com/fluendo/Elisa/Plugins/Database/Music')
 
-        # BusName implements __del__, eew
-        del self.bus_name
-
-        return defer.succeed(self)
-
     def _register_player_status_changes(self):
         bus = common.application.bus
         bus.register(self._components_loaded, ComponentsLoadedMessage)
@@ -395,23 +485,40 @@ class DatabaseDBusServiceProvider(ServiceProvider):
         if status != player.PLAYING:
             return defer.succeed(None)
 
-        current_track_path = player.playlist[player.current_index].uri.path
+        model = player.playlist[player.current_index]
+        if isinstance(model, MusicTrack):
+            current_track_path = model.file_path
+        elif isinstance(model, PlayableModel) and \
+                model.uri.scheme == 'file':
+            current_track_path = model.uri.path
+        else:
+            self.log('not emitting last_played_track for %s' % model)
+            return defer.succeed(None)
+
         if self._last_played_track_path is None or \
                 current_track_path != self._last_played_track_path:
-            
             self._last_played_track_path = current_track_path
 
-            dfr = common.application.store.find(MusicTrack,
-                    MusicTrack.file_path == current_track_path)
-            dfr.addCallback(result_set_one)
+            if isinstance(model, MusicTrack):
+                dfr = defer.succeed(model)
+            else:
+                dfr = common.application.store.find(MusicTrack,
+                        MusicTrack.file_path == current_track_path)
+                dfr.addCallback(result_set_one)
+
             dfr.addCallback(build_last_track_played_tuple)
             dfr.addCallback(self._emit_last_played_track)
+            def log_failure(failure):
+                self.info('not emitting last_played_track signal for %s: %s'
+                        % (current_track_path, getFailureMessage(failure)))
+            dfr.addErrback(log_failure)
 
             return dfr
 
         return defer.succeed(None)
     
     def _emit_last_played_track(self, last_played_track_tuple):
+        self.log('emitting last_played_track %s' % str(last_played_track_tuple))
         self.music.last_played_track(*last_played_track_tuple)
 
     def _slideshow_player_current_index_changed_cb(self,
@@ -435,6 +542,7 @@ class DatabaseDBusServiceProvider(ServiceProvider):
     def _emit_last_displayed_picture(self, last_displayed_picture_tuple):
         self.photo.last_displayed_picture(*last_displayed_picture_tuple)
 
+
 if __name__ == '__main__':
     dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
     loop = gobject.MainLoop()
