@@ -24,20 +24,32 @@ from elisa.core import common
 from elisa.core.utils.locale_helper import system_encoding
 from elisa.core.media_uri import MediaUri
 
-from twisted.internet import task, defer, reactor, threads
+from twisted.internet import task, reactor, threads
 from twisted.python.failure import Failure
 
+from elisa.core.utils.cancellable_defer import cancellable_coiterate, \
+        CancelledError
+
+from elisa.core.utils import defer
+
 import time, os
 
 from elisa.core.components.resource_provider import ResourceProvider
 
 from elisa.plugins.database.scanner_models import Statistic, ScanResource
 from elisa.plugins.database.database_parser import DatabaseParser
+from elisa.plugins.database.models import \
+         PICTURES_SECTION, MUSIC_SECTION, VIDEO_SECTION
 
 from elisa.plugins.base.local_resource import LocalResource
 
 from elisa.plugins.base.models.file import DirectoryModel
 
+try:
+    import dbus
+    from elisa.plugins.database.dbus_iface import DBusInterface
+except ImportError:
+    dbus = None
 
 # FIXME: we put here also schema stuff not strictly related with the media
 # scanner (the "favorites" table), because we don't have a general
@@ -60,6 +72,7 @@ SCHEMA = [
         "CREATE TABLE IF NOT EXISTS files" \
                 " (path VARCHAR PRIMARY KEY, source VARCHAR," \
                 " modification_time INTEGER DEFAULT 0, mime_type VARCHAR," \
+                " hidden INTEGER DEFAULT 0," \
                 " deleted INTEGER, playcount INTEGER DEFAULT 0, last_played" \
                 " INTEGER DEFAULT 0);",
         "CREATE TABLE IF NOT EXISTS file_tags" \
@@ -70,17 +83,47 @@ SCHEMA = [
                 " shot_time INTEGER, with_flash INTEGER DEFAULT 0," \
                 " orientation INTEGER, gps_altitude INTEGER, " \
                 " gps_latitude VARCHAR, gps_longitude VARCHAR," \
-                " album_name VARCHAR);",
+                " album_name VARCHAR, section INTEGER DEFAULT 0);",
         "CREATE TABLE IF NOT EXISTS photo_albums"\
                 " (name VARCHAR PRIMARY KEY, preview_uri VARCHAR);",
         "CREATE TABLE IF NOT EXISTS videos"\
-                " (file_path VARCHAR PRIMARY KEY, size VARCHAR,"\
-                " duration VARCHAR, codec VARCHAR, thumbnail_uri VARCHAR);",
+                " (file_path VARCHAR PRIMARY KEY, name VARCHAR,"\
+                " creation_time INTEGER, size VARCHAR, duration VARCHAR,"\
+                " codec VARCHAR, thumbnail_uri VARCHAR);",
 
         "CREATE TABLE IF NOT EXISTS favorites"\
                 " (id INTEGER PRIMARY KEY, section TEXT,"\
                 " added_time TEXT, title TEXT, subtitle,"\
                 " foreign_id TEXT, foreign_class TEXT, uri TEXT, type TEXT);",
+        # delete triggers
+        "CREATE TRIGGER IF NOT EXISTS \"file del\" "\
+            "BEFORE DELETE ON files BEGIN "\
+                "DELETE FROM music_tracks WHERE file_path = old.path; "\
+                "DELETE FROM images WHERE file_path = old.path; "\
+                "DELETE FROM videos WHERE file_path = old.path; "\
+            "END", 
+        "CREATE TRIGGER IF NOT EXISTS \"music track album del\" "\
+            "AFTER DELETE ON music_tracks "\
+            "WHEN (SELECT count(*) from music_tracks WHERE "\
+                    "album_name = old.album_name) = 0  BEGIN "\
+                "DELETE FROM music_albums WHERE name = old.album_name; "\
+            "END",
+        "CREATE TRIGGER IF NOT EXISTS \"del artist\" "\
+            "AFTER DELETE ON track_artists "\
+            "WHEN (SELECT count(*) from track_artists WHERE "\
+                   "artist_name = old.artist_name) = 0  BEGIN "\
+                "DELETE FROM artists WHERE name = old.artist_name; "\
+            "END",
+        "CREATE TRIGGER IF NOT EXISTS \"music track artists del\" "\
+            "AFTER DELETE ON music_tracks BEGIN "\
+                "DELETE FROM track_artists WHERE track_path = old.file_path; "\
+            "END",
+        "CREATE TRIGGER IF NOT EXISTS \"image album del\" "\
+            "AFTER DELETE ON images "\
+            "WHEN (SELECT count(*) from images WHERE album_name = "\
+                    "old.album_name) = 0  BEGIN " \
+                "DELETE FROM photo_albums WHERE name = old.album_name; "\
+            "END",
         ]
 
 class MediaScanner(ResourceProvider):
@@ -91,9 +134,11 @@ class MediaScanner(ResourceProvider):
     @type store: L{storm.twisted.storm.DeferredStore}
     """
 
-    default_config = {"delay" : 0.1}
+    default_config = {"delay" : 0.1, "scan_every" : 24}
     config_doc = {
-        "delay": "the delay (in seconds) between processing two files"}
+        "delay": "the delay (in seconds) between processing two files",
+        "scan_every" : "the delay (in hours) between two automatic scans"
+        }
 
     supported_uri = '^media_scanner://localhost/'
 
@@ -102,9 +147,11 @@ class MediaScanner(ResourceProvider):
         self.store = None
         self._current_delay = None
         self._start_delay = None
+        self._current_scan_deferred = None
 
         self.scan_stat = Statistic()
         self.scan_stat.uri = MediaUri('media_scanner://localhost/statistic')
+        self._pending_auto_rescan = None
 
         self.running = False
 
@@ -115,13 +162,14 @@ class MediaScanner(ResourceProvider):
         if not directories_section:
             self.debug("no default directories")
             return
-        for place in ('music', 'pictures'):
-            paths = directories_section.get(place, [])
+
+        for section in ('music', 'pictures', 'video'):
+            paths = directories_section.get(section, [])
             for path in paths:
                 if path != '*default*': # skip the default configuration thingy
                     uri = MediaUri('file://%s' % path)
                     self.debug("adding %s" % uri)
-                    self.put(uri, None)
+                    self.put(uri, None, section=section)
 
     def initialize(self):
 
@@ -147,9 +195,36 @@ class MediaScanner(ResourceProvider):
         dfr.addCallback(set_parser)
         dfr.addCallback(lambda x: LocalResource.create({}))
         dfr.addCallback(set_local_resource)
+        # start dbus at the end
+        dfr.addCallback(self._initialize_dbus)
         return dfr
 
+    def _initialize_dbus(self, result=None):
+        if dbus is None:
+            # no dbus support
+            return self
+
+        bus = dbus.SessionBus()
+
+        self.dbus_scanner = DBusInterface(self, bus,
+                '/com/fluendo/Elisa/Plugins/Database/MediaScanner')
+        return self
+
+    def _clean_dbus(self):
+        if dbus is None:
+            # no dbus support
+            return
+
+        bus = dbus.SessionBus()
+        self.dbus_scanner.remove_from_connection(bus,
+                '/com/fluendo/Elisa/Plugins/Database/MediaScanner')
+
+        # remove the reference cycle
+        self.dbus_scanner = None
+
+
     def clean(self):
+        self._clean_dbus()
         if self._current_delay and not self._current_delay.called:
             self._current_delay.cancel()
 
@@ -182,11 +257,52 @@ class MediaScanner(ResourceProvider):
             if not u'last_played' in result:
                 sqlexec("ALTER TABLE files ADD last_played INTEGER DEFAULT 0;")
                 commit = True
+            if u'hidden' not in result:
+                sqlexec("ALTER TABLE files ADD hidden INTEGER DEFAULT 0;")
+                commit = True
 
             if commit:
                 return self.store.commit()
 
         def got_images(result):
+
+            def build_update_query(section, path_list):
+                path = path_list[0]
+                query = "UPDATE images SET section = %d" \
+                        " WHERE file_path LIKE \"%s%%\"" % \
+                                             (section, os.path.join(path, ''))
+                for path in path_list[1:]:
+                    query += " OR file_path LIKE \"%s%%" % \
+                                              os.path.join(path, '')
+                return query
+
+            def update_image_albums(result):
+                # we remove the photo albums that are not related to images we
+                # got from the pictures section. That fixes databases affected
+                # by #259427.
+                sqlexec("DELETE FROM photo_albums WHERE name NOT IN " \
+                        "(SELECT DISTINCT album_name FROM images "\
+                        "WHERE section=%d)" % PICTURES_SECTION)
+                return self.store.commit()
+
+            def update_image_section(result):
+                # FIXME: blocks
+                # FIXME: doesn't handle images that are in more than one
+                # section
+                directories_section = common.application.config.get_section \
+                                                              ('directories')
+                for section_name, section in (('music', MUSIC_SECTION),
+                                              ('video', VIDEO_SECTION),
+                                              ('pictures', PICTURES_SECTION)):
+                    path_list = directories_section.get(section_name, [])
+                    if path_list != []:
+                        query = build_update_query(section, path_list)
+                        sqlexec(query)
+
+                dfr =  self.store.commit()
+                dfr.addCallback(update_image_albums)
+                return dfr
+
             if not 'shot_time' in result:
                 # shot_time is part of the first update we did for the image
                 # table. as it is not there, we have to update the table
@@ -198,7 +314,16 @@ class MediaScanner(ResourceProvider):
                 sqlexec("ALTER TABLE images ADD gps_latitude VARCHAR;")
                 sqlexec("ALTER TABLE images ADD gps_longitude VARCHAR;")
                 sqlexec("ALTER TABLE images ADD album_name VARCHAR;")
-                return self.store.commit()
+                sqlexec("ALTER TABLE images ADD section INTEGER DEFAULT 0;")
+                dfr = self.store.commit()
+                dfr.addCallback(update_image_section)
+                return dfr
+
+            elif not 'section' in result:
+                sqlexec("ALTER TABLE images ADD section INTEGER DEFAULT 0;")
+                dfr = self.store.commit()
+                dfr.addCallback(update_image_section)
+                return dfr
 
         def got_favorites(result):
             if not 'type' in result:
@@ -209,6 +334,14 @@ class MediaScanner(ResourceProvider):
             if not u'release_date' in result:
                 sqlexec("ALTER TABLE music_albums ADD release_date VARCHAR;")
                 return self.store.commit()
+        
+        def got_videos(result):
+            if not u'name' in result:
+                sqlexec("ALTER TABLE videos ADD name VARCHAR;")
+                return self.store.commit()
+            if not u'creation_time' in result:
+                sqlexec("ALTER TABLE videos ADD creation_time INTEGER;")
+                return self.store.commit()
 
         def do_files_update():
             dfr = get_table_info('files')
@@ -229,10 +362,16 @@ class MediaScanner(ResourceProvider):
             dfr = get_table_info("favorites")
             dfr.addCallback(got_favorites)
             return dfr
+        
+        def do_videos_update(old_res=None):
+            dfr = get_table_info("videos")
+            dfr.addCallback(got_videos)
+            return dfr
 
         dfr = do_files_update()
         dfr.addCallback(do_albums_update)
         dfr.addCallback(do_images_update)
+        dfr.addCallback(do_videos_update)
         dfr.addCallback(do_favorites_update)
         return dfr
 
@@ -253,6 +392,8 @@ class MediaScanner(ResourceProvider):
     def running_set(self, value):
         self._running = value
         self.scan_stat.running = value
+        if not value:
+            self._current_scan_deferred = None
 
     running = property(fget=running_get, fset=running_set)
 
@@ -267,10 +408,15 @@ class MediaScanner(ResourceProvider):
 
         return (None, defer.fail(NotImplementedError()))
 
-    def put(self, source_uri, container_uri, context_model=None):
+    def put(self, source_uri, container_uri,
+            context_model=None, section=None):
         """
         put another uri into the queue and start the scanning process if it is
         not yet running
+
+        @ivar section:  the name of the section relevant to the content that
+                        is put.
+        @type section:  L{str}
         """
         # FIXME check the container uri
         # FIXME no check if this uri might already in the queue...
@@ -279,10 +425,9 @@ class MediaScanner(ResourceProvider):
             # we want to have the content, so we have to add a trailing slash
             source_uri = source_uri.join('')
 
-        def start_scanning(result, source_uri):
+        def start_scanning(result, source_uri, section):
             name = os.path.basename(os.path.dirname(source_uri.path))
-            s = ScanResource(source_uri, name)
-            s.name
+            s = ScanResource(source_uri, name, section=section)
             self.scan_stat.queue.put(s)
 
             if not self.running:
@@ -291,8 +436,8 @@ class MediaScanner(ResourceProvider):
             return s.defer
 
         return self.parser.mark_deleted(source_uri.path).addCallback(start_scanning,
-                source_uri)
-       
+                source_uri, section)
+
     # internally used
     def _update_stat(self, result, model, stat):
         stat.files_scanned += 1
@@ -304,31 +449,43 @@ class MediaScanner(ResourceProvider):
     def _count_files(self, path):
         path = os.path.abspath(path)
 
+        deferred = defer.Deferred()
+
         def iterate(path):
             counter = 0
-            for root, dirs, files in os.walk(path.encode(system_encoding())):
-                # count only the files
-                counter += len(files)
-            return counter
+            try:
+                for root, dirs, files in os.walk(path.encode(system_encoding())):
 
-        dfr = threads.deferToThread(iterate, path)
-        return dfr
+                    if deferred.called:
+                        # we are cancelled. Stop the processing without calling
+                        # the deferred again
+                        return
+
+                    # count only the files
+                    counter += len(files)
+
+                reactor.callFromThread(deferred.callback, counter)
+
+            except Exception, exc:
+                reactor.callFromThread(deferred.errback, exc)
+
+        # push it into background
+        threads.deferToThread(iterate, path)
+
+        return deferred
 
     def _file_found(self, model, stat):
         dfr = self.parser.query_model(model, stat)
         dfr.addBoth(self._update_stat, model, stat)
         return dfr
 
-    def _scan(self):
+    def _cleanup_deleted_files(self, result, scan_resource):
+        return self.parser.delete_files(scan_resource.root_uri.path)
+
+    def _scan(self, result=None):
 
         def run_next(result, scan_resource):
-            if scan_resource.pending_dfrs:
-                # some metadata pending
-                scan_resource.state = ScanResource.SCANNING_METADATA
-            else:
-                # nothing pending, let's go on
-                scan_resource.state = ScanResource.SCANNING_DONE
-                scan_resource.defer.callback(ScanResource.SCANNING_DONE)
+            scan_resource.state = ScanResource.SCANNING_DONE
 
             self.scan_stat.currently_scanning = None
             self.scan_stat.scanned.append(scan_resource)
@@ -336,8 +493,9 @@ class MediaScanner(ResourceProvider):
             if self.scan_stat.queue.empty():
                 # stop processing
                 self.running = False
+                self._reset_auto_rescan(True)
             else:
-                return self._scan()
+                reactor.callLater(0.1, self._scan)
 
         self.running = True
 
@@ -356,14 +514,42 @@ class MediaScanner(ResourceProvider):
 
         dfr = self._count_files(scan_resource.root_uri.path)
         dfr.addCallback(set_total_files, scan_resource)
+        dfr.addCallback(self._cleanup_deleted_files, scan_resource)
         dfr.addCallback(run_next, scan_resource)
 
+        self._current_scan_deferred = dfr
+
+    def _rescan(self):
+        self.debug("Starting to scan everything again")
+        dfr = self._reschedule_scanned()
+        dfr.addCallback(self._scan)
         return dfr
 
+    def _reset_auto_rescan(self, restart=False):
+        if self._pending_auto_rescan and self._pending_auto_rescan.active():
+            self._pending_auto_rescan.cancel()
+
+        rescan_every = self.config.get('scan_every', 0) * 3600
+        if restart and rescan_every != 0:
+            pending = reactor.callLater(rescan_every, self._rescan)
+            self._pending_auto_rescan = pending
+
+    def _reschedule_scanned(self):
+        scan_stat = self.scan_stat
+
+        scanned, scan_stat.scanned = scan_stat.scanned, [] 
+
+        deleted_dfrs = []
+        for stat in scanned:
+            scan_stat.queue.put(stat)
+            deleted_dfrs.append(self.parser.mark_deleted(stat.root_uri.path))
+
+        if len(deleted_dfrs) == 0:
+            return defer.succeed(None)
+
+        return defer.DeferredList(deleted_dfrs)
+
     def _scan_recursive(self, scan_resource):
-        #resources = common.application.resource_manager
-        resources = self.local_resource
-        uri = scan_resource.root_uri
 
         def stopper(data, file):
             self.warning("%s failed: %s" % (file, data))
@@ -380,23 +566,21 @@ class MediaScanner(ResourceProvider):
                         dfr = self._file_found(item, scan_resource)
                         yield dfr
 
-            return task.coiterate(iterator(model, dirs))
+            return cancellable_coiterate(iterator, model, dirs)
 
         def scan_dirs(dirs):
             def iterator(dirs):
                 while len(dirs):
-                    item_uri = dirs.pop(0)
-                    rewrite = '%s'
-                    if item_uri.filename != '':
+                    uri = dirs.pop(0)
+                    if uri.filename != '':
                         # we have a filename append the slash to get the
                         # content
-                        rewrite += '/'
-                    uri = MediaUri(rewrite % item_uri)
-                    model, dfr = resources.get(uri)
+                        uri = uri.join('')
+                    model, dfr = self.local_resource.get(uri)
                     dfr.addCallback(scan_children, dirs)
                     dfr.addErrback(stopper, uri.path)
                     yield dfr
 
-            return task.coiterate(iterator(dirs))
+            return cancellable_coiterate(iterator, dirs)
 
-        return scan_dirs([uri])
+        return scan_dirs([scan_resource.root_uri])
