@@ -44,7 +44,8 @@ from elisa.plugins.poblesec.base.grid import GridItemGridController
 from elisa.plugins.poblesec.actions import Action
 from elisa.plugins.database.actions import ArtistPlayAllAction, \
                                            AlbumPlayAllAction, \
-                                           TrackPlayAllAction
+                                           TrackPlayAllAction, \
+                                           TrackRemoveAction
 from elisa.plugins.database.actions import ArtistShuffleAction, \
                                            AlbumShuffleAction, \
                                            TrackShuffleAction
@@ -63,7 +64,20 @@ import datetime
 
 _ = install_translation('database')
 
+# this is totally searcher controller based
+from elisa.plugins.poblesec.search_controller import SearcherEntry
 
+
+def music_search_result_decorator(controller):
+    searcher = 'DBSearcher'
+    title = _('Local Search Results')
+    icon_resource = 'elisa.plugins.poblesec.search'
+    searcher_entry = SearcherEntry(searcher, title, icon_resource)
+    controller.searchers.append(searcher_entry)
+    return defer.succeed(None)
+
+
+# other decorators
 def music_lib_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/music_library'
@@ -123,32 +137,28 @@ class GenericArtistsController(HierarchyController):
 
     artist_controller_path = '/poblesec/database/music/albums'
 
+    empty_label = _('There are no artists in this section')
+    empty_icon = 'elisa.plugins.poblesec.by_artist'
+
     def initialize(self):
         dfr = super(GenericArtistsController, self).initialize()
         self.actions.extend(self.make_actions())
         return dfr
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
+        self.cancel_deferreds(id(self))
 
-        if selected_item != self._previous_clicked:
-            self.cancel_deferreds(id(self))
-            selected_item.activate(previous=self._previous_clicked)
+        if isinstance(item, Action):
+            def silence_canceller(failure):
+                failure.trap(CancelledError)
 
-            self._previous_clicked = selected_item
-
-            if isinstance(item, Action):
-                def silence_canceller(failure):
-                    failure.trap(CancelledError)
-
-                dfr = item.run()
-                dfr.addErrback(silence_canceller)
-                self.register_deferred(id(self), dfr)
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = self.artist_controller_path
-                deferred = browser.history.append_controller(path, item.name, artist=item)
+            dfr = item.run()
+            dfr.addErrback(silence_canceller)
+            self.register_deferred(id(self), dfr)
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            path = self.artist_controller_path
+            deferred = browser.history.append_controller(path, item.name, artist=item)
 
     def make_actions(self):
         actions = []
@@ -176,7 +186,6 @@ class ArtistsDbController(GenericArtistsController):
         self.artists = artists
 
         def got_artists(artists):
-            artists = sorted(artists, key=lambda a: a.name.lower())
             self.model.extend(artists)
             return self
 
@@ -258,6 +267,9 @@ class GenericAlbumsDbController(HierarchyController):
 
     album_controller_path = '/poblesec/database/music/tracks'
 
+    empty_label = _('There are no albums in this section')
+    empty_icon = 'elisa.plugins.poblesec.by_album'
+
     def initialize(self, artist=None):
         self.artist = artist
         dfr = super(GenericAlbumsDbController, self).initialize()
@@ -288,9 +300,15 @@ class GenericAlbumsDbController(HierarchyController):
             action.toggled_subtitle = _("Remove The Artist '%s' From Your Favorites") % self.artist.name
             actions.append(action)
 
-            dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.artist.name,
-                                                FavoritesItem.foreign_class == u'Artist')
+            uri = getattr(self.artist, 'uri', None)
+            if uri:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.uri == unicode(uri),
+                                                    FavoritesItem.type == u'artist')
+            else:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.foreign_id == self.artist.name,
+                                                    FavoritesItem.foreign_class == u'Artist')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         else:
@@ -309,32 +327,24 @@ class GenericAlbumsDbController(HierarchyController):
         return actions
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(item, Action):
-                dfr = item.run()
-            elif hasattr(item, 'all_tracks'):
-                # Dummy "All Tracks" album
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                dfr = browser.history.append_controller('/poblesec/database/music/tracks',
-                                                        item.name,
-                                                        artist=item.artist)
-            elif hasattr(item, 'action') and issubclass(item.action, Action):
-                action = item.action(self)
-                dfr = action.run(item)
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = self.album_controller_path
-                dfr = browser.history.append_controller(path,
-                                                        item.name,
-                                                        album=item)
-            return dfr
+        if isinstance(item, Action):
+            dfr = item.run()
+        elif hasattr(item, 'all_tracks'):
+            # Dummy "All Tracks" album
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            dfr = browser.history.append_controller('/poblesec/database/music/tracks',
+                                                    item.name,
+                                                    artist=item.artist)
+        elif hasattr(item, 'action') and issubclass(item.action, Action):
+            action = item.action(self)
+            dfr = action.run(item)
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            path = self.album_controller_path
+            dfr = browser.history.append_controller(path,
+                                                    item.name,
+                                                    album=item)
+        return dfr
 
 
 class AlbumsDbController(GenericAlbumsDbController):
@@ -392,13 +402,19 @@ class AlbumsDbController(GenericAlbumsDbController):
                 return result_set.values(MusicTrack.album_name)
 
             def get_tracks(self):
-                return artist.tracks.find()
+                return artist.tracks.find(MusicTrack.file_path == File.path,
+                                          File.hidden == False)
 
-            deferred.addCallback(get_tracks)
-            deferred.addCallback(order_tracks)
-            deferred.addCallback(get_albums)
-            deferred.addCallback(sort_albums)
-            deferred.addCallback(add_albums)
+            # retrieve tracks for not-database artists
+            if getattr(artist, 'uri', None):
+                self.model.extend(artist.albums)
+            # retrieve tracks for database artists
+            else:
+                deferred.addCallback(get_tracks)
+                deferred.addCallback(order_tracks)
+                deferred.addCallback(get_albums)
+                deferred.addCallback(sort_albums)
+                deferred.addCallback(add_albums)
 
         return deferred
 
@@ -471,6 +487,9 @@ class GenericTracksDbController(HierarchyController):
 
     track_controller_path = '/poblesec/database/music/tracks'
 
+    empty_label = _('There are no tracks in this section')
+    empty_icon = 'elisa.plugins.poblesec.by_track'
+
     def initialize(self, album=None, track=None, tracks=None, artist=None):
         self.album = album
         self.track = track
@@ -487,6 +506,9 @@ class GenericTracksDbController(HierarchyController):
         self.actions.extend(self.make_actions())
         return dfr
 
+    def is_empty(self):
+        return super(GenericTracksDbController, self).is_empty() and self.track is None
+
     def item_to_label(self, item):
         if isinstance(item, Action):
             return '#'
@@ -513,9 +535,15 @@ class GenericTracksDbController(HierarchyController):
             action.toggled_subtitle = _("Remove The Album '%s' From Your Favorites") % self.album.name
             actions.append(action)
 
-            dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.album.name,
-                                                FavoritesItem.foreign_class == u'MusicAlbum')
+            uri = getattr(self.album, 'uri', None)
+            if uri:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.uri == unicode(uri),
+                                                    FavoritesItem.type == u'music_album')
+            else:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.foreign_id == self.album.name,
+                                                    FavoritesItem.foreign_class == u'MusicAlbum')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         elif self.track:
@@ -531,11 +559,28 @@ class GenericTracksDbController(HierarchyController):
             action.toggled_subtitle = _("Remove '%s' From Your Favorites") % self.track.title
             actions.append(action)
 
-            dfr = common.application.store.find(FavoritesItem,
-                                                FavoritesItem.foreign_id == self.track.file_path,
-                                                FavoritesItem.foreign_class == u'MusicTrack')
+            uri = getattr(self.track, 'uri', None)
+            if uri:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.uri == unicode(uri),
+                                                    FavoritesItem.type == u'music_track')
+            else:
+                dfr = common.application.store.find(FavoritesItem,
+                                                    FavoritesItem.foreign_id == self.track.file_path,
+                                                    FavoritesItem.foreign_class == u'MusicTrack')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
+
+            # FIXME: implement this properly instead of hijacking the app config
+            app_config = common.application.config
+            if int(app_config.get_option('enable_remove',
+                                         'poblesec.main:PoblesecController',
+                                         default='0')):
+                remove_action = TrackRemoveAction(self)
+                remove_action.title = _('Remove Track')
+                remove_action.subtitle = _("Remove '%s'") % self.track.title
+                actions.append(remove_action)
+
         else:
             action = TrackPlayAllAction(self)
             action.title = _('Play All')
@@ -548,44 +593,38 @@ class GenericTracksDbController(HierarchyController):
         return actions
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-            self._previous_clicked = selected_item
-            if isinstance(item, Action):
-                item.run()
-            elif hasattr(item, 'action') and issubclass(item.action, Action):
-                action = item.action(self)
-                dfr = action.run(item)
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = self.track_controller_path
-                title = item.title
-                tracks = [t for t in self.model if not isinstance(t, Action)]
-                args = {'track': item,
-                        'tracks': tracks}
-                dfr = browser.history.append_controller(path, title, **args)
+        if isinstance(item, Action):
+            item.run()
+        elif hasattr(item, 'action') and issubclass(item.action, Action):
+            action = item.action(self)
+            dfr = action.run(item)
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            path = self.track_controller_path
+            title = item.title
+            tracks = [t for t in self.model if not isinstance(t, Action)]
+            args = {'track': item,
+                    'tracks': tracks}
+            dfr = browser.history.append_controller(path, title, **args)
 
 
 class TracksDbController(GenericTracksDbController):
 
     def initialize(self, album=None, track=None, tracks=None, artist=None):
-        deferred = super(TracksDbController, self).initialize(album=album,
-                                                              track=track,
-                                                              tracks=tracks,
-                                                              artist=artist)
+        dfr = super(TracksDbController, self).initialize(album=album,
+                                                         track=track,
+                                                         tracks=tracks,
+                                                         artist=artist)
 
         if track:
-            return deferred
+            return dfr
 
         def populate_model(self, tracks):
             self.model.extend(tracks)
             return self
 
         if tracks is not None:
-            return deferred.addCallback(populate_model, tracks)
+            return dfr.addCallback(populate_model, tracks)
 
         def got_tracks(tracks):
             self.model.extend(tracks)
@@ -595,10 +634,6 @@ class TracksDbController(GenericTracksDbController):
             result_set.order_by(MusicTrack.track_number, MusicTrack.title)
             return result_set.all()
 
-        def sort_tracks_title(result_set):
-            result_set.order_by(MusicTrack.title)
-            return result_set.all()
-
         def sort_tracks_title_client_side(tracks):
             trash_letters = " .,-_"
             tracks.sort(key=lambda t: t.title.strip(trash_letters).lower())
@@ -609,30 +644,53 @@ class TracksDbController(GenericTracksDbController):
             if self.artist is not None:
                 # oh, hey, let's fetch all the tracks of the given artist
                 dfr = store.find(MusicTrack,
+                                 MusicTrack.file_path == File.path,
+                                 File.hidden == False,
                                  MusicTrack.file_path == TrackArtist.track_path,
                                  TrackArtist.artist_name == self.artist.name)
                 # Add client side ordering - can this be done "server" side? Is
                 # there a better way to trash spurious characters?
-                dfr.addCallback(sort_tracks_title)
+                dfr.addCallback(lambda result_set: result_set.all())
                 dfr.addCallback(sort_tracks_title_client_side)
             elif self.album is not None:
                 # FIXME: in this case we should not display an alphabetical
                 # fastscroller, which does not make sense for a small number of
                 # tracks and will be broken anyway, because the tracks are not
                 # ordered alphabetically. See bug #253061 on Launchpad.
-                dfr = store.find(MusicTrack, MusicTrack.album_name == self.album.name)
+                dfr = store.find(MusicTrack, MusicTrack.album_name == self.album.name,
+                                 MusicTrack.file_path == File.path,
+                                 File.hidden == False)
                 dfr.addCallback(sort_tracks_number_title)
             else:
-                dfr = store.find(MusicTrack, MusicTrack.title != None)
+                dfr = store.find(MusicTrack, MusicTrack.title != None,
+                                 MusicTrack.file_path == File.path,
+                                 File.hidden == False)
                 # Add client side ordering - can this be done "server" side? Is
                 # there a better way to trash spourious characters?
-                dfr.addCallback(sort_tracks_title)
+                dfr.addCallback(lambda result_set: result_set.all())
                 dfr.addCallback(sort_tracks_title_client_side)
 
             dfr.addCallback(got_tracks)
             return dfr
 
-        return deferred.addCallback(get_tracks)
+        def retrieve_tracks(controller):
+            if tracks is None:
+                return album.get_tracks()
+            return tracks
+
+        def insert_tracks(tracks):
+            self.model.extend(tracks)
+            return self
+
+        # retrieve tracks for not-database albums
+        if album and getattr(album, 'uri', None):
+            dfr.addCallback(retrieve_tracks)
+            dfr.addCallback(insert_tracks)
+        # retrieve tracks for database albums
+        else:
+            dfr.addCallback(get_tracks)
+
+        return dfr
 
 
 class DBTracksViewMode(TracksViewMode):
@@ -735,6 +793,9 @@ GENRES = [u"Acid Jazz", u"Alternative", u"Alternative & Punk",
 
 class GenresDbController(HierarchyController):
 
+    empty_label = _('There are no tracks in this section')
+    empty_icon = 'elisa.plugins.poblesec.by_genre'
+
     def initialize(self, all_genres=False):
         self.all_genres = all_genres
 
@@ -769,7 +830,7 @@ class GenresDbController(HierarchyController):
             dfr.addCallback(iterate_done, models)
             return dfr
 
-        def sort_by_title(result_set):
+        def sort_by_genre(result_set):
             result_set.order_by(MusicTrack.genre)
             return result_set
 
@@ -790,25 +851,19 @@ class GenresDbController(HierarchyController):
 
         deferred.addCallback(get_tracks)
         deferred.addCallback(set_distinct)
-        deferred.addCallback(sort_by_title)
+        deferred.addCallback(sort_by_genre)
         deferred.addCallback(get_genres)
         deferred.addCallback(got_genres, all_genres)
         return deferred
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-            self._previous_clicked = selected_item
-            if isinstance(item, Action):
-                item.run()
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = item.controller_path
-                args = item.controller_args
-                dfr = browser.history.append_controller(path, item.label, **args)
+        if isinstance(item, Action):
+            item.run()
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            path = item.controller_path
+            args = item.controller_args
+            dfr = browser.history.append_controller(path, item.label, **args)
 
 
 class GenresDbPreviewListController(GenresDbController, MenuItemPreviewListController):
@@ -827,51 +882,58 @@ class GenresDbListSwitcherController(ListSwitcherController):
     default_mode = GenresDbPreviewListController
 
 
-class GenreDbController(GenericTracksDbController):
-
-    # FIXME: shouldn't we display a list of albums rather than a list of tracks?
+class GenreDbController(GenericAlbumsDbController):
 
     def initialize(self, genre=None):
         self.genre = genre
+        store = common.application.store
 
         deferred = super(GenreDbController, self).initialize()
 
-        def got_tracks(tracks):
-            self.model.extend(tracks)
+        def got_albums(albums):
+            self.model.extend(albums)
             return self
 
-        def sort_by_title(result_set):
-            result_set.order_by(MusicTrack.genre)
+        def sort_by(result_set):
+            result_set.order_by(MusicAlbum.name)
             return result_set.all()
 
-        def get_tracks(self):
-            store = common.application.store
-            if genre:
-                dfr = store.find(MusicTrack, MusicTrack.genre == unicode(genre))
-            else:
-                dfr = store.find(MusicTrack, Not(MusicTrack.genre.is_in(GENRES)))
+        def distinct_all(result_set):
+            result_set.config(distinct=True)
+            return result_set.all()
+
+        def get_albums(results):
+            return store.find(MusicAlbum, MusicAlbum.name.is_in(results))
+
+        def get_album_names(self):
+            dfr = store.find(MusicTrack.album_name,
+                    MusicTrack.genre == unicode(genre), \
+                    MusicTrack.file_path == File.path, \
+                    File.hidden == False)
+            dfr.addCallback(distinct_all)
             return dfr
 
-        deferred.addCallback(get_tracks)
-        deferred.addCallback(sort_by_title)
-        deferred.addCallback(got_tracks)
+        deferred.addCallback(get_album_names)
+        deferred.addCallback(get_albums)
+        deferred.addCallback(sort_by)
+        deferred.addCallback(got_albums)
         return deferred
 
 
-class GenrePreviewListController(GenreDbController, DoubleLineMenuItemPreviewListController):
-    view_mode = DBTracksViewMode
+class GenrePreviewListController(GenreDbController, MenuItemPreviewListController):
+    view_mode = AlbumsDbViewMode
     fastscroller = True
 
     def item_to_label(self, item):
         if isinstance(item, Action):
             return '#'
-        return item.title
+        return item.name
 
 class GenreDbCoverflowController(GenreDbController, ImageWithReflectionCoverflowController):
-    view_mode = TracksViewMode
+    view_mode = AlbumsDbViewMode
 
 class GenreDbGridController(GenreDbController, GridItemGridController):
-    view_mode = TracksViewMode
+    view_mode = AlbumsDbViewMode
 
 class GenreDbListSwitcherController(ListSwitcherController):
     modes = [GenrePreviewListController,
@@ -915,19 +977,13 @@ class DecadesDbController(HierarchyController):
         return deferred
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-            self._previous_clicked = selected_item
-            if isinstance(item, Action):
-                item.run()
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = '/poblesec/database/music/time'
-                args = item.controller_args
-                dfr = browser.history.append_controller(path, item.label, **args)
+        if isinstance(item, Action):
+            item.run()
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            path = '/poblesec/database/music/time'
+            args = item.controller_args
+            dfr = browser.history.append_controller(path, item.label, **args)
 
 
 class DecadesDbPreviewListController(DecadesDbController, MenuItemPreviewListController):
@@ -948,6 +1004,8 @@ class DecadesDbListSwitcherController(ListSwitcherController):
 
 class TimeDbController(GenericAlbumsDbController):
 
+    empty_icon = 'elisa.plugins.poblesec.by_decade'
+
     def initialize(self, begin=None, end=None):
         self.begin = begin
         self.end = end
@@ -962,7 +1020,7 @@ class TimeDbController(GenericAlbumsDbController):
             self.model.extend(albums)
             return self
 
-        def sort_by_title(result_set):
+        def sort_by_name(result_set):
             result_set.order_by(MusicAlbum.name)
             return result_set.all()
 
@@ -978,23 +1036,11 @@ class TimeDbController(GenericAlbumsDbController):
             return self
 
         deferred.addCallback(get_albums)
-        deferred.addCallback(sort_by_title)
+        deferred.addCallback(sort_by_name)
         deferred.addCallback(got_albums)
         deferred.addErrback(missing_data)
         return deferred
 
-    def nodes_setup(self):
-        super(TimeDbController, self).nodes_setup()
-        model = [m for m in self.model if not isinstance(m, Action)]
-        if len(model) == 0:
-            self.model[:] = []
-            msg = _("There are no albums for this decade.")
-            resource = 'elisa.plugins.poblesec.by_decade'
-            img = self.frontend.get_theme().get_resource(resource)
-            self.nothing_to_display_widget.icon.set_from_file(img)
-            self.nothing_to_display_widget.text.label = msg
-            self.nothing_to_display_widget.visible = True
-
 
 class TimeDbPreviewListController(TimeDbController, MenuItemPreviewListController):
     view_mode = AlbumsDbViewMode
