@@ -16,6 +16,7 @@
 #
 # Author: Benjamin Kampmann <benjamin@fluendo.com>
 
+from elisa.core.metadata_manager import IncompleteMetadataResponse
 from elisa.core.utils.i18n import install_translation
 _ = install_translation('database')
 
@@ -25,6 +26,7 @@ from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
 from elisa.plugins.poblesec.base.hierarchy import HierarchyController
 from elisa.plugins.poblesec.link import Link
 from elisa.plugins.base.models.image import ImageModel
+from elisa.plugins.base.models import image as image_model
 
 from elisa.plugins.poblesec.base.preview_list import \
     MenuItemPreviewListController, DoubleLineMenuItemPreviewListController
@@ -34,13 +36,15 @@ from elisa.plugins.poblesec.base.grid import GridItemGridController
 
 from elisa.plugins.poblesec.base.list import  GenericListViewMode
 
-from elisa.plugins.database.models import Image, PhotoAlbum
+from elisa.plugins.database.models import Image, PhotoAlbum, File, \
+                                                             PICTURES_SECTION
 from elisa.core import common
 from elisa.core.media_uri import MediaUri
 
 from elisa.plugins.poblesec.actions import Action
 from elisa.plugins.poblesec.actions import ViewSlideshowAction
 from elisa.plugins.database.actions import PhotoAddToFavoritesAction
+from elisa.plugins.database.actions import PhotoRemoveAction
 from elisa.plugins.database.actions import ViewPhotoAlbumSlideshowAction, \
         ViewPhotoDateSlideshowAction
 
@@ -49,6 +53,8 @@ from elisa.plugins.favorites.models import FavoritesItem
 import os, datetime
 import os.path
 
+from gtk import gdk
+
 
 def photo_lib_decorator(controller):
     link = Link()
@@ -62,7 +68,7 @@ def photo_lib_decorator(controller):
 def photo_lib_albums_decorator(controller):
     albums = Link()
     albums.controller_path = '/poblesec/database/photo/albums'
-    albums.label = _('Browse by Albums')
+    albums.label = _('Albums')
     albums.icon = 'elisa.plugins.poblesec.photo_album'
     controller.model.append(albums)
     return defer.succeed(None)
@@ -71,7 +77,7 @@ def photo_lib_albums_decorator(controller):
 def photo_lib_images_decorator(controller):
     tracks = Link()
     tracks.controller_path = '/poblesec/database/photo/list'
-    tracks.label = _('Browse all Photos')
+    tracks.label = _('Photos')
     tracks.icon = 'elisa.plugins.poblesec.photo'
     controller.model.append(tracks)
     return defer.succeed(None)
@@ -80,7 +86,7 @@ def photo_lib_images_decorator(controller):
 def photo_lib_month_decorator(controller):
     times = Link()
     times.controller_path = '/poblesec/database/photo/by_date'
-    times.label = _('Browse by date')
+    times.label = _('Browse by Date')
     times.icon = 'elisa.plugins.poblesec.by_decade'
     controller.model.append(times)
     return defer.succeed(None)
@@ -95,13 +101,41 @@ class ThumbnailMixin(object):
         frontend = common.application.interface_controller.frontends.values()[0]
         # Retrieve and store a reference to gst_metadata
         controllers = frontend.retrieve_controllers('/poblesec')
-        self.gst_metadata = controllers[0].gst_metadata
+        try:
+            self.gst_metadata = controllers[0].gst_metadata
+        except AttributeError:
+            msg = 'GstMetadata missing: thumbnails will not be generated.'
+            self.warning(msg)
+            self.gst_metadata = None
 
     def _updated_thumbnail(self, thumbnail, item):
+        # apply the image rotation if there's any
+        # FIXME: this is ugly, we get a thumbnail from gst_metadata and
+        #        a new one is generated...
+        if item.orientation:
+            path, ext = os.path.splitext(thumbnail)
+            # FIXME: not compliant with XDG thumbnail spec:
+            rotated_thumbnail = "%s-%s%s" % (path, item.orientation, ext)
+            if not os.path.exists(rotated_thumbnail):
+
+                pixbuf = gdk.pixbuf_new_from_file(thumbnail)
+                pixbuf_rotations = {image_model.ROTATED_0: gdk.PIXBUF_ROTATE_NONE,
+                                    image_model.ROTATED_90_CCW: gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE,
+                                    image_model.ROTATED_180: gdk.PIXBUF_ROTATE_UPSIDEDOWN,
+                                    image_model.ROTATED_90_CW:  gdk.PIXBUF_ROTATE_CLOCKWISE}
+                rotated = pixbuf.rotate_simple(pixbuf_rotations[item.orientation])
+                rotated.save(rotated_thumbnail, ext[1:])
+
+            thumbnail = rotated_thumbnail
+
         item.thumbnail = thumbnail
         return thumbnail
 
     def _request_thumbnail(self, path):
+        if self.gst_metadata is None:
+            msg = 'No gst_metadata.'
+            return defer.fail(IncompleteMetadataResponse(msg))
+
         def got_metadata(metadata):
             return metadata['thumbnail']
 
@@ -145,6 +179,9 @@ class PhotoViewMode(GenericListViewMode, ThumbnailMixin):
 
 class GenericPhotosController(HierarchyController):
 
+    empty_label = _('There are no photos in this section')
+    empty_icon = 'elisa.plugins.poblesec.photo'
+
     def initialize(self, album=None, photo=None, photos=None):
         self.album = album
         self.photo = photo
@@ -154,6 +191,9 @@ class GenericPhotosController(HierarchyController):
         self.actions.extend(self.make_actions())
         return dfr
 
+    def is_empty(self):
+        return super(GenericPhotosController, self).is_empty() and self.photo is None
+
     def make_actions(self):
         actions = []
 
@@ -176,6 +216,7 @@ class GenericPhotosController(HierarchyController):
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
         elif self.photo:
+
             action = ViewSlideshowAction(self)
             action.title = _('View Photo')
             action.subtitle = _("View '%s'") % os.path.basename(self.photo.file_path)
@@ -193,6 +234,17 @@ class GenericPhotosController(HierarchyController):
                                                 FavoritesItem.foreign_class == u'Image')
             dfr.addCallback(lambda rs: rs.all())
             dfr.addCallback(action.setup)
+
+            # FIXME: implement this properly instead of hijacking the app config
+            app_config = common.application.config
+            if int(app_config.get_option('enable_remove',
+                                         'poblesec.main:PoblesecController',
+                                         default='0')):
+                remove_action = PhotoRemoveAction(self)
+                remove_action.title = _('Remove Photo')
+                remove_action.subtitle = _("Remove '%s'") % os.path.basename(self.photo.file_path)
+                actions.append(remove_action)
+
         else:
             action = ViewSlideshowAction(self)
             action.title = _('Play Slideshow')
@@ -202,24 +254,16 @@ class GenericPhotosController(HierarchyController):
         return actions
 
     def node_clicked(self, widget, model):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(model))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(model, Action):
-                model.run()
-            else:
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                photos = filter(lambda p: not isinstance(p, Action), self.model)
-                title = os.path.basename(model.file_path)
-                dfr = browser.history.append_controller('/poblesec/database/photo/photo_actions',
-                                                        title,
-                                                        photo=model,
-                                                        photos=photos)
+        if isinstance(model, Action):
+            model.run()
+        else:
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            photos = filter(lambda p: not isinstance(p, Action), self.model)
+            title = os.path.basename(model.file_path)
+            dfr = browser.history.append_controller('/poblesec/database/photo/photo_actions',
+                                                    title,
+                                                    photo=model,
+                                                    photos=photos)
 
 class PhotosController(GenericPhotosController):
 
@@ -230,9 +274,13 @@ class PhotosController(GenericPhotosController):
             store = common.application.store
 
             if self.album:
-                dfr = store.find(Image, Image.album_name == album.name)
+                dfr = store.find(Image, Image.album_name == album.name,
+                                 Image.file_path == File.path,
+                                 File.hidden == False)
             else:
-                dfr = store.find(Image)
+                dfr = store.find(Image, Image.file_path == File.path,
+                                 File.hidden == False,
+                                 Image.section == PICTURES_SECTION)
 
             return dfr
 
@@ -267,8 +315,12 @@ class PhotosMonthController(GenericPhotosController):
                 # january next year
                 last = datetime.datetime(year+1, 1, 1)
 
-            dfr = store.find(Image, Image.shot_time >= self.date,
-                        Image.shot_time < last)
+            dfr = store.find(Image,
+                             Image.section == PICTURES_SECTION,
+                             Image.shot_time >= self.date,
+                             Image.shot_time < last,
+                             Image.file_path == File.path,
+                             File.hidden == False)
 
             return dfr
 
@@ -339,6 +391,9 @@ class PhotoAlbumViewMode(GenericListViewMode, ThumbnailMixin):
 
 class GenericPhotoAlbumsController(HierarchyController):
 
+    empty_label = _('There are no albums in this section')
+    empty_icon = 'elisa.plugins.poblesec.photo_album'
+
     def initialize(self):
         dfr = super(GenericPhotoAlbumsController, self).initialize()
         self.actions.extend(self.make_actions())
@@ -356,21 +411,13 @@ class GenericPhotoAlbumsController(HierarchyController):
 
      # copy-paste-code again :(
     def node_clicked(self, widget, model):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(model))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(model, Action):
-                model.run()
-            else:
-                controllers = self.frontend.retrieve_controllers('/poblesec/browser')
-                history = controllers[0].history
-                dfr = history.append_controller('/poblesec/database/photo/list',
-                                                        model.name, album=model)
+        if isinstance(model, Action):
+            model.run()
+        else:
+            controllers = self.frontend.retrieve_controllers('/poblesec/browser')
+            history = controllers[0].history
+            dfr = history.append_controller('/poblesec/database/photo/list',
+                                                    model.name, album=model)
 
 class PhotoTimesViewMode(GenericListViewMode):
 
@@ -397,7 +444,13 @@ class PhotoTimesViewMode(GenericListViewMode):
 class PhotosTimesController(HierarchyController):
 
     query = "SELECT DISTINCT STRFTIME('%m %Y', shot_time)" \
-            " from images where shot_time not NULL order by shot_time;"
+            " from images, files where shot_time not NULL" \
+            + (" and images.section = %d" % PICTURES_SECTION) + \
+            " and files.path = images.file_path and files.hidden = 0" \
+            " order by shot_time;"
+
+    empty_label = _('There are no photos in this section')
+    empty_icon = 'elisa.plugins.poblesec.by_decade'
 
     def initialize(self):
         dfr = super(PhotosTimesController, self).initialize()
@@ -441,22 +494,14 @@ class PhotosTimesController(HierarchyController):
 
      # copy-paste-code again :(
     def node_clicked(self, widget, model):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(model))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(model, Action):
-                model.run()
-            else:
-                controllers = self.frontend.retrieve_controllers('/poblesec/browser')
-                history = controllers[0].history
-                dfr = history.append_controller('/poblesec/database/photo/by_month',
-                        model.strftime(_('%B, %Y')), year=model.year, \
-                        month=model.month)
+        if isinstance(model, Action):
+            model.run()
+        else:
+            controllers = self.frontend.retrieve_controllers('/poblesec/browser')
+            history = controllers[0].history
+            dfr = history.append_controller('/poblesec/database/photo/by_month',
+                    model.strftime(_('%B, %Y')), year=model.year, \
+                    month=model.month)
 
 
 
