@@ -20,6 +20,8 @@ from elisa.core import common
 
 from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
 
+from elisa.plugins.poblesec.base.hierarchy import HierarchyController
+
 from elisa.plugins.database.photo_controller import PhotoVerticalWithPreview, \
     PhotoCoverflow, PhotoGrid
 from elisa.plugins.database.photo_controller import GenericPhotosController
@@ -35,9 +37,12 @@ from elisa.plugins.database.music_controller import GenericTracksDbController, \
 
 from elisa.plugins.database.photo_controller import PhotosController, \
                                               PhotoViewMode
+                                              
+from elisa.plugins.database.video_controller import VideoController, \
+                                              VideoViewMode
 from elisa.plugins.poblesec.link import Link
 
-from elisa.plugins.database.models import MusicTrack, File, Image
+from elisa.plugins.database.models import MusicTrack, File, Image, Video
 from storm.expr import Desc
 
 from twisted.internet import defer
@@ -62,18 +67,57 @@ def last_played_tracks_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/music/tracks_played'
     link.label = _('Recently Played Tracks')
-    link.controller_args = {'limit':20,  'order_by' : 'last_played'}
+    link.controller_args = {'limit': 20, 'order_by': 'last_played'}
     link.icon = 'elisa.plugins.poblesec.playlist'
     controller.model.append(link)
 
     return defer.succeed(None)
 
+def recently_added_video_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/database/video/video_added'
+    link.label = _('Recently Added Videos')
+    link.controller_args = {'limit': 20,}
+    link.icon = 'elisa.plugins.poblesec.video_playlist'
+    controller.model.append(link)
+
+    return defer.succeed(None)
+
+def most_played_video_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/database/video/video_played'
+    link.label = _('Most Played Videos')
+    link.controller_args = {'limit': 20,}
+    link.icon = 'elisa.plugins.poblesec.video_playlist'
+    controller.model.append(link)
+
+    return defer.succeed(None)
+
+def last_played_video_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/database/video/video_played'
+    link.label = _('Last Played Videos')
+    link.controller_args = {'limit': 20, 'order_by': 'last_played'}
+    link.icon = 'elisa.plugins.poblesec.video_playlist'
+    controller.model.append(link)
+
+    return defer.succeed(None)
+    
+def unplayed_video_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/database/video/video_unplayed'
+    link.label = _('Unwatched Videos')
+    link.controller_args = {'limit': 20,}
+    link.icon = 'elisa.plugins.poblesec.video_playlist'
+    controller.model.append(link)
+
+    return defer.succeed(None)
 
 def recently_added_tracks_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/music/tracks_added'
     link.label = _('Recently Added Tracks')
-    link.controller_args = {'limit' : 20}
+    link.controller_args = {'limit': 20,}
     link.icon = 'elisa.plugins.poblesec.playlist'
     controller.model.append(link)
 
@@ -88,15 +132,21 @@ def music_playlists_decorator(controller):
 
     return defer.succeed(None)
 
-# FIXME: even though the by_artist Icon is mainly used for the music section we
-# want to have persons silhouette. Later we should replace it but for now we use
-# the same one
+def video_playlists_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/database/video/playlists'
+    link.label = _('Playlists')
+    link.icon = 'elisa.plugins.poblesec.video_playlist'
+    controller.model.append(link)
+
+    return defer.succeed(None)
+
 def recently_added_photos_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/photo/photos_added'
-    link.label = _('Recently added Photos')
-    link.controller_args = {'limit' : 20}
-    link.icon = 'elisa.plugins.poblesec.by_artist'
+    link.label = _('Recently Added Photos')
+    link.controller_args = {'limit': 20}
+    link.icon = 'elisa.plugins.poblesec.photo_playlist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -104,9 +154,9 @@ def recently_added_photos_decorator(controller):
 def last_played_photos_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/photo/photos_played'
-    link.label = _('Recently viewed Photos')
-    link.controller_args = {'limit':20,  'order_by' : 'last_played'}
-    link.icon = 'elisa.plugins.poblesec.by_artist'
+    link.label = _('Recently Viewed Photos')
+    link.controller_args = {'limit': 20,  'order_by' : 'last_played'}
+    link.icon = 'elisa.plugins.poblesec.photo_playlist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -114,9 +164,9 @@ def last_played_photos_decorator(controller):
 def most_played_photos_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/photo/photos_played'
-    link.label = _('Most viewed Photos')
+    link.label = _('Most Viewed Photos')
     link.controller_args = {'limit': 20,}
-    link.icon = 'elisa.plugins.poblesec.by_artist'
+    link.icon = 'elisa.plugins.poblesec.photo_playlist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -124,8 +174,8 @@ def most_played_photos_decorator(controller):
 def pictures_playlists_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/pictures/playlists'
-    link.label = _('Your History')
-    link.icon = 'elisa.plugins.poblesec.by_artist'
+    link.label = _('History')
+    link.icon = 'elisa.plugins.poblesec.photo_playlist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -156,8 +206,8 @@ class PlayedMixin(object):
 
         store = common.application.store
 
-        dfr = store.find(filter, filter.file_path == File.path, \
-                File.playcount != 0)
+        dfr = store.find(filter, filter.file_path == File.path,
+                         File.playcount != 0, File.hidden == False)
 
         if order_by == "last_played":
             dfr.addCallback(sort_by_date)
@@ -191,7 +241,8 @@ class AddedMixin(object):
 
         store = common.application.store
 
-        dfr = store.find(filter, filter.file_path == File.path)
+        dfr = store.find(filter, filter.file_path == File.path,
+                         File.hidden == False)
 
         dfr.addCallback(sort_by_added)
 
@@ -199,7 +250,31 @@ class AddedMixin(object):
         dfr.addCallback(get_items)
         dfr.addCallback(got_items)
         return dfr
+        
+class UnPlayedMixin(object):
+    def _load(self, result, filter, limit):
+        self.filter = filter
 
+        def got_items(models):
+            self.model.extend(models)
+            return self
+
+        def get_items(result_set):
+            return result_set.all()
+
+        def set_limit(result_set, limit):
+            result_set.config(limit=limit)
+            return result_set
+
+        store = common.application.store
+
+        dfr = store.find(filter, filter.file_path == File.path, \
+                File.playcount == 0)
+
+        dfr.addCallback(set_limit, limit)
+        dfr.addCallback(get_items)
+        dfr.addCallback(got_items)
+        return dfr
 
 class PlayedTracksController(GenericTracksDbController, PlayedMixin):
     def initialize(self, order_by=None, limit=10):
@@ -230,6 +305,27 @@ class AddedPhotosController(GenericPhotosController, AddedMixin):
 
         dfr.addCallback(self._load, Image, limit)
         return dfr
+    
+class AddedVideoController(AddedMixin, VideoController):
+    def initialize(self, limit=10):
+        dfr = super(AddedVideoController, self).initialize()
+
+        dfr.addCallback(self._load, Video,limit)
+        return dfr
+
+class PlayedVideoController(PlayedMixin, VideoController):
+     def initialize(self, order_by=None, limit=10):
+        dfr = super(PlayedVideoController, self).initialize()
+
+        dfr.addCallback(self._load, Video, order_by, limit)
+        return dfr
+        
+class UnPlayedVideoController(UnPlayedMixin, VideoController):
+     def initialize(self, limit=10):
+        dfr = super(UnPlayedVideoController, self).initialize()
+
+        dfr.addCallback(self._load, Video, limit)
+        return dfr
 
 # Added Tracks 
 class AddedTracksVerticalWithPreview(AddedTracksController, DoubleLineMenuItemPreviewListController):
@@ -247,6 +343,53 @@ class AddedTracksListSwitcherController(ListSwitcherController):
              AddedTracksGrid]
     default_mode = AddedTracksVerticalWithPreview
 
+# Added Video 
+class AddedVideoVerticalWithPreview(AddedVideoController, MenuItemPreviewListController):
+    view_mode = VideoViewMode
+
+class AddedVideoCoverflow(AddedVideoController, ImageWithReflectionCoverflowController):
+    view_mode = VideoViewMode
+
+class AddedVideoGrid(AddedVideoController, GridItemGridController):
+    view_mode = VideoViewMode
+
+class AddedVideoListSwitcherController(ListSwitcherController):
+    modes = [AddedVideoVerticalWithPreview,
+             AddedVideoCoverflow,
+             AddedVideoGrid]
+    default_mode = AddedVideoVerticalWithPreview
+    
+# Played Video
+class PlayedVideoVerticalWithPreview(PlayedVideoController, MenuItemPreviewListController):
+    view_mode = VideoViewMode
+
+class PlayedVideoCoverflow(PlayedVideoController, ImageWithReflectionCoverflowController):
+    view_mode = VideoViewMode
+
+class PlayedVideoGrid(PlayedVideoController, GridItemGridController):
+    view_mode = VideoViewMode
+
+class PlayedVideoListSwitcherController(ListSwitcherController):
+    modes = [PlayedVideoVerticalWithPreview,
+             PlayedVideoCoverflow,
+             PlayedVideoGrid]
+    default_mode = PlayedVideoVerticalWithPreview
+    
+# UnPlayed Video
+class UnPlayedVideoVerticalWithPreview(UnPlayedVideoController, MenuItemPreviewListController):
+    view_mode = VideoViewMode
+
+class UnPlayedVideoCoverflow(UnPlayedVideoController, ImageWithReflectionCoverflowController):
+    view_mode = VideoViewMode
+
+class UnPlayedVideoGrid(UnPlayedVideoController, GridItemGridController):
+    view_mode = VideoViewMode
+
+class UnPlayedVideoListSwitcherController(ListSwitcherController):
+    modes = [UnPlayedVideoVerticalWithPreview,
+             UnPlayedVideoCoverflow,
+             UnPlayedVideoGrid]
+    default_mode = UnPlayedVideoVerticalWithPreview
 
 # Played Tracks
 class PlayedTracksVerticalWithPreview(PlayedTracksController, DoubleLineMenuItemPreviewListController):
