@@ -38,6 +38,11 @@ setup(name='elisa-plugin-database',
     [elisa.plugins.search.searchers]
     DBSearcher = elisa.plugins.database.searcher:DBSearcher
 
+    [elisa.core.components.resource_provider]
+    MediaScanner = elisa.plugins.database.media_scanner:MediaScanner
+    
+    [elisa.core.components.service_provider]
+    DatabaseDBusServiceProvider = elisa.plugins.database.dbus_service:DatabaseDBusServiceProvider
     """,
     package_data={'': ['*.png', '*.mo', '*.po', '*.conf'],
                   'elisa.plugins.database.i18n': ['*/LC_MESSAGES/*.po', '*.pot'],
@@ -61,6 +66,12 @@ setup(name='elisa-plugin-database',
                           'elisa.plugins.database.music_controller:TimeDbListSwitcherController'),
                          ('/poblesec/database/music/tracks_played',
                           'elisa.plugins.database.playlists_controller:PlayedTracksListSwitcherController'),
+                         ('/poblesec/database/video/video_added',
+                          'elisa.plugins.database.playlists_controller:AddedVideoListSwitcherController'),
+                         ('/poblesec/database/video/video_played',
+                          'elisa.plugins.database.playlists_controller:PlayedVideoListSwitcherController'),
+                         ('/poblesec/database/video/video_unplayed',
+                          'elisa.plugins.database.playlists_controller:UnPlayedVideoListSwitcherController'),
                          ('/poblesec/database/music/tracks_added',
                           'elisa.plugins.database.playlists_controller:AddedTracksListSwitcherController'),
                          ('/poblesec/database/photo/list',
@@ -77,6 +88,14 @@ setup(name='elisa-plugin-database',
                           'elisa.plugins.database.playlists_controller:AddedPhotosListSwitcherController'),
                          ('/poblesec/database/photo/photos_played',
                           'elisa.plugins.database.playlists_controller:PlayedPhotosListSwitcherController'),
+                         ('/poblesec/database/video/list',
+                          'elisa.plugins.database.video_controller:AllVideosListSwitcherController'),
+                         ('/poblesec/database/video/by_month',
+                          'elisa.plugins.database.video_controller:VideosMonthListSwitcherController'),
+                         ('/poblesec/database/video/by_date',
+                          'elisa.plugins.database.video_controller:VideoTimesListSwitcherController'),
+                         ('/poblesec/database/video/track',
+                          'elisa.plugins.database.video_controller:VideoTrackListSwitcherController'),
                          ('/poblesec/database/.*playlists',
                           'elisa.plugins.poblesec.section:SectionMenuController'),
                          ('/poblesec/database/.*library',
@@ -94,6 +113,8 @@ setup(name='elisa-plugin-database',
                         # ordering issue in the menus.
                         ('/poblesec/database/music_library',
                         'elisa.plugins.poblesec.music:filesystem_decorator'),
+                        ('/poblesec/database/video_library',
+                        'elisa.plugins.poblesec.video:filesystem_decorator'),
                         # music library entries
                         ('/poblesec/database/music_library',
                          'elisa.plugins.database.music_controller:music_lib_decades_decorator'),
@@ -116,15 +137,27 @@ setup(name='elisa-plugin-database',
                         ('/poblesec/database/photo_library',
                         'elisa.plugins.poblesec.pictures:filesystem_decorator'),
                         # images
-
                         ('/poblesec/database/photo_library',
                          'elisa.plugins.database.photo_controller:photo_lib_month_decorator'),
                         ('/poblesec/database/photo_library',
                          'elisa.plugins.database.photo_controller:photo_lib_images_decorator'),
                         ('/poblesec/database/photo_library',
                          'elisa.plugins.database.photo_controller:photo_lib_albums_decorator'),
-
-                        # playlists
+                        # videos library
+                        ('/poblesec/database/video_library',
+                         'elisa.plugins.database.video_controller:video_lib_date_decorator'),
+                        ('/poblesec/database/video_library',
+                         'elisa.plugins.database.video_controller:video_lib_all_videos_decorator'),
+                         # videos playlists
+                        ('/poblesec/database/video/playlists',
+                         'elisa.plugins.database.playlists_controller:recently_added_video_decorator'),
+                        ('/poblesec/database/video/playlists',
+                         'elisa.plugins.database.playlists_controller:most_played_video_decorator'),
+                        ('/poblesec/database/video/playlists',
+                         'elisa.plugins.database.playlists_controller:last_played_video_decorator'),
+                        ('/poblesec/database/video/playlists',
+                         'elisa.plugins.database.playlists_controller:unplayed_video_decorator'),
+                        # music playlists
                         ('/poblesec/database/music/playlists',
                          'elisa.plugins.database.playlists_controller:recently_added_tracks_decorator'),
                         ('/poblesec/database/music/playlists',
@@ -138,9 +171,7 @@ setup(name='elisa-plugin-database',
                          'elisa.plugins.database.playlists_controller:most_played_photos_decorator'),
                         ('/poblesec/database/pictures/playlists',
                          'elisa.plugins.database.playlists_controller:last_played_photos_decorator'),
-                       # the scanning widget. Can someone please tell me, that
-                        # there is a reg-exp that makes it work much easier?
-                        ('/poblesec/database/.*',
+                        ('/poblesec/browser',
                          'elisa.plugins.database.scanning_widget:scanning_decorator'),],
     cmdclass=cmdclass,
 )
