@@ -186,6 +186,7 @@ class TestGeneralFileParsing(ParserSetupMixin, TestCase):
             source = Dummy()
             source.files_failed = []
             source.root_uri = dummy.uri
+            source.section = "somewhere"
             return self.parser.query_model(dummy, source)
 
         file = File()
@@ -206,6 +207,7 @@ class TestGeneralFileParsing(ParserSetupMixin, TestCase):
         stat = Dummy()
         stat.files_failed = []
         stat.root_uri = MediaUri('file:///test')
+        stat.section = "something"
         return self.parser.query_model(dummy, stat).addCallback(check)
 
     def test_update_modification_time(self):
@@ -246,6 +248,319 @@ class TestGeneralFileParsing(ParserSetupMixin, TestCase):
         return self.store.add(file).addCallback(mark_as_deleted, u'/tmp'
                 ).addCallback(check_deleted, file)
 
+class TestRemoving(ParserSetupMixin, TestCase):
+    """
+    Test whether the removal of files works properly.
+    """
+
+    def add_items(self, items):
+        def iterator(items):
+            for item in items:
+                yield self.store.add(item)
+        dfr = task.coiterate(iterator(items))
+        return dfr
+
+    def test_simple_removing_per_source(self):
+
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def got_files(files):
+            self.assertEquals(len(files), 0)
+
+        def check(result):
+            dfr = self.store.find(File)
+            dfr.addCallback( lambda x: x.all())
+            dfr.addCallback(got_files)
+            return dfr
+
+        file = File()
+        file.path = u"test"
+        file.source = u"/tmp"
+        file.deleted = 1
+
+        return self.store.add(file).addCallback(remover).addCallback(check)
+
+    def test_remove_only_of_source(self):
+
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def got_files(files):
+            self.assertEquals(files, [u't4'])
+
+        def check(result):
+            dfr = self.store.find(File)
+            dfr.addCallback( lambda x: x.values(File.path))
+            dfr.addCallback(got_files)
+            return dfr
+         
+        files = []
+
+        file1 = File()
+        file1.path = u"test"
+        file1.source = u"/tmp"
+        file1.deleted = 1
+
+        files.append(file1)
+
+        file2 = File()
+        file2.path = u"t2"
+        file2.source = u"/tmp"
+        file2.deleted = 1
+
+        files.append(file2)
+
+        file3 = File()
+        file3.path = u"t3"
+        file3.source = u"/tmp"
+        file3.deleted = 1
+        
+        files.append(file3)
+
+        other_file = File()
+        other_file.path = u"t4"
+        other_file.source = u"yeah"
+        other_file.deleted = 1
+
+        files.append(other_file)
+
+        return self.add_items(files).addCallback(remover).addCallback(check)
+
+    def test_dont_remove_non_marked(self):
+
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def got_files(files):
+            self.assertEquals(files, [u't3'])
+
+        def check(result):
+            dfr = self.store.find(File)
+            dfr.addCallback( lambda x: x.values(File.path))
+            dfr.addCallback(got_files)
+            return dfr
+         
+        files = []
+
+        file1 = File()
+        file1.path = u"test"
+        file1.source = u"/tmp"
+        file1.deleted = 1
+
+        files.append(file1)
+
+        file2 = File()
+        file2.path = u"t2"
+        file2.source = u"/tmp"
+        file2.deleted = 1
+
+        files.append(file2)
+
+        file3 = File()
+        file3.path = u"t3"
+        file3.source = u"/tmp"
+        # NOT MARKED !!!
+        file3.deleted = 0
+        
+        files.append(file3)
+
+        return self.add_items(files).addCallback(remover).addCallback(check)
+
+    def test_remove_related(self):
+ 
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        file1 = File()
+        file1.path = u"test"
+        file1.source = u"/tmp"
+        file1.deleted = 1
+
+        items = [file1]
+
+        klasses =  ((MusicTrack, 0), (Image, 0), (Video, 0))
+
+        for klass, count in klasses:
+            obj = klass()
+            obj.file_path = u"test"
+            items.append(obj)
+
+        dfr = self.add_items(items)
+        dfr.addCallback(remover)
+        dfr.addCallback(self.check_count_for_classes, klasses)
+        return dfr
+
+    def check_count_for_classes(self, result, klasses):
+        def check_count(count, klass, reference):
+            msg = "%s has an unexpected number of entries" % klass
+            self.assertEquals(count, reference, msg)
+
+        def iter_count(klasses):
+            for klass, count in klasses:
+                dfr = self.store.find(klass)
+                dfr.addCallback(lambda x: x.count())
+                dfr.addCallback(check_count, klass, count)
+                yield dfr
+
+        dfr = task.coiterate(iter_count(klasses))
+        return dfr
+
+    def test_remove_related_album_and_artist(self):
+ 
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def set_up():
+
+            file = File()
+            file.path = u"test"
+            file.source = u"/tmp"
+            file.deleted = 1
+            yield self.store.add(file)
+
+            album = MusicAlbum()
+            album.name = u"Super troopers"
+            yield self.store.add(album)
+
+            artist = Artist()
+            artist.name = u"Yuppie"
+            yield self.store.add(artist)
+
+            track = MusicTrack()
+            track.file_path = u"test"
+            track.album_name = u"Super troopers"
+            yield self.store.add(track)
+
+            yield track.artists.add(artist)
+
+            yield self.store.commit()
+
+        dfr = task.coiterate(set_up())
+
+        dfr.addCallback(remover)
+
+        klasses = ((File, 0), (MusicTrack, 0), (MusicAlbum, 0), (Artist, 0))
+        dfr.addCallback(self.check_count_for_classes, klasses)
+        return dfr
+
+    def test_remove_related_album_and_artist_still_tracks(self):
+ 
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def set_up():
+
+            file = File()
+            file.path = u"test"
+            file.source = u"/tmp"
+            file.deleted = 1
+            yield self.store.add(file)
+
+            album = MusicAlbum()
+            album.name = u"Super troopers"
+            yield self.store.add(album)
+
+            artist = Artist()
+            artist.name = u"Yuppie"
+            yield self.store.add(artist)
+
+            track = MusicTrack()
+            track.file_path = u"test"
+            track.album_name = u"Super troopers"
+            yield self.store.add(track)
+
+            yield track.artists.add(artist)
+
+            # this track is not going to be removed, so there is still a track
+            # existing in the album for that artist
+            track2 = MusicTrack()
+            track2.file_path = u"tested2"
+            track2.album_name = u"Super troopers"
+            yield self.store.add(track2)
+
+            yield track2.artists.add(artist)
+
+            yield self.store.commit()
+
+        dfr = task.coiterate(set_up())
+
+        dfr.addCallback(remover)
+
+        klasses = ((MusicTrack, 1),
+                    (File, 0),
+                    (MusicAlbum, 1),
+                    (Artist, 1))
+        dfr.addCallback(self.check_count_for_classes, klasses)
+        return dfr
+
+    def test_remove_related_photo_album(self):
+ 
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def set_up():
+
+            file = File()
+            file.path = u"test"
+            file.source = u"/tmp"
+            file.deleted = 1
+            yield self.store.add(file)
+
+            album = PhotoAlbum()
+            album.name = u"Super troopers"
+            yield self.store.add(album)
+        
+            image = Image()
+            image.file_path = u"test"
+            image.album_name = u"Super troopers"
+            yield self.store.add(image)
+
+            yield self.store.commit()
+
+        dfr = task.coiterate(set_up())
+
+        dfr.addCallback(remover)
+
+        klasses = ((File, 0), (Image, 0), (PhotoAlbum, 0))
+        dfr.addCallback(self.check_count_for_classes, klasses)
+        return dfr
+
+    def test_remove_related_album_still_photos(self):
+ 
+        def remover(result):
+            return self.parser.delete_files(u"/tmp")
+
+        def set_up():
+            file = File()
+            file.path = u"test"
+            file.source = u"/tmp"
+            file.deleted = 1
+            yield self.store.add(file)
+
+            album = PhotoAlbum()
+            album.name = u"Super troopers"
+            yield self.store.add(album)
+        
+            image = Image()
+            image.file_path = u"test"
+            image.album_name = u"Super troopers"
+            yield self.store.add(image)
+
+            image2 = Image()
+            image2.file_path = u"else"
+            image2.album_name = u"Super troopers"
+            yield self.store.add(image2)
+
+            yield self.store.commit()
+
+        dfr = task.coiterate(set_up())
+
+        dfr.addCallback(remover)
+
+        klasses = ((File, 0), (Image, 1), (PhotoAlbum, 1))
+        dfr.addCallback(self.check_count_for_classes, klasses)
+        return dfr
 
 class TestFindingRightTypeParsing(ParserSetupMixin, TestCase):
     """
@@ -280,6 +595,8 @@ class TestFindingRightTypeParsing(ParserSetupMixin, TestCase):
         check whether file_type:video leads to having an video entry
         """
 
+        path = self.ensure_file(u'test')
+
         def check(res):
 
             def got_video(video):
@@ -613,7 +930,7 @@ class TestImageParsing(ParserSetupMixin, TestCase):
                     'gps-longitude' :  '2,10,0E',
                     'file_type' : 'image',
                     }
-            return self.parser.parse_tags(tags, file)
+            return self.parser.parse_tags(tags, file, 'pictures')
 
         file = File()
         file.path = path
@@ -729,7 +1046,7 @@ class TestImageParsing(ParserSetupMixin, TestCase):
 
         def parse_it(ref, file):
             tags = {'file_type' : 'image'}
-            return self.parser.parse_tags(tags, file
+            return self.parser.parse_tags(tags, file, 'pictures'
                     ).addCallback(check, ref, path)
 
         file = File()
@@ -793,7 +1110,7 @@ class TestImageParsing(ParserSetupMixin, TestCase):
 
         def parse_it(res, file):
             tags = {'file_type' : 'image'}
-            return self.parser.parse_tags(tags, file)
+            return self.parser.parse_tags(tags, file, 'pictures')
 
         file = File()
         file.path = path
@@ -825,7 +1142,7 @@ class TestImageParsing(ParserSetupMixin, TestCase):
                         'gps-latitude' : None,
                         'gps-longitude' : None,
                         }
-                return self.parser.parse_tags(tags, file)
+                return self.parser.parse_tags(tags, file, 'pictures')
 
             file = File()
             file.path = path
