@@ -18,9 +18,10 @@
 
 import dbus
 from dbus.exceptions import DBusException
+import exceptions
 from twisted.trial.unittest import TestCase
 from twisted.internet import defer, task, reactor
-from twisted.python.reflect import qual
+from twisted.python.reflect import qual, namedAny
 
 from elisa.core import common
 from elisa.core.media_uri import MediaUri
@@ -28,13 +29,28 @@ from elisa.extern.storm_wrapper import store
 from storm.locals import create_database
 from elisa.plugins.database.dbus_service import DatabaseDBusServiceProvider, \
         ArtistNotFoundError, MusicAlbumNotFoundError, MusicTrackNotFoundError, \
-        PhotoAlbumNotFoundError, PictureNotFoundError
+        PhotoAlbumNotFoundError, PictureNotFoundError, _thumbnail_file, \
+        Music, MusicImpl, Photo, PhotoImpl
 from elisa.plugins.database.media_scanner import SCHEMA
 from elisa.plugins.database.models import Artist, MusicAlbum, MusicTrack, \
-        PhotoAlbum, Image, File
+        PhotoAlbum, Image, File, PlayableModel
 
-def defer_dbus(method, *args, **kw):
+def translateDBusFailure(failure):
+    failure.trap(DBusException)
+
+    skip_len = len('org.freedesktop.DBus.Python.')
+    exception_name = failure.value.get_dbus_name()[skip_len:]
+    
+    exception_class = getattr(exceptions, exception_name, None)
+    if exception_class is None:
+        exception_class = namedAny(exception_name)
+
+    raise exception_class()
+
+def method(*args, **kw):
     dfr = defer.Deferred()
+    dfr.addErrback(translateDBusFailure)
+
     def callback(*args):
         dfr.callback(*args)
     
@@ -76,7 +92,7 @@ class ImageFilePath(unicode):
     def __new__(cls, album_name, name):
         return unicode.__new__(cls, '/%s - %s.png' % (album_name, name))
 
-class TestDatabaseDBusServiceProvider(DatabaseDBusServiceProvider):
+class TestDatabaseDBusServiceProviderBase(DatabaseDBusServiceProvider):
     _last_played_track_tuple = None
     _last_displayed_picture_tuple = None
 
@@ -89,20 +105,117 @@ class TestDatabaseDBusServiceProvider(DatabaseDBusServiceProvider):
     def _emit_last_displayed_picture(self, last_displayed_picture_tuple):
         self._last_displayed_picture_tuple = last_displayed_picture_tuple
 
-class TestDBMixin(object):
+class DeferredDBusMethod(object):
+    def __init__(self, method_name):
+        self.method_name = method_name
+    
+    def __get__(self, instance, klass):
+        method = getattr(instance.dbus_proxy, self.method_name)
+        return self._decorate(method)
+
+    def _decorate(self, method):
+        def wrapper(*args, **kw):
+            dfr = defer.Deferred()
+            dfr.addErrback(translateDBusFailure)
+
+            def callback(*args):
+                args = args or (None,)
+                dfr.callback(*args)
+            
+            def errback(*args):
+                dfr.errback(*args)
+
+            kw['reply_handler'] = callback
+            kw['error_handler'] = errback
+
+            method(*args, **kw)
+
+            return dfr
+
+        return wrapper
+
+class DeferredDBusProxyMeta(type):
+    method_names = []
+
+    def __new__(cls, name, bases, dic):
+        newcls = type.__new__(cls, name, bases, dic)
+
+        for method_name in newcls.method_names:
+            setattr(newcls, method_name, DeferredDBusMethod(method_name))
+
+        return newcls
+
+class DeferredDBusPhotoProxy(object):
+    __metaclass__ = DeferredDBusProxyMeta
+
+    method_names = ['get_albums', 'get_album_pictures',
+            'get_last_displayed_picture']
+
+    def __init__(self, dbus_proxy):
+        self.dbus_proxy = dbus_proxy
+
+class DeferredDBusMusicProxy(object):
+    __metaclass__ = DeferredDBusProxyMeta
+
+    method_names = ['get_albums', 'get_album_artwork',
+            'get_album_tracks', 'get_last_played_track']
+    
+    def __init__(self, dbus_proxy):
+        self.dbus_proxy = dbus_proxy
+
+class TestMusicImpl(MusicImpl):
+    download = None
+
+    def _download_cover_thumbnail(self, thumbnail_uri, filename):
+        self.download = thumbnail_uri
+        return defer.succeed(filename)
+
+class TestMusic(Music):
+    implFactory = TestMusicImpl
+
+class TestPhotoImpl(PhotoImpl):
+    pass
+
+class TestPhoto(Photo):
+    pass
+
+class TestDatabaseDBusServiceProvider(TestDatabaseDBusServiceProviderBase):
+    def _start_dbus(self):
+        bus = dbus.SessionBus()
+        self.photo = TestPhoto(bus, '/com/fluendo/Elisa/Plugins/Database/Photo')
+        self.music = TestMusic(bus, '/com/fluendo/Elisa/Plugins/Database/Music')
+
+class TestDatabaseDBusServiceProviderNoDBus(TestDatabaseDBusServiceProviderBase):
+    """
+    A DBus service provider that doesn't use DBus. Yeah, really.
+
+    And bonus points for winning the longest-class-name-ever award.
+    """
+    def _start_dbus(self):
+        self.photo = TestPhotoImpl()
+        self.music = TestMusicImpl()
+
+    def _stop_dbus(self):
+        pass
+
+class TestMixin(object):
+    test_dbus = True
+
     def setUp(self):
         # create and start the service 
         def service_created_cb(service):
             self.service = service
             return service.start()
 
-        def create_dbus_proxy(service):
-            bus = dbus.SessionBus()
-            self.proxy = bus.get_object(self.connection, self.object_path)
+        if self.test_dbus:
+            service_provider_class = TestDatabaseDBusServiceProvider
+        else:
+            service_provider_class = TestDatabaseDBusServiceProviderNoDBus
 
-        dfr1 = TestDatabaseDBusServiceProvider.create({})
+        dfr1 = service_provider_class.create({})
         dfr1.addCallback(service_created_cb)
-        dfr1.addCallback(create_dbus_proxy)
+        dfr1.addCallback(self._get_proxy)
+        dfr1.addCallback(lambda proxy: setattr(self, 'proxy', proxy))
 
         # create and start the database
         self.db = create_database('sqlite:')
@@ -133,36 +246,12 @@ class TestDBMixin(object):
     def unpatch_application(self):
         common.application = self.application
 
-    def failUnlessDBusException(self, dfr, *expected_failures):
-        expected_failure_names = []
-        for expected_failure in expected_failures:
-            name = qual(expected_failure)
-            if name.startswith('exceptions.'):
-                name = name[11:]
-
-            name = 'org.freedesktop.DBus.Python.' + name
-            expected_failure_names.append(name)
-
-        def _cb(ignore):
-            raise self.failureException(
-                "did not catch an error, instead got %r" % (ignore,))
-
-        def _eb(failure):
-            if failure.check(DBusException) and \
-                failure.value.get_dbus_name() in expected_failure_names:
-                return failure.value
-            else:
-                output = ('\nExpected DBusException: %r\nGot:\n%s'
-                          % (expected_failure_names, str(failure)))
-                raise self.failureException(output)
-        return dfr.addCallbacks(_cb, _eb)
-
-class TestMusic(TestDBMixin, TestCase):
+class TestMusicMixin(TestMixin):
     number_artists = 3
     number_albums = 5
     number_tracks = 16
 
-    connection = 'com.fluendo.Elisa.Plugins.Database'
+    connection = 'com.fluendo.Elisa'
     object_path = '/com/fluendo/Elisa/Plugins/Database/Music'
 
     def _populate_database(self, result):
@@ -207,16 +296,16 @@ class TestMusic(TestDBMixin, TestCase):
     def test_get_albums_wrong_arguments(self):
         deferreds = []
 
-        dfr = defer_dbus(self.proxy.get_albums, 0, 0)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(0, 0)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
 
-        dfr = defer_dbus(self.proxy.get_albums, -1, 10)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(-1, 10)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
         
-        dfr = defer_dbus(self.proxy.get_albums, 1, -1)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(1, -1)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
 
         return defer.DeferredList(deferreds)
@@ -228,7 +317,7 @@ class TestMusic(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), 10)
 
-            dfr = defer_dbus(self.proxy.get_albums, 0, 10)
+            dfr = self.proxy.get_albums(0, 10)
             dfr.addCallback(get_albums_cb)
         
             return dfr
@@ -237,7 +326,7 @@ class TestMusic(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), total_albums)
 
-            dfr = defer_dbus(self.proxy.get_albums, 0, total_albums + 3)
+            dfr = self.proxy.get_albums(0, total_albums + 3)
             dfr.addCallback(get_albums_cb)
             
             return dfr
@@ -246,7 +335,7 @@ class TestMusic(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), 7)
 
-            dfr = defer_dbus(self.proxy.get_albums, total_albums - 7, 18)
+            dfr = self.proxy.get_albums(total_albums - 7, 18)
             dfr.addCallback(get_albums_cb)
             
             return dfr
@@ -266,15 +355,15 @@ class TestMusic(TestDBMixin, TestCase):
         cover_uri = MusicAlbumCoverUri(album_name) 
 
         def test_wrong_artist_name(result):
-            dfr = defer_dbus(self.proxy.get_album_artwork,
-                    'not in the library', album_name)
-            self.failUnlessDBusException(dfr, ArtistNotFoundError)
+            dfr = self.proxy.get_album_artwork(
+                    u'not in the library', album_name)
+            self.failUnlessFailure(dfr, ArtistNotFoundError)
             return dfr
 
         def test_wrong_album_name(result):
-            dfr = defer_dbus(self.proxy.get_album_artwork,
-                    artist_name, 'not in the library')
-            self.failUnlessDBusException(dfr, MusicAlbumNotFoundError)
+            dfr = self.proxy.get_album_artwork(
+                    artist_name, u'not in the library')
+            self.failUnlessFailure(dfr, MusicAlbumNotFoundError)
             return dfr
 
         dfr = defer.Deferred()
@@ -287,7 +376,8 @@ class TestMusic(TestDBMixin, TestCase):
     def test_get_album_artwork(self):
         def get_album_artwork_cb(result, album_name):
             cover_uri = MusicAlbumCoverUri(album_name)
-            self.failUnlessEqual(result, cover_uri)
+            filename = _thumbnail_file(cover_uri)
+            self.failUnlessEqual(result, filename)
         
         def iterator():
             for n_artist in xrange(self.number_artists):
@@ -296,7 +386,7 @@ class TestMusic(TestDBMixin, TestCase):
                 for n_album in xrange(self.number_albums):
                     album_name = MusicAlbumName(artist_name, n_album)
 
-                    dfr = defer_dbus(self.proxy.get_album_artwork,
+                    dfr = self.proxy.get_album_artwork(
                             artist_name, album_name)
                     dfr.addCallback(get_album_artwork_cb, album_name)
 
@@ -311,15 +401,15 @@ class TestMusic(TestDBMixin, TestCase):
         cover_uri = MusicAlbumCoverUri(album_name) 
 
         def test_wrong_artist_name(result):
-            dfr = defer_dbus(self.proxy.get_album_tracks,
-                    'not in the library', album_name)
-            self.failUnlessDBusException(dfr, ArtistNotFoundError)
+            dfr = self.proxy.get_album_tracks(
+                    u'not in the library', album_name)
+            self.failUnlessFailure(dfr, ArtistNotFoundError)
             return dfr
 
         def test_wrong_album_name(result):
-            dfr = defer_dbus(self.proxy.get_album_tracks,
-                    artist_name, 'not in the library')
-            self.failUnlessDBusException(dfr, MusicAlbumNotFoundError)
+            dfr = self.proxy.get_album_tracks(
+                    artist_name, u'not in the library')
+            self.failUnlessFailure(dfr, MusicAlbumNotFoundError)
             return dfr
 
         dfr = defer.Deferred()
@@ -349,7 +439,7 @@ class TestMusic(TestDBMixin, TestCase):
                 for n_album in xrange(self.number_albums):
                     album_name = MusicAlbumName(artist_name, n_album)
 
-                    dfr = defer_dbus(self.proxy.get_album_tracks,
+                    dfr = self.proxy.get_album_tracks(
                             artist_name, album_name)
                     dfr.addCallback(get_album_tracks_cb,
                             artist_name, album_name)
@@ -366,14 +456,15 @@ class TestMusic(TestDBMixin, TestCase):
             artist_name, album_name, title, path = last_played_tuple
             self.failUnlessEqual(self._last_played_track_path, path)
 
-        dfr = defer_dbus(self.proxy.get_last_played_track)
+        dfr = self.proxy.get_last_played_track()
         dfr.addCallback(check_last_played)
         return dfr
 
     def test_last_played_track_signal(self):
         # create a fake PlayableModel
-        class Model(object):
+        class Model(PlayableModel):
             def __init__(self, filename):
+                PlayableModel.__init__(self)
                 self.uri = MediaUri('file://' + filename)
 
         # create a fake Player
@@ -429,11 +520,11 @@ class TestMusic(TestDBMixin, TestCase):
 
         return dfr
 
-class TestPhoto(TestDBMixin, TestCase):
+class TestPhotoMixin(TestMixin):
     number_albums = 12
     number_images = 17
 
-    connection = 'com.fluendo.Elisa.Plugins.Database'
+    connection = 'com.fluendo.Elisa'
     object_path = '/com/fluendo/Elisa/Plugins/Database/Photo'
 
     def _populate_database(self, result):
@@ -468,20 +559,20 @@ class TestPhoto(TestDBMixin, TestCase):
 
         dfr = task.coiterate(iterator())
         return dfr
-
+    
     def test_get_albums_wrong_arguments(self):
         deferreds = []
 
-        dfr = defer_dbus(self.proxy.get_albums, 0, 0)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(0, 0)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
 
-        dfr = defer_dbus(self.proxy.get_albums, -1, 10)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(-1, 10)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
         
-        dfr = defer_dbus(self.proxy.get_albums, 1, -1)
-        self.failUnlessDBusException(dfr, IndexError)
+        dfr = self.proxy.get_albums(1, -1)
+        self.failUnlessFailure(dfr, IndexError)
         deferreds.append(dfr)
 
         return defer.DeferredList(deferreds)
@@ -493,7 +584,7 @@ class TestPhoto(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), 10)
 
-            dfr = defer_dbus(self.proxy.get_albums, 0, 10)
+            dfr = self.proxy.get_albums(0, 10)
             dfr.addCallback(get_albums_cb)
         
             return dfr
@@ -502,7 +593,7 @@ class TestPhoto(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), total_albums)
 
-            dfr = defer_dbus(self.proxy.get_albums, 0, total_albums + 3)
+            dfr = self.proxy.get_albums(0, total_albums + 3)
             dfr.addCallback(get_albums_cb)
             
             return dfr
@@ -511,7 +602,7 @@ class TestPhoto(TestDBMixin, TestCase):
             def get_albums_cb(albums):
                 self.failUnlessEqual(len(albums), 7)
 
-            dfr = defer_dbus(self.proxy.get_albums, total_albums - 7, 18)
+            dfr = self.proxy.get_albums(total_albums - 7, 18)
             dfr.addCallback(get_albums_cb)
             
             return dfr
@@ -527,9 +618,8 @@ class TestPhoto(TestDBMixin, TestCase):
     
     def test_get_album_pictures_wrong_arguments(self):
         def test_wrong_album(result):
-            dfr = defer_dbus(self.proxy.get_album_pictures,
-                    'not in the library')
-            self.failUnlessDBusException(dfr, PhotoAlbumNotFoundError)
+            dfr = self.proxy.get_album_pictures(u'not in the library')
+            self.failUnlessFailure(dfr, PhotoAlbumNotFoundError)
             return dfr
 
         dfr = defer.Deferred()
@@ -546,7 +636,7 @@ class TestPhoto(TestDBMixin, TestCase):
                 def get_album_pictures_cb(result):
                     self.failUnlessEqual(len(result), self.number_images)
 
-                dfr = defer_dbus(self.proxy.get_album_pictures, album_name)
+                dfr = self.proxy.get_album_pictures(album_name)
                 dfr.addCallback(get_album_pictures_cb)
 
                 yield dfr
@@ -561,7 +651,7 @@ class TestPhoto(TestDBMixin, TestCase):
             album_name, path = last_played_tuple
             self.failUnlessEqual(self._last_displayed_picture_path, path)
 
-        dfr = defer_dbus(self.proxy.get_last_displayed_picture)
+        dfr = self.proxy.get_last_displayed_picture()
         dfr.addCallback(check_last_played)
         return dfr
 
@@ -608,3 +698,29 @@ class TestPhoto(TestDBMixin, TestCase):
         dfr.callback(None)
 
         return dfr
+
+class TestMusicDBus(TestMusicMixin, TestCase):
+    def _get_proxy(self, service):
+        bus = dbus.SessionBus()
+        proxy = bus.get_object(self.connection, 
+                '/com/fluendo/Elisa/Plugins/Database/Music')
+        return DeferredDBusMusicProxy(proxy)
+
+class TestPhotoDBus(TestPhotoMixin, TestCase):
+    def _get_proxy(self, service):
+        bus = dbus.SessionBus()
+        proxy = bus.get_object(self.connection, 
+                '/com/fluendo/Elisa/Plugins/Database/Photo')
+        return DeferredDBusPhotoProxy(proxy)
+
+class TestMusicNoDBus(TestMusicMixin, TestCase):
+    test_dbus = False
+
+    def _get_proxy(self, service):
+        return service.music
+
+class TestPhotoNoDBus(TestPhotoMixin, TestCase):
+    test_dbus = False
+    
+    def _get_proxy(self, service):
+        return service.photo
