@@ -91,13 +91,15 @@ class FakeResourceManager(object):
         return (context, defer.succeed(context))
 
 class Application(object):
-    pass 
+    pass
 
 class DummyParser(object):
     def query_model(self, *args):
         return defer.succeed(None)
     def mark_deleted(self, *args):
         return defer.succeed(None)
+    def delete_files(self, *args):
+        return defer.succeed(None)
 
 class DummyResult(object):
     result = None
@@ -115,14 +117,26 @@ class DummyStore(object):
     def commit(self, *args):
         return defer.succeed(True)
 
+class DummyConfig(object):
+    def get_section(self, name):
+        return {}
+
 class TestMediaScanner(TestCase):
     """
     Test the media scanning part
     """
     stat_uri = MediaUri('media_scanner://localhost/statistic')
     queue_uri = MediaUri('media_scanner://localhost/queue')
+    # rescan after ~ 4 secs
+    config = {'delay' : 0.2, 'scan_every' : 0.001}
 
     def setUp(self):
+        self._dbus_init = MediaScanner._initialize_dbus
+        self._dbus_clean = MediaScanner._clean_dbus
+
+        MediaScanner._initialize_dbus = lambda x,y :x
+        MediaScanner._clean_dbus = lambda x :x
+
         def set_scanner(result):
             self.scanner = result
             # overwrite the Parser and stop the old one
@@ -133,7 +147,7 @@ class TestMediaScanner(TestCase):
             return parser.clean()
 
         def create(result):
-            return MediaScanner.create({'delay':0.2}).addCallback(set_scanner)
+            return MediaScanner.create(self.config).addCallback(set_scanner)
     
         return self.patch_application().addCallback(create)
 
@@ -142,11 +156,20 @@ class TestMediaScanner(TestCase):
         common.application = Application()
 #        common.application.resource_manager = FakeResourceManager()
         common.application.store = DummyStore()
+        common.application.config = DummyConfig()
         return defer.succeed(self)
 
     def tearDown(self):
+        def reset_dbus(result):
+            MediaScanner._initialize_dbus = self._dbus_init
+            MediaScanner._clean_dbus = self._dbus_clean
+            return result
+
         common.application = self.app
-        return self.scanner.clean()
+
+        dfr  = self.scanner.clean()
+        dfr.addCallback(reset_dbus)
+        return dfr
 
     def test_get_stats(self):
         model, dfr = self.scanner.get(MediaUri(self.stat_uri))
@@ -162,24 +185,77 @@ class TestMediaScanner(TestCase):
             self.assertFalse(self.scanner.running)
             self.assertFalse(stat.running)
 
-            # start the asnyc waiter
-            dfr = defer.Deferred()
-            async_wait(0, dfr)
-            return dfr
-
-        def async_wait(counter, dfr):
-            if len(s.pending_dfrs):
-                if counter == 5:
-                    self.assertFalse(True, "Even after 5 Seconds it is peding")
-                return reactor.callLater(1, async_wait, counter+1, dfr)
-
             self.assertEquals(s.files_scanned, 5)
             self.assertEquals(s.files_failed, [])
-
-            dfr.callback(True)
+            return dfr
 
         return self.scanner._scan_recursive(s).addCallback(check_result)
 
+    def test_scan_auto_restart(self):
+
+        # we don't really want to scan. we just emulate it
+        wait_dfr = defer.Deferred()
+        self.scanner._scan_recursive = lambda x: wait_dfr
+
+        # fake the _rescan method
+        self._rescan_called = False
+        def rescan(*args):
+            self._rescan_called = True
+
+        self.scanner._rescan = rescan
+
+        # let's assume, we are the last one to scan
+        while not self.scanner.scan_stat.queue.empty():
+            self.scanner.scan_stat.queue.get()
+
+        s = ScanResource()
+        s.root_uri = MediaUri("file:///root/")
+        
+        self.scanner.scan_stat.queue.put(s)
+            
+        # start the 'scanning' process
+        self.scanner._scan()
+
+        scan_dfr = self.scanner._current_scan_deferred
+        wait_dfr.callback(s)
+
+        def check(result):
+            self.assertTrue(self._rescan_called)
+
+        def wait_for_second_scan(result):
+            # rescan was not called yet!
+            self.assertFalse(self._rescan_called)
+
+            dfr = defer.Deferred()
+            dfr.addCallback(check)
+
+            reactor.callLater(5, dfr.callback, None)
+
+            return dfr
+
+        scan_dfr.addCallback(wait_for_second_scan)
+        return scan_dfr
+    
+    test_scan_auto_restart.timeout = 10
+
+    def test_reschedule(self):
+        scan_stat = self.scanner.scan_stat
+        for i in xrange(5):
+            scan_stat.scanned.append('Source %s' % i)
+
+        self.assertTrue(scan_stat.queue.empty())
+
+        self.scanner._reschedule_scanned()
+
+        # scanned is cleared
+        self.assertEquals(scan_stat.scanned, [])
+
+        for i in xrange(5):
+            self.assertEquals(scan_stat.queue.get_nowait(), "Source %s" % i)
+        
+        self.assertTrue(scan_stat.queue.empty())
+        
+
     def test_put_deleting(self):
         """
         Try to put some requests into the media scanner and check if it works
@@ -190,19 +266,21 @@ class TestMediaScanner(TestCase):
         # processes
         self.scanner._file_found = lambda model, stat: defer.succeed(model)
         self.scanner._count_files = lambda path: defer.succeed(19)
+        self.scanner._scan = lambda: None
 
         stat, dfr = self.scanner.get(self.stat_uri)
 
         # overwrite the mark_deleted method
         self.scanner.parser.mark_deleted = lambda x: defer.succeed(marked.append(x))
+        dfrs = []
         for name in ('root', 'usr', 'var'):
-            dfr = self.scanner.put(MediaUri('file:///%s' % name), self.queue_uri)
+            dfrs.append(self.scanner.put(MediaUri('file:///%s' % name),
+                    self.queue_uri))
 
         # as soon as they are put, they should be marked as deleted
         self.assertEquals(marked, [u'/root/', u'/usr/', u'/var/'])
 
-        self.assertEquals(stat.currently_scanning.root_uri.path, u'/root/')
-        return dfr
+    test_put_deleting.timeout = 4
 
 
 class TestFileCounting(TestCase):
