@@ -14,4 +14,8 @@ setup(name='elisa-plugin-discogs',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    DiscogsResource = elisa.plugins.discogs.discogs_resource:DiscogsResource
+    """,
     cmdclass=cmdclass)
