@@ -245,6 +245,7 @@ class DvdPlayerController(PlayerController):
     MENU_KEY = 'm'
 
     log_category = "dvd_player"
+    default_config = {}
 
     def __init__(self):
         super(DvdPlayerController, self).__init__()
@@ -254,6 +255,11 @@ class DvdPlayerController(PlayerController):
 
         self.in_menu = False
 
+    def initialize(self):
+        dfr = super(PlayerController, self).initialize()
+        self.player_osd.time_before_hiding = 3
+        return dfr
+
     def _enter_menu(self):
         self.debug("enter menu")
         self.in_menu = True
