@@ -18,5 +18,7 @@ setup(name='elisa-plugin-elisa-updater',
     package_dir=package_dir,
     package_data={'': ['*.png', '*.mo', '*.po']},
     entry_points="""\
+    [elisa.core.components.resource_provider]
+    UpdaterResourceProvider = elisa.plugins.elisa_updater.resource_provider:UpdaterResourceProvider
     """,
     cmdclass=cmdclass)
