@@ -18,6 +18,7 @@
 
 
 from elisa.core.common import application
+from elisa.core.resource_manager import NoMatchingResourceProvider
 
 from elisa.core.media_uri import MediaUri
 from elisa.plugins.poblesec.link import Link
@@ -39,6 +40,9 @@ from elisa.plugins.database.photo_controller import GenericPhotoAlbumsController
 
 from elisa.plugins.database.photo_controller import PhotoViewMode, \
                                                     PhotoAlbumViewMode
+                                                    
+from elisa.plugins.database.video_controller import VideoViewMode, \
+                                                    VideoController
 
 from elisa.plugins.poblesec.base.preview_list import \
     MenuItemPreviewListController, DoubleLineMenuItemPreviewListController
@@ -80,8 +84,8 @@ def music_decorator(controller):
 def music_internet_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/music/internet/favorites'
-    link.label = _('Favorites')
-    link.icon = 'elisa.plugins.favorites.icon'
+    link.label = _('Radios')
+    link.icon = 'elisa.plugins.poblesec.radio'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -95,7 +99,16 @@ def pictures_decorator(controller):
     controller.model.append(link)
 
     return defer.succeed(None)
+    
+def videos_decorator(controller):
+    link = Link()
+    link.controller_path = '/poblesec/videos/favorites/videos'
+    link.controller_args = {} 
+    link.label = _('Favorites')
+    link.icon = 'elisa.plugins.favorites.icon'
+    controller.model.append(link)
 
+    return defer.succeed(None)
 
 class FavoritesController(SectionMenuController):
 
@@ -105,14 +118,6 @@ class FavoritesController(SectionMenuController):
         dfr = super(FavoritesController, self).initialize()
 
         if section == '/music':
-            link = Link()
-            link.controller_path = '/poblesec/music/favorites/tracks'
-            link.controller_args = {}
-            link.label = _('Tracks')
-            link.icon = 'elisa.plugins.poblesec.by_track'
-
-            self.model.append(link)
-
             link = Link()
             link.controller_path = '/poblesec/music/favorites/artists'
             link.controller_args = {}
@@ -128,6 +133,15 @@ class FavoritesController(SectionMenuController):
             link.icon = 'elisa.plugins.poblesec.by_album'
 
             self.model.append(link)
+
+            link = Link()
+            link.controller_path = '/poblesec/music/favorites/tracks'
+            link.controller_args = {}
+            link.label = _('Tracks')
+            link.icon = 'elisa.plugins.poblesec.by_track'
+
+            self.model.append(link)
+
         elif section == '/pictures':
             link = Link()
             link.controller_path = '/poblesec/pictures/favorites/albums'
@@ -143,6 +157,14 @@ class FavoritesController(SectionMenuController):
             link.label = _('Photos')
             link.icon = 'elisa.plugins.poblesec.photo'
 
+            self.model.append(link)
+        elif section == '/videos':
+            link = Link()
+            link.controller_path = '/poblesec/videos/favorites/videos'
+            link.controller_args = {}
+            link.label = _('Videos')
+            link.icon = 'elisa.plugins.poblesec.file_video'
+
             self.model.append(link)
 
         return dfr
@@ -164,27 +186,80 @@ def focus_refresh_cb(widget, focus, controller):
 
 class FavoritesAlbumsController(GenericAlbumsDbController):
 
+    empty_label = _('There are no favorite albums')
+    empty_icon = 'elisa.plugins.poblesec.by_album'
+
     def initialize(self, artist=None):
         self.artist = artist
 
         dfr = super(FavoritesAlbumsController, self).initialize()
 
-        def get_albums(result):
+        def get_db_albums(result):
             subselect = Select(FavoritesItem.foreign_id,
                                FavoritesItem.foreign_class == u'MusicAlbum')
-            return application.store.find(MusicAlbum, MusicAlbum.name.is_in(subselect))
+            dfr = application.store.find(MusicAlbum, MusicAlbum.name.is_in(subselect))
+            dfr.addCallback(lambda rs: rs.all())
+            return dfr
 
-        def got_albums(albums):
+        def got_db_albums(albums):
             self.model.extend(albums)
             return self
 
-        def sort_albums(result_set):
-            result_set.order_by(MusicAlbum.name)
-            return result_set.all()
+        def get_other_albums(result):
+            dfr = application.store.find(FavoritesItem,
+                                         FavoritesItem.type == u'music_album',
+                                         FavoritesItem.uri != None)
+            dfr.addCallback(lambda rs: rs.all())
+            return dfr
 
-        dfr.addCallback(get_albums)
+        def got_other_albums(items):
+            # FIXME: activate me and reimplement insert() when the model itself
+            # implements sorted insertion
+            # Asynchronously inserting items in the model: it isn't necessary to
+            # wait for completion
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result, i.title)
+
+                dfr.addCallback(callback)
+
+        def got_other_albums_waiting(items):
+            dfrs = []
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result, i.title)
+                dfr.addCallback(callback)
+                dfrs.append(dfr)
+
+            return defer.DeferredList(dfrs)
+
+        def insert(item, title):
+            self.model.append(item)
+
+        def sort_albums(results):
+            def key(item):
+                if isinstance(item, Action):
+                    return ''
+                return item.name.lower()
+
+            self.model.sort(key=key)
+            return self
+
+        dfr.addCallback(get_db_albums)
+        dfr.addCallback(got_db_albums)
+        dfr.addCallback(get_other_albums)
+        dfr.addCallback(got_other_albums_waiting)
         dfr.addCallback(sort_albums)
-        dfr.addCallback(got_albums)
 
         if not getattr(self, '_focus_refresh_cb_handle', None):
             self._focus_refresh_cb_handle = self.widget.connect('focus', focus_refresh_cb, self)
@@ -194,25 +269,81 @@ class FavoritesAlbumsController(GenericAlbumsDbController):
 
 class FavoritesArtistsController(GenericArtistsController):
 
+    empty_label = _('There are no favorite artists')
+    empty_icon = 'elisa.plugins.poblesec.by_artist'
+
     def initialize(self):
         dfr = super(FavoritesArtistsController, self).initialize()
 
-        def get_artists(result):
+        def get_db_artists(result):
             subselect = Select(FavoritesItem.foreign_id,
                                FavoritesItem.foreign_class == u'Artist')
             return application.store.find(Artist, Artist.name.is_in(subselect))
 
-        def got_artists(artists):
+        def got_db_artists(artists):
             self.model.extend(artists)
             return self
 
-        def sort_artists(result_set):
+        def sort_db_artists(result_set):
             result_set.order_by(Artist.name)
             return result_set.all()
 
-        dfr.addCallback(get_artists)
+        def get_other_artists(result):
+            dfr = application.store.find(FavoritesItem,
+                                         FavoritesItem.type == u'artist',
+                                         FavoritesItem.uri != None)
+            dfr.addCallback(lambda rs: rs.all())
+            return dfr
+
+        def got_other_artists(items):
+            # FIXME: activate me and reimplement insert() when the model itself
+            # implements sorted insertion
+            # Asynchronously inserting items in the model: it isn't necessary to
+            # wait for completion
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result)
+                dfr.addCallback(callback)
+
+        def got_other_artists_waiting(items):
+            dfrs = []
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result)
+                dfr.addCallback(callback)
+
+                dfrs.append(dfr)
+
+            return defer.DeferredList(dfrs)
+
+        def insert(item):
+            self.model.append(item)
+
+        def sort_artists(results):
+            def key(item):
+                if isinstance(item, Action):
+                    return ''
+                return (item.name or '').lower()
+
+            self.model.sort(key=key)
+            return self
+
+        dfr.addCallback(get_db_artists)
+        dfr.addCallback(sort_db_artists)
+        dfr.addCallback(got_db_artists)
+        dfr.addCallback(get_other_artists)
+        dfr.addCallback(got_other_artists_waiting)
         dfr.addCallback(sort_artists)
-        dfr.addCallback(got_artists)
 
         if not getattr(self, '_focus_refresh_cb_handle', None):
             self._focus_refresh_cb_handle = self.widget.connect('focus', focus_refresh_cb, self)
@@ -222,29 +353,89 @@ class FavoritesArtistsController(GenericArtistsController):
 
 class FavoritesTracksController(GenericTracksDbController):
 
-    def initialize(self, album=None, track=None):
+    empty_label = _('There are no favorite tracks')
+    empty_icon = 'elisa.plugins.poblesec.file_music'
+
+    def initialize(self, album=None, track=None, tracks=None):
         self.album = album
         self.track = track
+        self.tracks = tracks
 
         dfr = super(FavoritesTracksController, self).initialize()
 
-        def get_tracks(result):
+        def get_db_tracks(result):
             subselect = Select(FavoritesItem.foreign_id,
                                FavoritesItem.foreign_class == u'MusicTrack')
-            return application.store.find(MusicTrack, MusicTrack.file_path.is_in(subselect))
+            return application.store.find(MusicTrack,
+                                          MusicTrack.file_path.is_in(subselect),
+                                          MusicTrack.file_path == File.path,
+                                          File.hidden == False)
 
-        def got_tracks(tracks):
+        def got_db_tracks(tracks):
             self.model.extend(tracks)
             return self
 
-        def sort_tracks_title(result_set):
+        def sort_db_tracks_title(result_set):
             result_set.order_by(MusicTrack.title)
             return result_set.all()
 
+        def get_other_tracks(result):
+            dfr = application.store.find(FavoritesItem,
+                                         FavoritesItem.type == u'music_track',
+                                         FavoritesItem.uri != None)
+            dfr.addCallback(lambda rs: rs.all())
+            return dfr
+
+        def got_other_tracks(items):
+            # FIXME: activate me and reimplement insert() when the model itself
+            # implements sorted insertion
+            # Asynchronously inserting items in the model: it isn't necessary to
+            # wait for completion
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result)
+                dfr.addCallback(callback)
+
+        def got_other_tracks_waiting(items):
+            dfrs = []
+            for item in items:
+                try:
+                    model, dfr = application.resource_manager.get(MediaUri(item.uri))
+                except NoMatchingResourceProvider:
+                    return
+
+                def callback(result, i=item):
+                    insert(result)
+                dfr.addCallback(callback)
+
+                dfrs.append(dfr)
+
+            return defer.DeferredList(dfrs)
+
+        def insert(item):
+            self.model.append(item)
+
+        def sort_tracks(results):
+            def key(item):
+                if isinstance(item, Action):
+                    return ''
+                return (item.title or '').lower()
+
+            self.model.sort(key=key)
+            return self
+
         if self.track is None:
-            dfr.addCallback(get_tracks)
-            dfr.addCallback(sort_tracks_title)
-            dfr.addCallback(got_tracks)
+            dfr.addCallback(get_db_tracks)
+            dfr.addCallback(sort_db_tracks_title)
+            dfr.addCallback(got_db_tracks)
+            dfr.addCallback(get_other_tracks)
+            dfr.addCallback(got_other_tracks_waiting)
+            dfr.addCallback(sort_tracks)
 
         if not getattr(self, '_focus_refresh_cb_handle', None):
             self._focus_refresh_cb_handle = self.widget.connect('focus', focus_refresh_cb, self)
@@ -253,6 +444,9 @@ class FavoritesTracksController(GenericTracksDbController):
 
 class FavoritesRadiosController(GenericShoutcastController):
 
+    empty_label = _('There are no favorite radios')
+    empty_icon = 'elisa.plugins.poblesec.radio'
+
     def initialize(self, uri=None, station=None):
         dfr = super(FavoritesRadiosController, self).initialize(uri=uri, station=station)
 
@@ -369,13 +563,18 @@ class FavoritesRadiosListSwitcher(ListSwitcherController):
 
 class FavoritesPhotosController(GenericPhotosController):
 
+    empty_label = _('There are no favorite photos')
+    empty_icon = 'elisa.plugins.poblesec.photo'
+
     def initialize(self, album=None, photo=None, photos=None):
         dfr = super(FavoritesPhotosController, self).initialize(album=album, photo=photo, photos=photos)
 
         def get_photos(result):
             subselect = Select(FavoritesItem.foreign_id,
                                FavoritesItem.foreign_class == u'Image')
-            return application.store.find(Image, Image.file_path.is_in(subselect))
+            return application.store.find(Image, Image.file_path.is_in(subselect),
+                                          Image.file_path == File.path,
+                                          File.hidden == False)
 
         def got_photos(photos):
             self.model.extend(photos)
@@ -398,6 +597,9 @@ class FavoritesPhotosController(GenericPhotosController):
 
 class FavoritesPhotoAlbumsController(GenericPhotoAlbumsController):
 
+    empty_label = _('There are no favorite albums')
+    empty_icon = 'elisa.plugins.poblesec.photo_album'
+
     def initialize(self):
         dfr = super(FavoritesPhotoAlbumsController, self).initialize()
 
@@ -451,4 +653,50 @@ class FavoritesPhotoAlbumsListSwitcher(ListSwitcherController):
     modes = [FavoritesPhotoAlbumsVerticalWithPreview, FavoritesPhotoAlbumsCoverflow, FavoritesPhotoAlbumsGrid]
     default_mode = FavoritesPhotoAlbumsVerticalWithPreview
 
+    
+class FavoritesVideosController(VideoController):
+
+    empty_label = _('There are no favorite videos')
+    empty_icon = 'elisa.plugins.poblesec.file_video'
+
+    def initialize(self):
+        dfr = super(FavoritesVideosController, self).initialize()
+        
+        def get_videos(result):
+            subselect = Select(FavoritesItem.foreign_id,
+                               FavoritesItem.foreign_class == u'Video')
+            return application.store.find(Video, Video.file_path.is_in(subselect),
+                                          Video.file_path == File.path,
+                                          File.hidden == False)
+                                          
+        def got_videos(videos):
+            self.model.extend(videos)
+            return self
+
+        def sort_videos_title(result_set):
+            result_set.order_by(Video.name)
+            return result_set.all()
+
+        dfr.addCallback(get_videos)
+        dfr.addCallback(sort_videos_title)
+        dfr.addCallback(got_videos)
+
+        if not getattr(self, '_focus_refresh_cb_handle', None):
+            self._focus_refresh_cb_handle = self.widget.connect('focus', focus_refresh_cb, self)
+
+        return dfr
+    
+class FavoritesVideosVerticalWithPreview(FavoritesVideosController, MenuItemPreviewListController):
+    view_mode = VideoViewMode
+
+class FavoritesVideosCoverflow(FavoritesVideosController, ImageWithReflectionCoverflowController):
+    view_mode = VideoViewMode
+
+class FavoritesVideosGrid(FavoritesVideosController, GridItemGridController):
+    view_mode = VideoViewMode
+
+class FavoritesVideosListSwitcher(ListSwitcherController):
+    modes = [FavoritesVideosVerticalWithPreview, FavoritesVideosCoverflow, FavoritesVideosGrid]
+    default_mode = FavoritesVideosVerticalWithPreview
+
 
