@@ -55,8 +55,12 @@ setup(name='elisa-plugin-favorites',
                          ('/poblesec/pictures/favorites/albums',
                          'elisa.plugins.favorites.controller:FavoritesPhotoAlbumsListSwitcher'),
                          ('/poblesec/pictures/favorites/photos',
-                         'elisa.plugins.favorites.controller:FavoritesPhotosListSwitcher'),],
-    decorator_mappings=[('/poblesec/music/internet',
-                        'elisa.plugins.favorites.controller:music_internet_decorator'),],
+                         'elisa.plugins.favorites.controller:FavoritesPhotosListSwitcher'),
+                         ('/poblesec/videos/favorites',
+                         'elisa.plugins.favorites.controller:FavoritesController'),
+                         ('/poblesec/videos/favorites/videos',
+                         'elisa.plugins.favorites.controller:FavoritesVideosListSwitcher'),],
+#    decorator_mappings=[('/poblesec/music/favorites',
+#                        'elisa.plugins.favorites.controller:music_internet_decorator'),],
     cmdclass=cmdclass,
 )
