@@ -14,4 +14,8 @@ setup(name='elisa-plugin-filtered-shares',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    FilteredSharesResource = elisa.plugins.filtered_shares.filtered_shares_resource:FilteredSharesResource
+    """)
