@@ -17,11 +17,13 @@
 # Authors: Benjamin Kampmann <benjamin@fluendo.com>
 #          Alessandro Decina <alessandro@fluendo.com>
 
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 from twisted.internet.error import ConnectionDone, ConnectionLost
+import os
 
 from elisa.core.component import Component
 from elisa.core.log import Loggable
+from elisa.core.utils.misc import callAfterDeferredCalled
 from elisa.extern.log.log import getFailureMessage
 from elisa.plugins.gstreamer.request import RequestQueue
 from elisa.plugins.gstreamer.gst_metadata import able_to_handle, \
@@ -45,6 +47,7 @@ class GstMetadataMaster(Loggable, Master):
     slaveProcessProtocolFactory = GstMetadataSlaveProcessProtocol 
     socket_prefix = 'elisa-metadata-'
     max_restart_retry = 10
+    spawn_timeout = 10
 
     def __init__(self):
         Loggable.__init__(self)
@@ -73,12 +76,29 @@ class GstMetadataMaster(Loggable, Master):
         return
 
     def get_metadata(self, metadata):
+        # fast path for deleted local files
+        dfr = self._check_deleted_file(metadata)
+        if dfr is not None:
+            return dfr
+
         dfr = self._requests.enqueue(metadata)
         if not self._in_request:
             self._schedule_next_request()
 
         return dfr
 
+    def _check_deleted_file(self, metadata):
+        try:
+            uri = metadata['uri']
+        except KeyError:
+            return None
+        
+        if uri.scheme == 'file' and \
+                not os.path.exists(uri.path):
+            return defer.fail(Exception('file not found %s' % uri))
+
+        return None
+
     def _schedule_next_request(self):
         if self._restart_defer or self._next_request_call:
             return
@@ -104,8 +124,6 @@ class GstMetadataMaster(Loggable, Master):
             self.debug('metadata queue empty')
             return
 
-        self.debug('starting request %s' % request.metadata)
-
         # FIXME: fix this properly but not so close to a release
         try:
             uri = str(request.metadata['uri'])
@@ -114,6 +132,8 @@ class GstMetadataMaster(Loggable, Master):
                 request.defer.errback()
             return
 
+        self.debug('starting request %s' % request.metadata)
+
         keys = [{'key': key} for key in request.metadata.keys()
                 if key != 'uri']
 
@@ -184,20 +204,27 @@ class GstMetadataMaster(Loggable, Master):
             if not request.cancelled:
                 request.defer.errback(failure)
 
-
 class GstMetadataAmpClient(Component):
     def initialize(self):
         def start_slaves_cb(result):
+            self._start_dfr = None
             return self
         
         self._master = GstMetadataMaster()
         self._master.start()
-        dfr = self._master.startSlaves(1)
+        self._start_dfr = dfr = self._master.startSlaves(1)
         dfr.addCallback(start_slaves_cb)
 
-        return dfr
+        return super(GstMetadataAmpClient, self).initialize()
 
     def clean(self):
+        if self._start_dfr is not None:
+            return callAfterDeferredCalled(self._start_dfr,
+                    self.clean)
+
+        def parent_clean(result):
+            return super(GstMetadataAmpClient, self).clean()
+        
         def stop_slaves_cb(result):
             dfr = self._master.stop()
             dfr.addCallback(lambda result: self)
@@ -206,6 +233,7 @@ class GstMetadataAmpClient(Component):
 
         dfr = self._master.stopSlaves()
         dfr.addCallback(stop_slaves_cb)
+        dfr.addBoth(parent_clean)
 
         return dfr
 
@@ -217,4 +245,8 @@ class GstMetadataAmpClient(Component):
                 supported_keys, metadata)
 
     def get_metadata(self, metadata):
+        if self._start_dfr is not None:
+            return callAfterDeferredCalled(self._start_dfr,
+                    self.get_metadata, metadata)
+
         return self._master.get_metadata(metadata)
