@@ -81,7 +81,7 @@ class GstMetadata(MetadataProvider):
             self._process_next_call = None
         self._pipeline.clean()
 
-        return defer.succeed(self)
+        return super(GstMetadata, self).clean()
 
     def initialize(self, retself=True):
         try:
@@ -218,7 +218,11 @@ class GstMetadataFast(MetadataProvider):
         dfr2 = self.metadata_provider.clean()
         
         dfr = defer.DeferredList([dfr1, dfr2], fireOnOneErrback=True)
-        dfr.addCallback(lambda result: self)
+
+        def parent_clean(result):
+            return super(GstMetadataFast, self).clean()
+
+        dfr.addCallbacks(parent_clean)
 
         return dfr
 
