@@ -64,18 +64,9 @@ def tweak_gstreamer_elements():
         elif name in ('fftype_wsaud',):
             # we don't really want to typefind .py files as audio files...
             registry.remove_feature(typefinder)
-        elif name == 'audio/x-m4a':
-            # okay, this typefinder is a bit annoying. If it finds the
-            # ftypmp42 atom it suggests with _MAXIMUM probability that it's
-            # an audio/x-m4a. This means that all youtube high definition
-            # videos are found as audio. 
-            # We simply remove the typefinder here, as the file will still
-            # be recognized by the video/quicktime typefinder and demuxed by
-            # qtdemux anyway.
-            registry.remove_feature(typefinder)
         elif platform.system() == 'Windows' and \
-                    name in  ('application/x-ape', 'application/x-apetag'):
-                registry.remove_feature(typefinder)
+                name in  ('application/x-ape', 'application/x-apetag'):
+            registry.remove_feature(typefinder)
     
     # raise the rank of souphttpsrc as it's better than gnomevfs for http
     soupfactory = gst.element_factory_find('souphttpsrc')
@@ -295,6 +286,11 @@ class GstMetadataPipeline(Loggable):
             # FIXME: see http://bugzilla.gnome.org/show_bug.cgi?id=340375 and use this
             # hack until we write our typefinder for this
             file_type = None
+        elif gst_mime_type == 'video/x-ms-asf':
+            # this typefinder matches quite a few wma files. Disable for
+            # now so that the pipeline goes to PAUSED and the correct file type
+            # is detected
+            file_type = self._typefind_file_type = None
 
         req_keys = set(self._req_metadata.keys())
 
