@@ -338,10 +338,14 @@ class ElisaHttpClient(Loggable):
         # the other hand, if a client wants to submit another request in this
         # callback and it isn't pipelining, it must do it in
         # reactor.callLater(0, do_next_request) not to get a NotPipeliningError
-        request_dfr.addCallback(request.defer.callback)
+        request_dfr.addCallback(self._callback_request_defer, request)
         request_dfr.addErrback(request_failure, request)
         return request_dfr
 
+    def _callback_request_defer(self, result, request):
+        if not request.defer.called:
+            request.defer.callback(result)
+
     def connectionFailed(self, connector, reason):
         """
         Callback invoked by the HTTP client factory when the connection fails.
