@@ -15,13 +15,15 @@
 # for details on that license.
 
 from twisted.trial.unittest import TestCase
-from twisted.internet import reactor, defer, error, protocol
+from twisted.internet import reactor, error, protocol
 from twisted.web2.http import StatusResponse
 from twisted.web2.channel.http import HTTPFactory, HTTPChannel
 from twisted.web2 import iweb, server
 from twisted.web2 import responsecode
 from twisted.web2.stream import BufferedStream, MemoryStream
 
+from elisa.core.utils import defer
+
 import os
 
 from zope.interface import implements
@@ -879,7 +881,7 @@ class TestElisaHttpClientPipelining(TestCase, SetupServerMixin):
             self.fail('This should never happen')
 
         def third_request_cancelled(failure):
-            return failure
+            failure.trap(defer.CancelledError)
 
         def fourth_request_done(response):
             return request_done(response)
@@ -890,8 +892,8 @@ class TestElisaHttpClientPipelining(TestCase, SetupServerMixin):
 
         request_deferreds[0].addCallback(first_request_done)
         request_deferreds[1].addCallback(second_request_done)
-        request_deferreds[2].addCallback(third_request_done)
-        request_deferreds[2].addErrback(third_request_cancelled)
+        request_deferreds[2].addCallbacks(third_request_done,
+                third_request_cancelled)
         request_deferreds[3].addCallback(fourth_request_done)
 
         return defer.DeferredList(request_deferreds, consumeErrors=True)
@@ -925,7 +927,7 @@ class TestElisaHttpClientPipelining(TestCase, SetupServerMixin):
             self.fail('This should never happen')
 
         def fourth_request_cancelled(failure):
-            return failure
+            failure.trap(defer.CancelledError)
 
         for i in xrange(4):
             request_deferred = self.client.request(uri)
