@@ -18,6 +18,8 @@ setup(name='elisa-plugin-ipod',
     package_dir=package_dir,
     package_data={'': ['*.png', '*.mo', '*.po']},
     entry_points="""\
+    [elisa.core.components.resource_provider]
+    IpodResource = elisa.plugins.ipod.ipod_resource:IpodResource
     """,
     controller_mappings=[('/poblesec/music/ipod',
                           'elisa.plugins.ipod.controller:ArtistsIpodListSwitcherController'),
