@@ -25,6 +25,18 @@ class NodeNotInGroup(Exception):
     pass
 
 class Group(Node):
+    """
+    DOCME
+
+    Emits signals:
+        - 'resized': when the size of the group changes; parameters are its
+                     absolute width and height before the change.
+        - 'repositioned': when the position of the group changes: parameters
+                          are its absolute x, y and z coordinates before the
+                          change.
+
+    DOCME more signals.
+    """
 
     # FIXME: add other Pigment signals: pressured, changed
 
@@ -53,14 +65,21 @@ class Group(Node):
         'scrolled': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                 (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                 pgm.ScrollDirection, gobject.TYPE_UINT)),
+        'resized': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT)),
+        'repositioned': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT)),
     }
 
     def __init__(self):
         self._canvas = None
         self._layer = pgm.DRAWABLE_MIDDLE
         self._children = []
-        self.absolute_size = (1.0, 1.0)
-        self.absolute_position = (0.0, 0.0, 0.0)
+
+        # A dict instance->[<signal id>,...] to keep track of connected
+        # signals and be able to disconnect from them.
+        self._children_signal_id = {}
+        
         super(Group, self).__init__()
 
     def canvas__set(self, canvas):
@@ -113,16 +132,24 @@ class Group(Node):
         self._children.append(child)
         child.parent = self
 
-        if forward_signals:
+        if forward_signals and type(child) is not Node:
+            id_list = []
             for signal_name in ('double-clicked', 'drag-end', 'released'):
-                child.connect(signal_name,
-                              self._proxy_child_signal, signal_name)
+                signal_id = child.connect(signal_name,
+                                          self._proxy_child_signal, signal_name)
+                id_list.append(signal_id)
 
             for signal_name in ('clicked', 'pressed', 'drag-begin', 'drag-motion'):
-                child.connect(signal_name,
-                              self._proxy_child_signal_with_pressure, signal_name)
-            child.connect('scrolled',
-                          self._proxy_child_signal_scrolled, 'scrolled')
+                signal_id = child.connect(signal_name,
+                                      self._proxy_child_signal_with_pressure,
+                                      signal_name)
+                id_list.append(signal_id)
+            signal_id = child.connect('scrolled',
+                                      self._proxy_child_signal_scrolled,
+                                      'scrolled')
+            id_list.append(signal_id)
+
+            self._children_signal_id[child] = id_list
 
         if self._canvas != None:
             if isinstance(child, Drawable):
@@ -149,6 +176,11 @@ class Group(Node):
         except:
             raise NodeNotInGroup
 
+        if child in self._children_signal_id:
+            for signal_id in self._children_signal_id[child]:
+                child.disconnect(signal_id)
+            del self._children_signal_id[child]
+
         child.parent = None
 
         if self._canvas != None:
@@ -174,45 +206,59 @@ class Group(Node):
         return self._children.__iter__()
 
     def update_absolute_x(self, value=None):
+        x, y, z = self.absolute_position
         super(Group, self).update_absolute_x(value)
         for child in self._children:
             child.update_absolute_x()
+        self.emit("repositioned", x, y, z)
 
     def update_absolute_y(self, value=None):
+        x, y, z = self.absolute_position
         super(Group, self).update_absolute_y(value)
         for child in self._children:
             child.update_absolute_y()
+        self.emit("repositioned", x, y, z)
 
     def update_absolute_z(self, value=None):
+        x, y, z = self.absolute_position
         super(Group, self).update_absolute_z(value)
         for child in self._children:
             child.update_absolute_z()
+        self.emit("repositioned", x, y, z)
 
     def update_absolute_position(self, value=None):
+        x, y, z = self.absolute_position
         super(Group, self).update_absolute_position(value)
         for child in self._children:
             child.update_absolute_position()
+        self.emit("repositioned", x, y, z)
 
     def update_absolute_width(self, value=None):
+        width, height = self.absolute_size
         super(Group, self).update_absolute_width(value)
         for child in self._children:
             child.update_absolute_width()
             # FIXME: write test before uncommenting
             child.update_absolute_x()
+        self.emit("resized", width, height)
 
     def update_absolute_height(self, value=None):
+        width, height = self.absolute_size
         super(Group, self).update_absolute_height(value)
         for child in self._children:
             child.update_absolute_height()
             # FIXME: write test before uncommenting
             child.update_absolute_y()
+        self.emit("resized", width, height)
 
     def update_absolute_size(self, value=None):
+        width, height = self.absolute_size
         super(Group, self).update_absolute_size(value)
         for child in self._children:
             child.update_absolute_size()
             # FIXME: write test before uncommenting
             child.update_absolute_position()
+        self.emit("resized", width, height)
 
     def update_absolute_visible(self, value=None):
         super(Group, self).update_absolute_visible(value)
@@ -228,50 +274,11 @@ class Group(Node):
     def regenerate(self):
         for child in self._children:
             child.regenerate()
-
-    def absolute_width__get(self):
-        return self.absolute_size[0]
-
-    def absolute_width__set(self, value):
-        self.absolute_size = (value, self.absolute_size[1])
-
-    absolute_width = property(absolute_width__get, absolute_width__set)
-
-    def absolute_height__get(self):
-        return self.absolute_size[1]
-
-    def absolute_height__set(self, value):
-        self.absolute_size = (self.absolute_size[0], value)
-
-    absolute_height = property(absolute_height__get, absolute_height__set)
-
-    def absolute_x__get(self):
-        return self.absolute_position[0]
-
-    def absolute_x__set(self, value):
-        self.absolute_position = (value, self.absolute_position[1],
-                                  self.absolute_position[2])
-
-    absolute_x = property(absolute_x__get, absolute_x__set)
-
-    def absolute_y__get(self):
-        return self.absolute_position[1]
-
-    def absolute_y__set(self, value):
-        self.absolute_position = (self.absolute_position[0], value,
-                                  self.absolute_position[2])
-
-    absolute_y = property(absolute_y__get, absolute_y__set)
-
-    def absolute_z__get(self):
-        return self.absolute_position[2]
-
-    def absolute_z__set(self, value):
-        self.absolute_position = (self.absolute_position[0],
-                                  self.absolute_position[1], value)
-
-    absolute_z = property(absolute_z__get, absolute_z__set)
     
     def clean(self):
-        for child in self._children:
+        # We need to create a new list since  we modify it in the loop.
+        for child in list(self._children):
+            self.remove(child)
             child.clean()
+
+        return super(Group, self).clean()
