@@ -22,3 +22,7 @@ class Image(Drawable, pgm.Image):
     def __init__(self):
          pgm.Image.__init__(self)
          Drawable.__init__(self)
+
+    def clean(self):
+        self.clear()
+        return super(Image, self).clean()
