@@ -16,6 +16,7 @@
 
 import gobject
 import pgm
+import weakref
 
 class Node(gobject.GObject):
 
@@ -32,11 +33,56 @@ class Node(gobject.GObject):
         self._relative_height = 1.0
         self._relative_opacity = 255
         self.absolute_opacity = self._relative_opacity
+        self.absolute_size = (1.0, 1.0)
+        self.absolute_position = (0.0, 0.0, 0.0)
+
+    def absolute_width__get(self):
+        return self.absolute_size[0]
+
+    def absolute_width__set(self, value):
+        self.absolute_size = (value, self.absolute_size[1])
+
+    absolute_width = property(absolute_width__get, absolute_width__set)
+
+    def absolute_height__get(self):
+        return self.absolute_size[1]
+
+    def absolute_height__set(self, value):
+        self.absolute_size = (self.absolute_size[0], value)
+
+    absolute_height = property(absolute_height__get, absolute_height__set)
+
+    def absolute_x__get(self):
+        return self.absolute_position[0]
+
+    def absolute_x__set(self, value):
+        self.absolute_position = (value, self.absolute_position[1],
+                                  self.absolute_position[2])
+
+    absolute_x = property(absolute_x__get, absolute_x__set)
+
+    def absolute_y__get(self):
+        return self.absolute_position[1]
+
+    def absolute_y__set(self, value):
+        self.absolute_position = (self.absolute_position[0], value,
+                                  self.absolute_position[2])
+
+    absolute_y = property(absolute_y__get, absolute_y__set)
+
+    def absolute_z__get(self):
+        return self.absolute_position[2]
+
+    def absolute_z__set(self, value):
+        self.absolute_position = (self.absolute_position[0],
+                                  self.absolute_position[1], value)
+
+    absolute_z = property(absolute_z__get, absolute_z__set)
 
     def parent__set(self, parent):
-        self._parent = parent
 
         if parent != None:
+            self._parent = weakref.proxy(parent)
             # compute relative properties so that the node does not move when
             # reparented
             self._relative_width = self.absolute_width/self._parent.absolute_width
@@ -50,6 +96,7 @@ class Node(gobject.GObject):
             # update absolute properties using current relative values
             self.update_absolute_visible()
         else:
+            self._parent = None
             self.absolute_visible = self._relative_visible
 
     def parent__get(self):
@@ -279,4 +326,7 @@ class Node(gobject.GObject):
                     int(self._relative_opacity*self._parent.absolute_opacity/255.0)
 
     def clean(self):
-        pass
\ No newline at end of file
+        if self._parent:
+            self._parent.remove(self)
+            self._parent = None
+
