@@ -16,25 +16,30 @@
 
 __maintainer__ = "Benjamin Kampmann <benjamin@fluendo.com>"
 
-from elisa.plugins.pigment.widgets.theme import Theme
-from elisa.plugins.pigment.widgets import const
-from elisa.core.components.frontend import Frontend
-from elisa.core.pattern_matcher import UriPatternMatcher, MatchNotFound
-from twisted.python.failure import Failure
-from twisted.python import reflect
-from elisa.core.log import Loggable
-from elisa.extern.log.log import getFailureMessage
 from elisa.core import common
 from elisa.core.utils.dist import read_mappings
+from elisa.core.components.frontend import Frontend
+from elisa.core.pattern_matcher import UriPatternMatcher, MatchNotFound
+from elisa.core.plugin_registry import PluginStatusMessage, PluginNotFound
+from elisa.core.log import Loggable
+
+from elisa.extern.log.log import getFailureMessage
+
+from elisa.plugins.pigment.widgets.theme import Theme
+from elisa.plugins.pigment.widgets import const
+from elisa.plugins.pigment.pigment_input import PigmentInput
+
+from twisted.python.failure import Failure
+from twisted.python import reflect
 from twisted.internet import reactor, task, defer
-import os, platform
+
 import pgm
 from pgm.timing.controller import Controller
 from pgm.timing.ticker import Ticker
+
 import pkg_resources
-from elisa.plugins.pigment.pigment_input import PigmentInput
+import os, platform
 import weakref
-import threading
 import sys
 
 try:
@@ -56,30 +61,71 @@ class PigmentFrontendExtensionMixin(Loggable):
         self._load_controller_mappings()
         self._load_decorator_mappings()
 
+        bus = common.application.bus
+        bus.register(self._plugin_status_changed_cb, PluginStatusMessage)
+
         self._controller_store = {}
 
+    def _update_controller_mappings(self, plugin, enabled):
+        # Load/unload the controller mappings for a given plugin.
+        metadata = 'controller_mappings.txt'
+        if not plugin.has_metadata(metadata):
+            return
+        lines = plugin.get_metadata_lines(metadata)
+        mappings = read_mappings(lines)
+        for path_pattern, controller_path in mappings:
+            if enabled:
+                self.add_controller(path_pattern, controller_path)
+            else:
+                self.remove_controller(path_pattern)
 
     def _load_controller_mappings(self):
         """
-        Load the controller mappings from all plugins' metadata.
+        Load the controller mappings from all the enabled plugins' metadata.
         """
+        registry = common.application.plugin_registry
+        enabled_plugins = [plugin for plugin in registry.get_enabled_plugins()]
         for dist in pkg_resources.working_set:
-            if dist.has_metadata('controller_mappings.txt'):
-                lines = dist.get_metadata_lines('controller_mappings.txt')
-                mappings = read_mappings(lines)
-                for path_pattern, controller_path in mappings:
-                    self.add_controller(path_pattern, controller_path)
+            if dist.key not in enabled_plugins:
+                continue
+            self._update_controller_mappings(dist, True)
+
+    def _update_decorator_mappings(self, plugin, enabled):
+        # Load/unload the decorator mappings for a given plugin.
+        metadata = 'decorator_mappings.txt'
+        if not plugin.has_metadata(metadata):
+            return
+        lines = plugin.get_metadata_lines(metadata)
+        mappings = read_mappings(lines)
+        for path_pattern, decorator_path in mappings:
+            if enabled:
+                self.add_decorator(path_pattern, decorator_path)
+            else:
+                self.remove_decorator(path_pattern)
 
     def _load_decorator_mappings(self):
         """
-        Load the controller mappings from all plugins' metadata.
+        Load the decorator mappings from all the enabled plugins' metadata.
         """
+        registry = common.application.plugin_registry
+        enabled_plugins = [plugin for plugin in registry.get_enabled_plugins()]
         for dist in pkg_resources.working_set:
-            if dist.has_metadata('decorator_mappings.txt'):
-                lines = dist.get_metadata_lines('decorator_mappings.txt')
-                mappings = read_mappings(lines)
-                for path_pattern, decorator_path in mappings:
-                    self.add_decorator(path_pattern, decorator_path)
+            if dist.key not in enabled_plugins:
+                continue
+            self._update_decorator_mappings(dist, True)
+
+    def _plugin_status_changed_cb(self, message, sender):
+        # Notification that the status of a plugin has changed.
+        # Update the controller and decorator mappings accordingly.
+        try:
+            requirement = pkg_resources.Requirement.parse(message.plugin_name)
+            plugin = pkg_resources.working_set.find(requirement)
+        except KeyError:
+            raise PluginNotFound(message.plugin_name)
+
+        enabled = (message.action == PluginStatusMessage.ActionType.ENABLED)
+        self._update_controller_mappings(plugin, enabled)
+        self._update_decorator_mappings(plugin, enabled)
 
     def add_controller(self, path_pattern, controller):
         """
@@ -94,6 +140,17 @@ class PigmentFrontendExtensionMixin(Loggable):
         """
         self.controller_matcher.add_pattern(path_pattern, controller)
 
+    def remove_controller(self, path_pattern):
+        """
+        Remove a controller from the frontend.
+        The controller is the one that would be loaded for paths matching
+        C{path_pattern}.
+
+        @param path_pattern: a regular expression pattern
+        @type path_pattern:  C{str}
+        """
+        self.controller_matcher.remove_pattern(path_pattern)
+
     def create_controller(self, path, config=None,
             wait_for_decorators=False, **kwargs):
         """
@@ -164,6 +221,17 @@ class PigmentFrontendExtensionMixin(Loggable):
         """
         self.decorator_matcher.add_pattern(path_pattern, decorator)
 
+    def remove_decorator(self, path_pattern):
+        """
+        Remove a decorator function from the frontend.
+        The decorator is the one that would be called for controllers with a
+        path matching C{path_pattern}.
+
+        @param path_pattern: a regular expression pattern
+        @type path_pattern:  C{str}
+        """
+        self.decorator_matcher.remove_pattern(path_pattern)
+
     def _set_frontend(self, controller):
         controller.set_frontend(self)
 
@@ -304,7 +372,11 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         # all the CPU by rendering as fast as possible; it happens when
         # synchronisation with Vblank fails completely
         # See Elisa bug: https://bugs.launchpad.net/elisa/+bug/231899
-        os.environ["PGM_GL_FPS"] = "120"
+        os.environ["PGM_GL_FPS"] = "60"
+
+        # deactivate synchronisation to VBlank; avoids flickering on HP mini
+        # 1100
+        os.environ["PGM_GL_VBLANK"] = "0"
 
         # signal handlers id
         self._signal_handler_ids = []
@@ -313,13 +385,21 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         # some reason pgm is not defined. This should be fixed with Python
         # 3000
         import pgm
+        import gtk
 
         factory = pgm.ViewportFactory('opengl')
         viewport = factory.create()
 
         self.viewport = viewport
 
-        viewport.title = 'Elisa Media Center'
+        viewport.title = 'HP MediaStyle Powered by Elisa'
+
+	try:
+            pix = gtk.icon_theme_get_default ().load_icon ("elisa", 48, 0)
+            viewport.set_icon (pix)
+        except:
+            pass
+
         self.canvas = pgm.Canvas()
 
         viewport.set_canvas(self.canvas)
@@ -353,6 +433,10 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         viewport.width *= (screen_aspect_ratio / resolution_aspect_ratio)
 
         fullscreen = self.config.as_bool('start_fullscreen')
+        if common.application.options['fullscreen']:
+            fullscreen = True
+        elif common.application.options['unfullscreen']:
+            fullscreen = False
 
         headless = '--headless' in sys.argv or \
             self.config.as_bool('headless')
@@ -360,7 +444,6 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         if self.config.as_bool('use_gtk') is True:
             self.debug("using gtk")
 
-            import gtk
             import pgm.gtk
 
             self.gtk_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
@@ -492,9 +575,7 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
             widget_class = widget.__class__.__name__
             widget_path = widget_module + '.' + widget_class
 
-            # FIXME: use the choosen logging framework
-            import logging
-            logging.debug("Setting style for widget at path '%s'" % widget_path)
+            self.debug("Setting style for widget at path '%s'" % widget_path)
 
             for state in [const.STATE_NORMAL,
                           const.STATE_ACTIVE,
@@ -513,7 +594,7 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         """Get the current theme.
 
         @return: the current theme
-        @type theme: L{elisa.plugins.widgets.Theme}
+        @rtype:  L{elisa.plugins.widgets.Theme}
         """
         return self._theme
 
@@ -597,11 +678,8 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
             return
 
         bus = dbus.SessionBus()
-        self.bus_name = \
-                dbus.service.BusName('com.fluendo.Elisa.Plugins.Pigment', bus)
-    
         self.dbus_frontend = DBusFrontend(self, bus, 
-                '/com/fluendo/Elisa/Plugins/Pigment/Frontend', self.bus_name)
+                '/com/fluendo/Elisa/Plugins/Pigment/Frontend')
 
     def _clean_dbus(self):
         if 'dbus' not in sys.modules:
@@ -611,8 +689,6 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         bus = dbus.SessionBus()
         self.dbus_frontend.remove_from_connection(bus,
                 '/com/fluendo/Elisa/Plugins/Pigment/Frontend')
-        # BusName implements __del__, eew
-        del self.bus_name
 
         # remove the reference cycle
         del self.dbus_frontend
