@@ -1,15 +1,35 @@
+# -*- coding: utf-8 -*-
+# Elisa - Home multimedia server
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
+# All rights reserved.
+#
+# This file is available under one of two license agreements.
+#
+# This file is licensed under the GPL version 3.
+# See "LICENSE.GPL" in the root of this distribution including a special
+# exception to use Elisa with Fluendo's plugins.
+#
+# The GPL part of Elisa is also available under a commercial licensing
+# agreement from Fluendo.
+# See "LICENSE.Elisa" in the root directory of this distribution package
+# for details on that license.
+
 from twisted.trial.unittest import TestCase
 from twisted.internet import defer, reactor
 from twisted.python import reflect
 
+from elisa.core import common
 from elisa.core.config import Config
+from elisa.core.plugin_registry import PluginStatusMessage
+from elisa.core.utils.misc import pkg_resources_copy_dir
+
 from elisa.plugins.pigment.pigment_frontend import PigmentFrontendExtensionMixin
 from elisa.plugins.pigment.pigment_controller import PigmentController
-from elisa.core.utils.misc import pkg_resources_copy_dir 
 
 import pkg_resources
 import os.path
 
+
 class TestController(PigmentController):
     decorated = 0
 
@@ -39,10 +59,31 @@ def failing_decorator(controller):
 def failing_decorator_two(controller):
     return defer.fail(Exception('woot'))
 
+
+class PluginRegistryStub(object):
+    def get_enabled_plugins(self):
+        return ['elisa-plugin-test-pigment-frontend']
+
+
+class BusStub(object):
+    def register(self, callback, message_class):
+        pass
+
+
+class ApplicationStub(object):
+    def __init__(self):
+        self.plugin_registry = PluginRegistryStub()
+        self.bus = BusStub()
+
+
 class TestPigmentFrontendExtensionMixin(TestCase):
     def setUp(self):
+        common.application = ApplicationStub()
         self.frontend = PigmentFrontendExtensionMixin()
 
+    def tearDown(self):
+        common.application = None
+
     def test_create(self):
         """
         Register and create two controllers. Test that two patterns don't
@@ -380,9 +421,22 @@ class PigmentFrontendExtensionMixinStub(PigmentFrontendExtensionMixin):
     def add_controller(self, path_pattern, controller):
         self.added_controllers.append((path_pattern, controller))
 
+    def remove_controller(self, path_pattern):
+        for index, (pattern, controller) in enumerate(self.added_controllers):
+            if pattern == path_pattern:
+                self.added_controllers.pop(index)
+                return
+
     def add_decorator(self, path_pattern, decorator):
         self.added_decorators.append((path_pattern, decorator))
 
+    def remove_decorator(self, path_pattern):
+        for index, (pattern, decorator) in enumerate(self.added_decorators):
+            if pattern == path_pattern:
+                self.added_decorators.pop(index)
+                return
+
+
 class TestPigmentFrontendExtensionMixinMappings(TestCase):
     def setUp(self):
         # do everything under _trial_temp/test_pigment_plugin
@@ -392,8 +446,12 @@ class TestPigmentFrontendExtensionMixinMappings(TestCase):
         self.copy_plugin('tests/data/plugins/test_simple')
         self.load_plugins()
 
+        common.application = ApplicationStub()
         self.frontend = PigmentFrontendExtensionMixinStub()
 
+    def tearDown(self):
+        common.application = None
+
     def copy_plugin(self, path):
         # copy the plugin to _trial_temp/test_pigment_plugin
         pkg_resources_copy_dir('elisa.plugins.pigment', path, self.test_dir)
@@ -427,14 +485,51 @@ class TestPigmentFrontendExtensionMixinMappings(TestCase):
         loaded = set(self.frontend.added_decorators)
         self.failUnless(decorator_mappings.issubset(loaded))
 
+    def test_disable_then_enable_plugin(self):
+        """
+        Test disabling a plugin, then re-enabling it.
+        """
+        plugin_name = 'elisa-plugin-test-pigment-frontend'
+
+        # Disable the plugin.
+        message = PluginStatusMessage(plugin_name,
+                                      PluginStatusMessage.ActionType.DISABLED)
+        self.frontend._plugin_status_changed_cb(message, None)
+        self.failUnlessEqual(len(self.frontend.added_controllers), 0)
+        self.failUnlessEqual(len(self.frontend.added_decorators), 0)
+
+        # Re-enable the plugin.
+        message = PluginStatusMessage(plugin_name,
+                                      PluginStatusMessage.ActionType.ENABLED)
+        self.frontend._plugin_status_changed_cb(message, None)
+        controller_mappings = \
+            set([('test_controller_key1', 'test_controller_value1'),
+                 ('test_controller_key2', 'test_controller_value2')])
+        loaded_controllers = set(self.frontend.added_controllers)
+        self.failUnless(controller_mappings.issubset(loaded_controllers))
+        decorator_mappings = \
+            set([('test_decorator_key1', 'test_decorator_value1'),
+                 ('test_decorator_key2', 'test_decorator_value2')])
+        loaded_decorators = set(self.frontend.added_decorators)
+        self.failUnless(decorator_mappings.issubset(loaded_decorators))
+
     def test_initialize_theme(self):
-        self.fail()
+        self.fail(NotImplementedError("Test not yet implemented"))
+
+    test_initialize_theme.todo = "Test not yet implemented"
 
     def test_get_theme(self):
-        self.fail()
+        self.fail(NotImplementedError("Test not yet implemented"))
+
+    test_get_theme.todo = "Test not yet implemented"
 
     def test_set_theme(self):
-        self.fail()
+        self.fail(NotImplementedError("Test not yet implemented"))
+
+    test_set_theme.todo = "Test not yet implemented"
 
     def test_load_from_theme_theme(self):
-        self.fail()
+        self.fail(NotImplementedError("Test not yet implemented"))
+
+    test_load_from_theme_theme.todo = "Test not yet implemented"
+
