@@ -127,3 +127,28 @@ class TestTheme(unittest.TestCase):
 
         self.failUnlessEqual(theme, Theme.get_default())
 
+    def test_fallback_resources(self):
+        theme = Theme()
+        Theme.set_default(theme)
+
+        # check the resource is not in default theme
+        self.failIf(theme.get_resource('elisa.plugins.my_ui.resource1'))
+
+        theme.fallback_theme = Theme.load_from_module('elisa.plugins.pigment.tests.data.widgets')
+
+        # the theme will look in its fallback theme
+        self.failUnless(theme.get_resource('elisa.plugins.my_ui.resource1'))
+
+    def test_fallback_style(self):
+        theme = Theme()
+        Theme.set_default(theme)
+
+        # check the style is not in default theme
+        self.failIf(theme.get_style_for_widget('i.dont.exist'))
+
+        # setup a fallback theme
+        theme.fallback_theme = Theme()
+        theme.fallback_theme.widget_styles = {'i.dont.exist': {6: {'padding': 0}}}
+
+        # the theme will look in its fallback theme
+        self.failUnless(theme.get_style_for_widget('i.dont.exist'))
