@@ -28,13 +28,15 @@ import gobject
 import math
 import time as mod_time
 
+from twisted.internet import reactor
+
 class List(Widget):
     """
     Abstract list widget. It displays data stored in the L{model} list. Items
     of L{model} are rendered into widgets instances of the widget class passed
     to the constructor using the specified L{renderer} function.
 
-    Usage example for a list of strings rendered in Text drawables:
+    Usage example for a list of strings rendered in Text drawables::
 
         l = List(Text)
 
@@ -54,24 +56,24 @@ class List(Widget):
 
     @ivar selected_item_index:    index of the currently selected item in the
                                   L{model}
-    @type selected_item_index:    int
+    @type selected_item_index:    C{int}
     @ivar visible_range_size:     number of rendered items
-    @type visible_range_size:     float
+    @type visible_range_size:     C{float}
     @ivar model:                  list of data items that is rendered by the
                                   list widget
-    @type model:                  list
+    @type model:                  C{list}
     @ivar drag_motion_resolution: minimum time between 2 drag events in milliseconds
-    @type drag_motion_resolution: int
+    @type drag_motion_resolution: C{int}
     @ivar drag_threshold:         amount of movement needed to activate dragging
                                   in canvas coordinates
-    @type drag_threshold:         float
-    @ivar animated:               True if the list is animated, False otherwise
-    @type animated:               bool
+    @type drag_threshold:         C{float}
+    @ivar animated:               C{True} if the list is animated, C{False} otherwise
+    @type animated:               C{bool}
     @ivar preloaded:              number of items that are prerendered before
                                   they become visible
-    @type preloaded:              int
+    @type preloaded:              C{int}
     @ivar focus_on_click:         whether the button will grab focus when clicked
-    @type focus_on_click:         boolean
+    @type focus_on_click:         C{bool}
     """
 
     __gsignals__ = {
@@ -97,6 +99,7 @@ class List(Widget):
         @type visible_range_size:  int
         """
         super(List, self).__init__()
+        self._render_all_call = None
 
         self.focus_on_click = True
 
@@ -176,7 +179,7 @@ class List(Widget):
 
     def set_selector(self, selector):
         if self._selector:
-            self.remove(self._selector)
+            self._selector.clean()
 
         self._selector = selector
 
@@ -197,8 +200,6 @@ class List(Widget):
         else:
             self._selector.opacity = 0
 
-        self._selector.opacity = 255
-
     # Implementation methods
 
     def _connect_widget(self, widget):
@@ -225,6 +226,7 @@ class List(Widget):
         model = self.model[real_index]
 
         self.emit(name, model)
+        return True
 
     def _on_items_changed(self, notifier, index, items):
         widget_index = self._widget_index_from_item_index(index)
@@ -306,7 +308,14 @@ class List(Widget):
                 widget.visible = False
 
     def _render_all(self):
-        self._render_widgets(0, len(self._widgets)-1)
+        if self._render_all_call is not None:
+            return
+
+        def render():
+            self._render_all_call = None
+            self._render_widgets(0, len(self._widgets)-1)
+
+        self._render_all_call = reactor.callLater(0, render)
 
     def visible_range_start__get(self):
         return self._visible_range_start
@@ -358,7 +367,7 @@ class List(Widget):
     def _create_widgets(self):
          # FIXME: do not destroy and reload everything; support floats
         for widget in self._widgets:
-            self.remove(widget)
+            widget.clean()
         self._widgets[:] = []
 
         nb_widgets = self._visible_range_size+2*self.preloaded
@@ -387,7 +396,8 @@ class List(Widget):
         if self.animated and len(self.model) > 0:
             self._animated.visible_range_start = visible_range_start
         else:
-            self._animated.stop_animations()
+            if self.animated:
+                self._animated.stop_animations()
             self.visible_range_start = visible_range_start
 
         prev_selected = self._selected_item_index
@@ -672,4 +682,27 @@ class List(Widget):
 
         return widget
 
+    def clean(self):
+        for widget in self._widgets:
+            widget.clean()
+        self._widgets=[]
+        self._animated.stop_animations()
+        self._animated = None
+        self.animated = False
+
+        if self._selector is not None:
+            self._selector.clean()
+            self._selector = None
+        if self._animated_selector is not None:
+            self._animated_selector.stop_animations()
+            self._animated_selector = None
+
+        self._drag_zone.clean()
+        self._drag_zone = None
+
+        if self._render_all_call is not None:
+            self._render_all_call.cancel()
+            self._render_all_call = None
+
+        return super(List, self).clean()
 
