@@ -192,6 +192,7 @@ class Button(Widget):
         if image_resource is None:
             self._label.width = 0.7
             self._label.position = (0.15, 0.375, 0.0)
+            self._image = None
         else:
             self._image = Image()
             self.add(self._image, forward_signals=False)
@@ -214,7 +215,7 @@ class Button(Widget):
                 self._label.width = 0.7 - self._image.width
                 self._label.position = (0.15 + self._image.width, 0.375, 0.0)
 
-            self._loaded_id = self._image.connect('pixbuf-loaded', image_loaded)
+            self._loaded_id = self._image.connect('file-loaded', image_loaded)
 
         self._label.label = text
         self._label.visible = True
@@ -229,6 +230,20 @@ class Button(Widget):
 
     label = property(label__get, label__set)
 
+    def clean(self):
+        if self._background is not None:
+            self._background.clean()
+            self._background = None
+        if self._label is not None:
+            self._label.clean()
+            self._label = None
+
+        if self._image is not None:
+            self._image.clean()
+            self._image = None
+
+        return super(Button, self).clean()
+
 
 class OnScreenKeyboard(Widget):
 
@@ -282,7 +297,22 @@ class OnScreenKeyboard(Widget):
     def clean(self):
         for widget, signal_id in self._signal_handler_ids:
             widget.disconnect(signal_id)
-        super(OnScreenKeyboard, self).clean()
+        self._signal_handler_ids = []
+
+        if self._animated_selector:
+            self._animated_selector.stop_animations()
+            self._animated_selector = None
+
+        if self._selector:
+            self._selector.clean()
+            self._selector = None
+
+        self.keys_widget.clean()
+        self.keys_widget = None
+        if self.current_key:
+            self.current_key = None
+        self.keys = []
+        return super(OnScreenKeyboard, self).clean()
 
     def to_xml(self):
         """
@@ -365,7 +395,6 @@ class OnScreenKeyboard(Widget):
         self._selector.bg_color = (100, 200, 135, 150)
         self._selector.position = self.current_key.widget.position
         self._selector.size = self.current_key.widget.size
-        self._selector.visible = True
         self._animated_selector = implicit.AnimatedObject(self._selector)
         settings = {'duration': 300,
                     'transformation': implicit.DECELERATE,
@@ -512,6 +541,7 @@ class OnScreenKeyboard(Widget):
                                 'data', 'osk_alphabetic.xml')
         widget = cls(xml_file)
         widget.visible = True
+        widget.focus = True
 
         def on_key_press(osk, value):
             logging.info('OSK key-press: %s' % value)
@@ -538,3 +568,7 @@ if __name__ == '__main__':
         __IPYTHON__
     except NameError:
         pgm.main()
+    osk.clean()
+    #FIXME: why on earth do we need this?
+    import time
+    time.sleep(0.1)
