@@ -33,6 +33,9 @@ _logger.setLevel(logging.ERROR)
 _DEFAULT_THEME = None
 
 
+class ResourceNotExisting(Exception):
+    pass
+
 class Theme(object):
     """
     A theme object, that adds styles to widgets properties and to stock resources.
@@ -61,6 +64,7 @@ class Theme(object):
         self.widget_styles = {}
         self.stock_resources = {}
         self.fallback_themes = {}
+        self.fallback_theme = None
 
         if styles_conf:
             self._init_styles(styles_conf)
@@ -157,7 +161,6 @@ class Theme(object):
         @returns: the new theme, or None
         @rtype: L{elisa.plugins.pigment.widgets.Theme}
         """
-
         try:
             styles_conf = resource_filename(module_name, 'styles.conf')
         except (ImportError, KeyError), exc:
@@ -253,6 +256,8 @@ class Theme(object):
             return
 
         widget_styles = self.widget_styles.get(widget)
+        if not widget_styles and self.fallback_theme:
+            widget_styles = self.fallback_theme.widget_styles.get(widget)
 
         if not widget_styles:
             if not search:
@@ -273,12 +278,14 @@ class Theme(object):
         @type search: boolean
         """
         media = self.stock_resources.get(name)
+        if not media and self.fallback_theme:
+            media = self.fallback_theme.stock_resources.get(name)
 
         if media:
             return media
 
         if not search:
-            return
+            raise ResourceNotExisting(name)
 
         return self.lookup(name, 'resource')
 
@@ -301,7 +308,6 @@ class Theme(object):
         @returns: the found resource (filepath or style), if any
         @rtype: string, or L{elisa.plugins.pigment.widgets.Style}, or None
         """
-
         if type == 'style' and ':' in name:
             name = name.split(':')[0]
             state = getattr(const, name.split(':')[1], const.STATE_NORMAL)
@@ -323,6 +329,11 @@ class Theme(object):
 
                 if not module_theme:
                     return
+                else:
+                    if not self.fallback_theme:
+                        self.fallback_theme = module_theme
+                    else:
+                        self.fallback_theme.merge(module_theme, inplace=True)
 
             if type == 'resource':
                 #item = module_theme.get_resource(name.split('.')[-1], False)
