@@ -66,6 +66,9 @@ class Widget(Group):
                                          gobject.TYPE_PYOBJECT,
                                          gobject.TYPE_PYOBJECT))}
 
+    # The widget with the focus in the direct acyclic graph
+    _focused = None
+
     def __init__(self):
         """
         Set the default styles for the widget. Subclasses should override and
@@ -81,7 +84,7 @@ class Widget(Group):
 
         self._state = None
         self._styles = {}
-        self._focus = False
+        self._style = None
 
         self._init_styles()
 
@@ -89,6 +92,8 @@ class Widget(Group):
 
         self._initializing = False
 
+        self._focused_child = None
+
     def _init_styles(self):
         theme = Theme.get_default()
 
@@ -101,12 +106,15 @@ class Widget(Group):
         while issubclass(base, Widget):
             module_name = base.__module__
             # HACK, to allow interactive testing of some widgets
+            # FIXME: this prevents from testing interactively widgets that are
+            # not in the pigment plugin and use custom styles (defined in an
+            # external styles.conf file).
             if module_name == '__main__':
                 module_name = 'elisa.plugins.pigment.widgets'
             else:
-                module_name = '.'.join(module_name.split('.')[0:-1])
+                module_name = module_name.rsplit('.', 1)[0]
 
-            widget_class = module_name + '.' + base.__name__
+            widget_class = '%s.%s' % (module_name, base.__name__)
             for state in (STATE_NORMAL,):# STATE_ACTIVE, STATE_PRELIGHT,
                           #STATE_SELECTED, STATE_INSENSITIVE):
                 styles[state].insert(0, theme.get_style_for_widget(widget_class, state))
@@ -142,10 +150,11 @@ class Widget(Group):
         @type state: enum(STATE_NORMAL, STATE_ACTIVE, STATE_PRELIGHT,
                           STATE_SELECTED, STATE_INSENSITIVE)
         """
-        if self._state != state:
-            logging.debug("State changed from %s to %s" % (self._state, state))
+        previous_state = self._state
+        if state != previous_state:
+            logging.debug("State changed from %s to %s" % (previous_state, state))
             self._state = state
-            self.emit("state-changed", state)
+            self.emit("state-changed", previous_state)
 
     state = property(state__get, state__set)
 
@@ -166,13 +175,20 @@ class Widget(Group):
         for property change notifications.
 
         @param style: the style to set
-        @type param: L{pgm.widget.Style}
+        @type style:  L{pgm.widget.Style}
         """
         logging.debug("Style changed")
-        style.connect('property-changed', self._style_property_changed)
-        style.widget = self
-        self._style = style
-        self.emit("style-set", style)
+
+        if self._style is not None:
+            self._style.disconnect_by_func(self._style_property_changed)
+
+        if style is not None:
+            style.connect('property-changed', self._style_property_changed)
+            style.widget = self
+            self._style = style
+            self.emit("style-set", style)
+        else:
+            self._style = None
 
     style = property(style__get, style__set)
 
@@ -243,43 +259,73 @@ class Widget(Group):
 
         return items
 
-    def focus__get(self):
-        """Whether the widget has focus"""
-        return self._focus
-
-    def focus__set(self, value):
+    def _set_focused(self, state):
         """
-        Set or unset the focus on the widget and emit the 'focus' signal.
+        Set or unset the focused widget and update the focused child property of
+        the ancestors.
 
-        If setting the focus on the current widget, walk throught the widget
-        hierarchy it belongs to and unset the previously focused one.
-
-        @param value: the focus value.
+        @param value: the focus state.
         @type value: boolean.
         """
+        # Store the new focused widget depending on the requested state
+        if state == True:
+            focused = self
+        else:
+            focused = None
 
-        #if self._focus == value:
-        #    return
+        # Update the currently focused widget
+        Widget._focused = focused
 
-        if not value:
-            self._focus = value
-            self.emit('focus', value)
-            return
+        # Update focused child property of the ancestors
+        walk = self.parent
+        while walk != None:
+            walk._focused_child = focused
+            walk = walk.parent
 
-        for widget in self.get_root().get_descendants():
-            if widget != self:
-                widget.focus = False
+    def focus__get(self):
+        """Whether the widget has focus"""
+        return Widget._focused == self
 
-        self._focus = value
-        self.emit('focus', value)
+    def focus__set(self, state):
+        """
+        Set the focus state on the widget emitting the 'focus' signal on the
+        previously focused widget and then on self.
+
+        @param value: the focus state.
+        @type value: boolean.
+        """
+        if state == True:
+            # Get the previously focused widget
+            previously_focused = Widget._focused
+
+            # Nothing to do if the widget already has the focus
+            if previously_focused == self:
+                return
+
+            # Reset the previously focused widget and set the new one
+            if previously_focused != None:
+                previously_focused._set_focused(False)
+            self._set_focused(True)
+
+            # Emit the signals at the end of the process
+            if previously_focused != None:
+                previously_focused.emit('focus', False)
+            self.emit('focus', True)
+
+        else:
+            # Get the currently focused widget
+            currently_focused = Widget._focused
+
+            # If it's self, no widget has the focus anymore
+            if currently_focused == self:
+                self._set_focused(False)
+                currently_focused.emit('focus', False)
 
     focus = property(focus__get, focus__set)
 
     def focus_child__get(self):
         """Get the child with focus"""
-        for child in self.get_descendants():
-            if child.focus:
-                return child
+        return self._focused_child
 
     focus_child = property(focus_child__get)
 
@@ -298,6 +344,12 @@ class Widget(Group):
     def _do_real_key_press_event(self, viewport, event, widget):
         self.emit("key-press-event", viewport, event, widget)
 
+    def clean(self):
+        self.name = None
+        self.style = None
+        self._styles = {}
+        return super(Widget, self).clean()
+
     @classmethod
     def _demo_create_viewport(cls, plugin):
         viewport = pgm.viewport_factory_make('opengl')
