@@ -68,7 +68,7 @@ class PreviewWidget(Widget):
         self.add(self.image)
         self.image.bg_a = 0
         self.image.layout = pgm.IMAGE_SCALED
-        self.image.alignment = pgm.IMAGE_BOTTOM
+        self.image.alignment = pgm.IMAGE_BOTTOM_RIGHT
         self.image.x, self.image.y = (0.0, 0.0)
         self.image.width, self.image.height = (1.0, 0.5)
         self.image.visible = True
@@ -86,6 +86,25 @@ class PreviewWidget(Widget):
         self.reflection.opacity = 80
         self.reflection.visible = True
 
+        source_height = 0.04
+
+        self.source_icon = Image()
+        self.add(self.source_icon)
+        self.source_icon.alignment = pgm.IMAGE_BOTTOM_RIGHT
+        self.source_icon.size = (0.2, source_height)
+        self.source_icon.position = (1.0 - self.source_icon.width, 0.55, 0.0)
+        self.source_icon.bg_color = (0, 0, 0, 0)
+        self.source_icon.visible = True
+
+        self.source_label = Text()
+        self.add(self.source_label)
+        self.source_label.size = (0.77, source_height)
+        self.source_label.position = (0.0, 0.55, 0.0)
+        self.source_label.fg_color = (81, 81, 81, 255)
+        self.source_label.bg_color = (0, 0, 0, 0)
+        self.source_label.alignment = pgm.TEXT_ALIGN_RIGHT
+        self.source_label.visible = True
+
 
 class PreviewListController(ListController):
 
@@ -93,6 +112,10 @@ class PreviewListController(ListController):
     fastscroller = False
     fastscroller_threshold = 20
     shortcuts = list('#abcdefghijklmnopqrstuvwxyz')
+    
+    def __init__(self):
+        super(PreviewListController, self).__init__()
+        self._active_shortcuts_cache = None
 
     def _create_selector(self):
         selector = SlicedImageHorizontal()
@@ -147,15 +170,28 @@ class PreviewListController(ListController):
             return '#'
 
     def _get_active_shortcuts(self):
-        # FIXME: This call is very costly for a large list, the list of active
-        # shortcuts should be cached somehow.
-        def remove_duplicates(alist):
-            # remove duplicates from a list preserving the order
-            set = {}
-            return [set.setdefault(e, e) for e in alist if e not in set]
- 
-        letters = [self.item_to_letter(item) for item in self.model]
-        return remove_duplicates(letters)
+        try:
+            current_cookie = self.model.cookie
+        except AttributeError:
+            current_cookie = None
+
+        if current_cookie is not None and \
+                self._active_shortcuts_cache is not None and \
+                self._active_shortcuts_cache[0] == current_cookie:
+            cache = self._active_shortcuts_cache[1]
+        else:
+            def remove_duplicates(alist):
+                # remove duplicates from a list preserving the order
+                set = {}
+                return [set.setdefault(e, e) for e in alist if e not in set]
+
+            letters = [self.item_to_letter(item) for item in self.model]
+            cache = remove_duplicates(letters)
+
+            if current_cookie is not None:
+                self._active_shortcuts_cache = (current_cookie, cache)
+
+        return cache
 
     def _shortcut_renderer(self, shortcut, widget):
         if shortcut in self._get_active_shortcuts():
@@ -221,6 +257,20 @@ class PreviewListController(ListController):
             selected_item = self.model[self.nodes.selected_item_index]
             self._load_preview(selected_item)
 
+            source_icon = getattr(selected_item, 'source_icon', None)
+            if source_icon:
+                self.frontend.load_from_theme(source_icon, self.preview.source_icon)
+                self.preview.source_icon.visible = True
+            else:
+                self.preview.source_icon.visible = False
+
+            source_label = getattr(selected_item, 'source_label', None)
+            if source_icon:
+                self.preview.source_label.label = source_label
+                self.preview.source_label.visible = True
+            else:
+                self.preview.source_label.visible = False
+
         self.animated_preview.setup_next_animations(duration=200,
                                             transformation=implicit.DECELERATE,
                                             end_callback=load_new_preview)
