@@ -16,12 +16,22 @@
 
 from elisa.plugins.pigment.pigment_controller import PigmentController
 from elisa.plugins.poblesec.history import History
-from elisa.plugins.poblesec.transitions import CancelledTransition
 
 from elisa.core.input_event import *
+from elisa.core.utils import defer
+
+from elisa.core.utils import defer
 
 from elisa.plugins.poblesec.transitions import SoftFadeFrontIn, SoftFadeFrontOut
 from elisa.plugins.poblesec.transitions import SoftFadeBackIn, SoftFadeBackOut
+from elisa.plugins.pigment.graph import group
+
+try:
+    import dbus
+    from elisa.plugins.database.dbus_browser import DBusBrowser
+except ImportError:
+    dbus = None
+
 
 class BrowserController(PigmentController):
 
@@ -46,6 +56,8 @@ class BrowserController(PigmentController):
         
         self.widget.connect('focus', self._focus_cb)
 
+        self._initialize_dbus()
+
         return dfr
 
     def _focus_cb(self, widget, focus):
@@ -64,13 +76,31 @@ class BrowserController(PigmentController):
         # FIXME: PigmentController.set_frontend API does not return a deferred
         return dfr
 
-    def _push_controller_cb(self, history, previous, current):
+    def _push_controller_cb(self, history, previous, current_dfr):
+        current_dfr.addCallback(self._swap_new_controller, previous)
+        current_dfr.addErrback(self._new_controller_creation_failed, previous)
+
+    def _new_controller_creation_failed(self, failure, previous):
+        try:
+            previous.sensitive = True
+        except AttributeError:
+            pass
+
+        return failure
+
+    def _swap_new_controller(self, current, previous):
         self.debug("push new controller %s", current)
         if current != previous:
+            # set the focus and sensitive to the new one
             self._show_controller(current, self.transition_in.apply)
             current.widget.focus = True
+            current.sensitive = True
+
             if previous != None:
+                # and remove the old one, if still there
+                previous.sensitive = False
                 self._hide_controller(previous, self.transition_out.apply)
+        return current
 
     def _pop_controller_cb(self, history, previous, current):
         self.debug("pop controller %s", previous)
@@ -81,17 +111,18 @@ class BrowserController(PigmentController):
 
     def _hide_controller(self, controller, transition):
         def remove_controller(controller):
-            self.widget.remove(controller.widget)
+            # the widget may have been already removed by
+            # controller.widget.clean()
+            if controller.widget in self.widget.get_children():
+                self.widget.remove(controller.widget)
             controller.removed()
             del self._hiding_controllers[controller]
 
         def on_error(failure):
             del self._hiding_controllers[controller]
-            if failure.type is CancelledTransition:
-                pass
-            else:
-                return failure
+            failure.trap(defer.CancelledError)
 
+        controller.sensitive = False
         dfr = transition(controller)
         self._hiding_controllers[controller] = dfr
         dfr.addCallback(remove_controller)
@@ -114,12 +145,13 @@ class BrowserController(PigmentController):
             controller.widget.regenerate()
 
         def on_error(failure):
-            if failure.type is CancelledTransition:
-                pass
-            else:
-                return failure
+            failure.trap(defer.CancelledError)
+
+        def set_sensitive(controller):
+            controller.sensitive = True
 
         self._showing_controller_dfr = transition(controller)
+        self._showing_controller_dfr.addCallback(set_sensitive)
         self._showing_controller_dfr.addErrback(on_error)
 
         return self._showing_controller_dfr
@@ -140,3 +172,44 @@ class BrowserController(PigmentController):
 
         return self.history.current.handle_input(manager, input_event)
 
+    def _initialize_dbus(self):
+        if dbus is None:
+            # no dbus support
+            return
+
+        bus = dbus.SessionBus()
+
+        self.dbus_browser = DBusBrowser(self, bus,
+                '/com/fluendo/Elisa/Plugins/Poblesec/Browser')
+
+    def _clean_dbus(self):
+        if dbus is None:
+            # no dbus support
+            return
+
+        bus = dbus.SessionBus()
+        self.dbus_browser.remove_from_connection(bus,
+                '/com/fluendo/Elisa/Plugins/Poblesec/Browser')
+
+        # remove the reference cycle
+        self.dbus_browser = None
+
+    def clean(self):
+        self._clean_dbus()
+
+        def parent_clean(result):
+            return super(BrowserController, self).clean()
+
+        deferreds = list(self._hiding_controllers.values())
+
+        if self._showing_controller_dfr is not None and \
+                not self._showing_controller_dfr.called:
+            deferreds.append(self._showing_controller_dfr)
+
+        if len(deferreds) == 0:
+            return parent_clean(None)
+
+        dfr = defer.DeferredList(deferreds)
+        dfr.addCallback(parent_clean)
+        return dfr
+
