@@ -56,13 +56,13 @@ _ = install_translation('poblesec')
 
 """
 TODO
-  * images slideshow
-  * actually activate item ordering
-  ✓ on the fly updating of new/removed shares
-  ✓ playing of audio and video
-  ✓ media type filtering / item icons
-  ✓ item ordering
-  ✓ check whether the share is actually accessible
+ - images slideshow
+ - actually activate item ordering
+ - on the fly updating of new/removed shares
+ - playing of audio and video
+ - media type filtering / item icons
+ - item ordering
+ - check whether the share is actually accessible
 """
 
 
@@ -106,14 +106,22 @@ class NetworkSharesList(SectionMenuController):
 
     def _add_share(self, share):
         link = Link()
-        if share.elisa_uri and share.elisa_uri.scheme == 'daap':
-            link.controller_path = '/poblesec/music/daap'
-            link.controller_args = {'uri': share.elisa_uri}
-        else:
-            link.controller_path = "/poblesec/network"
-            link.controller_args = {'uri': share.elisa_uri,
-                                    'media_type': self.media_type}
+        link.controller_path = '/poblesec/network'
+        link.controller_args = {'uri': share.elisa_uri,
+                                'media_type': self.media_type}
         link.label = share.name
+        
+        # some protocols should be treated differently
+        scheme = share.elisa_uri.scheme
+        if scheme == 'daap':
+            # we need to use a different controller for daap
+            link.controller_path = '/poblesec/music/daap'
+            # it does not have the "media_type" keyword
+            del link.controller_args["media_type"]
+        elif scheme == 'smb':
+            # SMB shares look best capitalized
+            link.label = share.name.capitalize()
+            
         if self.media_type:
             link.icon = "elisa.plugins.poblesec.%s_folders" % (self.media_type)
         else:
@@ -127,8 +135,6 @@ class NetworkSharesList(SectionMenuController):
 #                break
 #        self.model.insert(i, link)
 #        self.shares.insert(i, share)
-        self.model.append(link)
-        self.shares.append(share)
 
         resource_manager = application.resource_manager
         try:
@@ -136,10 +142,10 @@ class NetworkSharesList(SectionMenuController):
             resource_provider = resource_manager._get_resource_provider(share.elisa_uri)
         except NoMatchingResourceProvider, err:
             self.warning(err)
-            index = self.model.index(link)
-            link = copy.copy(link)
-            link.icon = "elisa.plugins.poblesec.folder_unaccesible"
-            self.model[index] = link
+        else:
+            # show only the shares we have a resource provider for
+            self.model.append(link)
+            self.shares.append(share)
 
     def _share_added_cb(self, msg, sender):
         self.debug("New share appeared: %s" % msg.model.name)
@@ -172,6 +178,37 @@ class NetworkShareBrowser(HierarchyController):
                    'video': 'video',
                    'pictures': 'image'}
 
+    @property
+    def empty_label(self):
+        media_type = self.media_type
+        # FIXME: please clean the media_type values mess.
+        if media_type == 'video':
+            msg = _("This shared folder does not contain any video files.")
+        elif media_type in ('music', 'audio'):
+            msg = _("This shared folder does not contain any audio files.")
+        elif media_type in ('pictures', 'image'):
+            msg = _("This shared folder does not contain any image files.")
+        else:
+            msg = _("This shared folder does not contain any multimedia files")
+
+        return msg
+
+    @property
+    def empty_icon(self):
+        media_type = self.media_type
+        # FIXME: please clean the media_type values mess.
+        if media_type == 'video':
+            resource = 'elisa.plugins.poblesec.file_video'
+        elif media_type in ('music', 'audio'):
+            resource = 'elisa.plugins.poblesec.file_music'
+        elif media_type in ('pictures', 'image'):
+            resource = 'elisa.plugins.poblesec.file_picture'
+        else:
+            # FIXME: better generic icon
+            resource = 'elisa.plugins.poblesec.file'
+
+        return resource
+
     def initialize(self, uri=None, media_type=None):
         dfr = super(NetworkShareBrowser, self).initialize()
 
@@ -234,54 +271,30 @@ class NetworkShareBrowser(HierarchyController):
 
         return dfr
 
-    def nodes_setup(self):
-        super(NetworkShareBrowser, self).nodes_setup()
-        media_type = self.media_type
-        # FIXME: please clean the media_type values mess.
-        if media_type == 'video':
-            msg = _("This shared folder does not contain any video files.")
-        elif media_type in ('music', 'audio'):
-            msg = _("This shared folder does not contain any audio files.")
-        elif media_type in ('pictures', 'image'):
-            msg = _("This shared folder does not contain any image files.")
-        else:
-            msg = _("This shared folder does not contain any multimedia files")
-        self.nothing_to_display_widget.text.label = msg
-        if not len(self.model):
-            self.nothing_to_display_widget.visible = True
-
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
+        if isinstance(item, (NetworkServiceModel, UpnpContainerModel)):
+            controllers = self.frontend.retrieve_controllers('/poblesec/browser')
+            browser = controllers[0]
 
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(item, (NetworkServiceModel, UpnpContainerModel)):
-                controllers = self.frontend.retrieve_controllers('/poblesec/browser')
-                browser = controllers[0]
-
-                if item.elisa_uri.scheme == 'file':
-                    path = '/poblesec/%s/filesystem' % self.media_type
-                    file_media_type = self.translation[self.media_type]
-                else:
-                    path = self.path
-                    file_media_type = self.media_type
-                dfr = browser.history.append_controller(path,
-                                                  item.name,
-                                                  uri=item.elisa_uri,
-                                                  media_type=file_media_type)
-            elif isinstance(item, ImageModel):
-                self.play_image(item)
-                dfr = defer.succeed(True)
-            elif isinstance(item, VideoModel):
-                self.play_audio_video(item, 'video')
-                dfr = defer.succeed(True)
-            elif isinstance(item, TrackModel):
-                self.play_audio_video(item, 'music')
-                dfr = defer.succeed(True)
+            if item.elisa_uri.scheme == 'file':
+                path = '/poblesec/%s/filesystem' % self.media_type
+                file_media_type = self.translation[self.media_type]
+            else:
+                path = self.path
+                file_media_type = self.media_type
+            dfr = browser.history.append_controller(path,
+                                              item.name,
+                                              uri=item.elisa_uri,
+                                              media_type=file_media_type)
+        elif isinstance(item, ImageModel):
+            self.play_image(item)
+            dfr = defer.succeed(True)
+        elif isinstance(item, VideoModel):
+            self.play_audio_video(item, 'video')
+            dfr = defer.succeed(True)
+        elif isinstance(item, TrackModel):
+            self.play_audio_video(item, 'music')
+            dfr = defer.succeed(True)
 
     def play_image(self, item):
         controllers = self.frontend.retrieve_controllers('/poblesec/slideshow_player')
