@@ -173,7 +173,8 @@ class TopBar(Widget):
         for key, value in props.iteritems():
             if key == 'top-bar-buttons-x':
                 self.button_bar.x = value
-                self.button_bar.width = 1.0 - value
+            elif key == 'top-bar-buttons-width':
+                self.button_bar.width = value
             elif key == 'top-bar-image-x':
                 self.image.x = value
             elif key == 'top-bar-image-height':
@@ -289,6 +290,12 @@ class Crumb(Widget):
                 self.text.y = (1.0-self.text.height)/2.0
 
 class BreadCrumbs(Widget):
+    """
+    DOCME
+
+    @ivar logo: image visually located at the right end of the bread crumbs
+    @type logo: L{elisa.plugins.pigment.graph.image.Image}
+    """
 
     __gsignals__ = {'crumb-clicked': (gobject.SIGNAL_RUN_LAST,
                                       gobject.TYPE_BOOLEAN,
@@ -305,6 +312,15 @@ class BreadCrumbs(Widget):
         self.background.layout = pgm.IMAGE_FILLED
         self.background.visible = True
 
+        # placeholder for logo, optionally used by plugins
+        self.logo = Image()
+        self.add(self.logo)
+        self.logo.size = (0.1, 1.0)
+        self.logo.position = (1.0-self.logo.width, 0.0, 0.0)
+        self.logo.bg_color = (0, 0, 0, 0)
+        self.logo.opacity = 200
+        self.logo.visible = True
+
         self.master = Image()
         self.add(self.master)
         self.master.visible = False
@@ -426,7 +442,7 @@ class TopBarWithBreadCrumbs(Widget):
         self.bread_crumb.z = 0
         self.bread_crumb.visible = True
         self.bread_crumb.opacity = 0
-        
+
         #create animation stuff
         self.animated = implicit.AnimatedObject(self)
         settings = {'duration': 500,
@@ -546,9 +562,29 @@ class PoblesecBrowserController(BrowserController):
         if self.history:
             while self.history.index > 1:
                 self.history.go_back()
-     
-    def _push_controller_signal(self, history, previous, current):
+
+    def set_section_logo(self, logo_resource):
+        """
+        Load L{logo_resource} into the bread crumbs bar. That logo will be
+        visually located at the right end of the bar.
+        If logo_resource is None, unload previously loaded resource.
+
+        @param logo_resource: resource pointing to the logo
+        @type logo_resource:  str
+        """
+        image = self.topbar_with_bread_crumb.bread_crumb.logo
+        if logo_resource != None:
+            self.frontend.load_from_theme(logo_resource, image)
+        else:
+            image.clear()
+
+    def _push_controller_signal(self, history, previous, current_dfr):
+        current_dfr.addCallback(self._new_controller_pushed, previous)
+
+    def _new_controller_pushed(self, current, previous):
         if self.history.index > 0:
+            self._update_crumbs_logo(previous, current)
+
             if self.history.index == 1:
                 topbar = self.topbar_with_bread_crumb.topbar
                 topbar.text.label = current.display_name
@@ -566,9 +602,19 @@ class PoblesecBrowserController(BrowserController):
 
             self.bottombar.animated.y = 1 - self.bottombar.height
             self.bottombar.animated.opacity = 255
-                
+ 
         self.topbar_with_bread_crumb.topbar.button_bar.set_index(-1)
-            
+        return current
+
+    def _update_crumbs_logo(self, previous_controller, current_controller):
+        # if the new controller defines a logo attribute it is loaded replacing
+        # the currently displayed one
+        current_logo = getattr(current_controller, "crumbs_logo", None)
+        previous_logo = getattr(previous_controller, "crumbs_logo", None)
+
+        if current_logo != previous_logo:
+            self.set_section_logo(current_logo)
+
     def _pop_controller_signal(self, history, previous, current):
         if self.history.index < 1:
             delta = self.topbar_with_bread_crumb.topbar.height + \
@@ -583,10 +629,11 @@ class PoblesecBrowserController(BrowserController):
         #this last crumb will be removed after hide_bread_crumb() animation
         if self.history.index > 1:
             self.topbar_with_bread_crumb.bread_crumb.pop_crumb()
-            
+            self._update_crumbs_logo(previous, current)
+
         if self.history.index <= 1:
             self.topbar_with_bread_crumb.hide_bread_crumb()
-            
+
         self.topbar_with_bread_crumb.topbar.button_bar.set_index(-1)
 
     def fire_button_bar(self, index):
