@@ -59,11 +59,7 @@ class SectionsMenuController(PigmentController):
     def set_frontend(self, frontend):
         super(SectionsMenuController, self).set_frontend(frontend)
 
-        # reference to the section (a elisa.plugins.poblesec.link.Link instance)
-        # that has been clicked and for which the corresponding controller is
-        # being created
-        self._previously_clicked_section = None
-
+        self.sensitive = False
         # menu widget setup
         self.menu = SectionsMenuWidget(SectionItemWidget)
         self.widget.add(self.menu)
@@ -84,16 +80,13 @@ class SectionsMenuController(PigmentController):
         self.menu.set_renderer(renderer)
         self.menu.connect('item-clicked', self._section_clicked_handler)
 
-    def _reset_previously_clicked_section(self):
-        self._previously_clicked_section = None
-
     def _section_clicked_handler(self, menu, section):
-        # only try creating the controller behind the section that was clicked
-        # if the user has not already clicked on it and if it was the one
-        # selected
+        # only react if we are sensitive
+        if not self.sensitive:
+            return
+
         selected_section = self.model[self.menu.selected_item_index]
-        if section != self._previously_clicked_section and \
-            section == selected_section:
+        if section == selected_section:
             path = '/poblesec/browser'
             controllers = self.frontend.retrieve_controllers(path)
             # we assume there was only one controller created with that path
@@ -101,14 +94,6 @@ class SectionsMenuController(PigmentController):
             path = section.controller_path
             label = section.label
             dfr = browser.history.append_controller(path, label)
-            self._previously_clicked_section = section
-
-            # make sure that user will still be able to try entering any
-            # section in the future regardless of what might have happened
-            dfr.addErrback(lambda x: self._reset_previously_clicked_section())
-
-    def removed(self):
-        self._reset_previously_clicked_section()
 
     def handle_input(self, manager, input_event):
         if self.widget.focus == True:
