@@ -34,6 +34,15 @@ def directory_settings_decorator(controller):
     link.controller_path = "/poblesec/settings/directory_settings"
     link.label = _("Add folders")
     link.icon = "elisa.plugins.poblesec.directory_settings_add_folder"
+
+    return defer.succeed(None)
+
+def login_settings_decorator(controller):
+    link = Link()
+    link.controller_path = "/poblesec/settings/login"
+    link.label = _("User Accounts")
+    link.icon = "elisa.plugins.poblesec.key_icon"
     controller.model.append(link)
 
     return defer.succeed(None)
+
