@@ -22,9 +22,9 @@ setup(name='elisa-plugin-poblesec',
                          ('/poblesec/browser',
                           'elisa.plugins.poblesec.poblesec_browser_controller:PoblesecBrowserController'),
                          ('/poblesec/video_player',
-                          'elisa.plugins.poblesec.player_video:PlayerController'),
+                          'elisa.plugins.poblesec.player_video:VideoPlayerController'),
                          ('/poblesec/music_player',
-                          'elisa.plugins.poblesec.player_video:PlayerController'),
+                          'elisa.plugins.poblesec.player_audio:AudioPlayerController'),
                          ('/poblesec/slideshow_player',
                           'elisa.plugins.poblesec.player_slideshow:SlideshowController'),
                          ('/poblesec/sections_menu',
@@ -79,8 +79,10 @@ setup(name='elisa-plugin-poblesec',
                           'elisa.plugins.poblesec.popup:PopupController'),
                          ('/poblesec/search/.*',
                           'elisa.plugins.poblesec.search_controller:SearchController'),
-                       ],
-    decorator_mappings=[('/poblesec/sections_menu',
+                         ],
+    decorator_mappings=[('/poblesec',
+                         'elisa.plugins.poblesec.viewport_buttons:buttons_decorator'),
+                        ('/poblesec/sections_menu',
                          'elisa.plugins.poblesec.moblin:moblin_decorator'),
                         ('/poblesec/sections_menu',
                          'elisa.plugins.poblesec.settings:settings_decorator'),
@@ -126,7 +128,14 @@ setup(name='elisa-plugin-poblesec',
                         ('/poblesec/video_menu',
                          'elisa.plugins.poblesec.video:internet_decorator'),
                         ('/poblesec/video_menu',
-                         'elisa.plugins.poblesec.video:filesystem_decorator'),
+                         'elisa.plugins.database.playlists_controller:video_playlists_decorator'),
+                        # FIXME: the following decorator should not be here
+                        # but in the favorites plugin ; this works around an
+                        # ordering issue in the menus.
+                        ('/poblesec/video_menu',
+                         'elisa.plugins.favorites.controller:videos_decorator'),
+                        ('/poblesec/video_menu',
+                         'elisa.plugins.database.video_controller:video_lib_decorator'),
 
                         ('/poblesec/pictures_menu',
                          'elisa.plugins.poblesec.section:hardware_decorator'),
@@ -150,8 +159,9 @@ setup(name='elisa-plugin-poblesec',
                         ('/poblesec/pictures_menu',
                          'elisa.plugins.database.photo_controller:photo_lib_decorator'),
 
-                        ('/poblesec/settings_menu',
-                         'elisa.plugins.poblesec.settings:login_settings_decorator'),
+                        # Disabled, as there are no login services
+                        #('/poblesec/settings_menu',
+                        # 'elisa.plugins.poblesec.settings:login_settings_decorator'),
                         ('/poblesec/settings_menu',
                          'elisa.plugins.poblesec.settings:directory_settings_decorator'),
 
@@ -159,9 +169,9 @@ setup(name='elisa-plugin-poblesec',
                         # but in their respective plugins ; this works around
                         # an ordering issue.
                         ('/poblesec/search/music',
-                         'elisa.plugins.yesfm.searcher:music_search_result_decorator'),
+                         'elisa.plugins.yesfm.controllers:music_search_result_decorator'),
                         ('/poblesec/search/music',
-                         'elisa.plugins.database.searcher:music_search_result_decorator'),
+                         'elisa.plugins.database.music_controller:music_search_result_decorator'),
                        ],
     cmdclass=cmdclass,
     distclass=Distribution)
