@@ -60,7 +60,7 @@ class SearchResultNode(object):
         @param icon_resource: the icon resource for this category of results
         @type icon_resource:  C{str}
         @param results:       the list of results for this category
-        @type results:        C{list} of L{elisa.plugins.base.models.*}
+        @type results:        C{list} of L{elisa.plugins.base.models}.*
         """
         self.searcher = searcher
         self.result_type = result_type
@@ -144,6 +144,19 @@ class SearchResultWidget(Widget):
             elif key == '%s-total-fg-color' % self.style_prefix:
                 self.total.fg_color = value
 
+    def clean(self):
+
+        self._background.clean()
+        self._background = None
+        self.icon.clean()
+        self.icon = None
+        self.label.clean()
+        self.label = None
+        self.total.clean()
+        self.total = None
+
+        return super(SearchResultWidget, self).clean()
+
 
 class SearchResultEntryNode(object):
 
@@ -159,7 +172,7 @@ class SearchResultEntryNode(object):
         @param searcher_entry: a searcher entry
         @type searcher_entry:  L{elisa.plugins.poblesec.search_controller.SearcherEntry}
         @param results:        the result of a search
-        @type results:         one of L{elisa.plugins.search.models.*}
+        @type results:         one of L{elisa.plugins.search.models}.*
         """
         self.searcher_entry = searcher_entry
         self.results = results
@@ -281,7 +294,19 @@ class SearchResultEntryWidget(Widget):
 
     def clean(self):
         self.disconnect(self._clicked_id)
-        super(SearchResultEntryWidget, self).clean()
+
+        self.icon.clean()
+        self.icon = None
+        self.label.clean()
+        self.label = None
+        self.loading_animation.clean()
+        self.loading_animation = None
+        self.list.clean()
+        self.list = None
+        self._animated_self.stop_animations()
+        self._animated_self = None
+
+        return super(SearchResultEntryWidget, self).clean()
 
     def update(self, searcher, result_model):
         """
@@ -293,7 +318,7 @@ class SearchResultEntryWidget(Widget):
         @param searcher:     the searcher entry point
         @type searcher:      C{str}
         @param result_model: the results of a search
-        @type result_model:  one of L{elisa.plugins.search.models.*}
+        @type result_model:  one of L{elisa.plugins.search.models}.*
         """
         raise NotImplementedError()
 
@@ -376,6 +401,24 @@ class MusicSearchResultEntryWidget(SearchResultEntryWidget):
                                   result_model.tracks)]
         self.list.set_model(model)
 
+class VideosSearchResultEntryWidget(SearchResultEntryWidget):
+
+    """
+    A widget that displays the search results for one given videos searcher.
+    """
+
+    item_widget_class = SearchResultWidget
+
+    def update(self, searcher, result_model):
+        """
+        @type result_model: L{elisa.plugins.search.models.VideosSearchResultModel}
+        """
+        resource = 'elisa.plugins.poblesec.video_folders'
+        model = [SearchResultNode(searcher, 'videos', _('Videos'),
+                                  resource,
+                                  result_model.videos),
+                 ]
+        self.list.set_model(model)
 
 class SearchResultsWidget(ListHorizontal):
 
@@ -384,9 +427,12 @@ class SearchResultsWidget(ListHorizontal):
     search.
     """
 
-    # Currently the only supported media type is 'music'.
+    preloaded = 0
+
+    # Currently the only supported media types are 'music' and 'videos'.
     # To be completed later on when we support other types of search.
-    mediatype_to_widget = {'music': MusicSearchResultEntryWidget}
+    mediatype_to_widget = {'music': MusicSearchResultEntryWidget,
+                           'videos': VideosSearchResultEntryWidget}
 
     def __init__(self, frontend, mediatype):
         """
@@ -408,7 +454,7 @@ class SearchResultsWidget(ListHorizontal):
 
     def clean(self):
         self.disconnect(self._selected_id)
-        super(SearchResultsWidget, self).clean()
+        return super(SearchResultsWidget, self).clean()
 
     def set_searchers(self, searchers):
         """
@@ -453,7 +499,7 @@ class SearchResultsWidget(ListHorizontal):
         @param searcher_entry: a searcher entry
         @type searcher_entry:  L{elisa.plugins.poblesec.search_controller.SearcherEntry}
         @param result_model:   the result of a new search
-        @type result_model:    one of L{elisa.plugins.search.models.*}
+        @type result_model:    one of L{elisa.plugins.search.models}.*
         """
         for index, item in enumerate(self.model):
             if item.searcher_entry == searcher_entry:
