@@ -33,4 +33,8 @@ setup(name='elisa-plugin-search',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    SearchMetaresourceProvider = elisa.plugins.search.search_metaresource_provider:SearchMetaresourceProvider
+    """)
