@@ -14,4 +14,8 @@ setup(name='elisa-plugin-smb_win32',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    SmbWin32Resource = elisa.plugins.smbwin32.smbwin32_resource:SmbWin32Resource
+    """)
