@@ -14,4 +14,8 @@ setup(name='elisa-plugin-winremote',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.input_provider]
+    StreamzapInput = elisa.plugins.winremote.streamzap_input:StreamzapInput
+    """)
