@@ -14,4 +14,8 @@ setup(name='elisa-plugin-wmd',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.resource_provider]
+    WMDResource = elisa.plugins.wmd.wmd_resource:WMDResource
+    """)
